@@ -1,7 +1,7 @@
 // gui.cpp:  Top level GUI for SWF player, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -22,29 +22,28 @@
 #include "gnashconfig.h"
 #endif
 
-#include "MovieClip.h"
 #include "gui.h"
+
+#include <vector>
+#include <algorithm> 
+#include <boost/assign/list_of.hpp>
+
+#include "MovieClip.h"
 #include "Renderer.h"
 #include "sound_handler.h"
 #include "movie_root.h"
 #include "VM.h"
 #include "DisplayObject.h"
-#include "tu_file.h"
-#include "gnash.h"
+#include "GnashEnums.h"
 #include "RunResources.h"
+#include "StreamProvider.h"
+#include "ScreenShotter.h"
 
 #ifdef GNASH_FPS_DEBUG
 #include "ClockTime.h"
 #include <boost/format.hpp>
 #endif
 
-#include <vector>
-#include <cstdio>
-#include <cstring>
-#include <algorithm> 
-#include <boost/algorithm/string/replace.hpp>
-#include <boost/lexical_cast.hpp>
-
 /// Define this to make sure each frame is fully rendered from ground up
 /// even if no motion has been detected in the movie.
 //#define FORCE_REDRAW 1
@@ -80,6 +79,20 @@
 
 namespace gnash {
 
+struct Gui::Display
+{
+    Display(Gui& g, movie_root& r) : _g(g), _r(r) {}
+    void operator()() const {
+		InvalidatedRanges world_ranges;
+		world_ranges.setWorld();
+		_g.setInvalidatedRegions(world_ranges);
+        _g.display(&_r);
+    }
+private:
+    Gui& _g;
+    movie_root& _r;
+};
+
 Gui::Gui(RunResources& r) :
     _loop(true),
     _xid(0),
@@ -177,6 +190,12 @@ Gui::~Gui()
     }
 #endif
 }
+    
+void
+Gui::setClipboard(const std::string&)
+{
+    LOG_ONCE(log_unimpl(_("Clipboard not yet supported in this GUI")));
+}
 
 void
 Gui::setFullscreen()
@@ -199,11 +218,10 @@ Gui::unsetFullscreen()
 void
 Gui::quit()
 {
-    log_debug(__PRETTY_FUNCTION__);
-
     // Take a screenshot of the last frame if required.
-    if (_screenShotter.get()) {
-        _screenShotter->last();
+    if (_screenShotter.get() && _renderer.get()) {
+        Display dis(*this, *_stage);
+        _screenShotter->last(*_renderer, &dis);
     }
     
     quitUI();
@@ -263,7 +281,7 @@ Gui::restart()
 void
 Gui::updateStageMatrix()
 {
-    if ( ! VM::isInitialized() ) {
+    if (!_stage) {
         // When VM initializes, we'll get a call to resize_view, which
         // would call us again.
         //log_debug("Can't update stage matrix till VM is initialized");
@@ -421,14 +439,9 @@ Gui::resize_view(int width, int height)
 	assert(width>0);
 	assert(height>0);
 
-	if ( VM::isInitialized() )
-	{
-
-		if (_stage && _started) {
-			_stage->setDimensions(width, height);
-		}
-
-	}
+    if (_stage && _started) {
+        _stage->setDimensions(width, height);
+    }
 
 	_width = width;
 	_height = height;
@@ -795,35 +808,30 @@ Gui::display(movie_root* m)
   
 		// show invalidated region using a red rectangle
 		// (Flash debug style)
-		IF_DEBUG_REGION_UPDATES (
-		if (_renderer.get() && !changed_ranges.isWorld())
-		{
-		
-			for (size_t rno = 0; rno < changed_ranges.size(); rno++) {
-			
-				const geometry::Range2d<int>& bounds = 
-					changed_ranges.getRange(rno);
+		IF_DEBUG_REGION_UPDATES(
+            if (_renderer.get() && !changed_ranges.isWorld()) {
+            
+                for (size_t rno = 0; rno < changed_ranges.size(); rno++) {
+                
+                    const geometry::Range2d<int>& bounds = 
+                        changed_ranges.getRange(rno);
 
-				point corners[4];
-				float xmin = bounds.getMinX();
-				float xmax = bounds.getMaxX();
-				float ymin = bounds.getMinY();
-				float ymax = bounds.getMaxY();
-				
-				corners[0].x = xmin;
-				corners[0].y = ymin;
-				corners[1].x = xmax;
-				corners[1].y = ymin;
-				corners[2].x = xmax;
-				corners[2].y = ymax;
-				corners[3].x = xmin;
-				corners[3].y = ymax;
-				SWFMatrix no_transform;
-				_renderer->draw_poly(corners, 4,
-					rgba(0,0,0,0), rgba(255,0,0,255), no_transform, false);
-					
-			}
-		}
+                    float xmin = bounds.getMinX();
+                    float xmax = bounds.getMaxX();
+                    float ymin = bounds.getMinY();
+                    float ymax = bounds.getMaxY();
+
+                    const std::vector<point> box = boost::assign::list_of
+                        (point(xmin, ymin))
+                        (point(xmax, ymin))
+                        (point(xmax, ymax))
+                        (point(xmin, ymax));
+
+                    _renderer->draw_poly(box, rgba(0,0,0,0), rgba(255,0,0,255),
+                            SWFMatrix(), false);
+                        
+                }
+            }
 		);
 
 		// show frame on screen
@@ -911,21 +919,26 @@ Gui::start()
         return;
     }
 
-    // Initializes the stage with a Movie and the passed flash vars and
-    // Scriptable vars for ExternalInterface.
-    _stage->init(_movieDef.get(), _flashVars, _scriptableVars);
+    // Initializes the stage with a Movie and the passed flash vars.
+    _stage->init(_movieDef.get(), _flashVars);
 
     bool background = true; // ??
     _stage->set_background_alpha(background ? 1.0f : 0.05f);
 
+    // to properly update stageMatrix if scaling is given  
+    resize_view(_width, _height); 
+
     // @todo since we registered the sound handler, shouldn't we know
     //       already what it is ?!
     sound::sound_handler* s = _stage->runResources().soundHandler();
-    if ( s ) s->unpause();
+    if ( s ) {
+        if ( ! _audioDump.empty() ) {
+            s->setAudioDump(_audioDump);
+        }
+        s->unpause();
+    }
     _started = true;
     
-    // to properly update stageMatrix if scaling is given  
-    resize_view(_width, _height); 
 
     log_debug("Starting virtual clock");
     _virtualClock.resume();
@@ -933,17 +946,19 @@ Gui::start()
 }
 
 bool
-Gui::advanceMovie()
+Gui::advanceMovie(bool doDisplay)
 {
 
+
     if (isStopped()) {
-        return true;
+        return false;
     }
 
     if (!_started) {
         start();
     }
 
+    Display dis(*this, *_stage);
     gnash::movie_root* m = _stage;
     
     // Define REVIEW_ALL_FRAMES to have *all* frames
@@ -952,30 +967,25 @@ Gui::advanceMovie()
     
 #ifndef REVIEW_ALL_FRAMES
     // Advance movie by one frame
-    bool advanced = m->advance();
+    const bool advanced = m->advance();
 #else
     const size_t cur_frame = m->getRootMovie()->get_current_frame();
     const size_t tot_frames = m->getRootMovie()->get_frame_count();
     const bool advanced = m->advance();
     
-    m->getRootMovie.ensureFrameLoaded(tot_frames);
+    m->getRootMovie().ensureFrameLoaded(tot_frames);
     m->goto_frame(cur_frame + 1);
-    m->set_play_state(gnash::MovieClip::PLAYSTATE_PLAY);
+    m->getRootMovie().setPlayState(gnash::MovieClip::PLAYSTATE_PLAY);
     log_debug(_("Frame %d"), m->get_current_frame());
 #endif
     
 #ifdef GNASH_FPS_DEBUG
     // will be a no-op if fps_timer_interval is zero
     if (advanced) {
-	fpsCounterTick();
+        fpsCounterTick();
     }
 #endif
     
-    
-    // TODO: ask stage about doDisplay ?
-    // - if it didn't advance might need to check updateAfterEvent
-    bool doDisplay = true;
-    
 #ifdef SKIP_RENDERING_IF_LATE
     // We want to skip rendering IFF it's time to advance again.
     // We'll ask the stage about it
@@ -993,7 +1003,7 @@ Gui::advanceMovie()
     }
 #endif // ndef SKIP_RENDERING_IF_LATE
 
-	if (doDisplay) display(m);
+	if (doDisplay && visible()) display(m);
 
 	if (!loops()) {
 		size_t curframe = m->get_current_frame(); // can be 0 on malformed SWF
@@ -1003,8 +1013,8 @@ Gui::advanceMovie()
 		}
 	}
 
-    if (_screenShotter.get()) {
-        _screenShotter->screenShot(_advances);
+    if (_screenShotter.get() && _renderer.get()) {
+        _screenShotter->screenShot(*_renderer, _advances, doDisplay ? 0 : &dis);
     }
 
     // Only increment advances and check for exit condition when we've
@@ -1017,7 +1027,13 @@ Gui::advanceMovie()
         ++_advances;
     }
 
-	return true;
+	return advanced;
+}
+
+void
+Gui::setScreenShotter(std::auto_ptr<ScreenShotter> ss)
+{
+    _screenShotter.reset(ss.release());
 }
 
 void
@@ -1026,33 +1042,17 @@ Gui::takeScreenShot()
     if (!_screenShotter.get()) {
         // If no ScreenShotter exists, none was requested at startup.
         // We use a default filename pattern.
-        URL url(_runResources.baseURL());
+        URL url(_runResources.streamProvider().baseURL());
         std::string::size_type p = url.path().rfind('/');
         const std::string& name = (p == std::string::npos) ? url.path() :
             url.path().substr(p + 1);
         const std::string& filename = "screenshot-" + name + "-%f";
-        _screenShotter.reset(new ScreenShotter(_renderer, filename));
+        _screenShotter.reset(new ScreenShotter(filename, GNASH_FILETYPE_PNG));
     }
     assert (_screenShotter.get());
     _screenShotter->now();
 }
 
-void
-Gui::requestScreenShots(const ScreenShotter::FrameList& l, bool last,
-        const std::string& filename)
-{
-    // Nothing to do if there is no renderer or if no frames should be
-    // saved.
-    if (!_renderer.get() || (l.empty() && !last)) {
-	return;
-    }
-
-    _screenShotter.reset(new ScreenShotter(_renderer, filename));
-    if (last) _screenShotter->lastFrame();
-    _screenShotter->setFrames(l);
-
-}
-
 void
 Gui::setCursor(gnash_cursor_type /*newcursor*/)
 {
@@ -1092,24 +1092,24 @@ Gui::setInvalidatedRegions(const InvalidatedRanges& ranges)
 
 #ifdef USE_SWFTREE
 
-std::auto_ptr<Gui::InfoTree>
+std::auto_ptr<movie_root::InfoTree>
 Gui::getMovieInfo() const
 {
-    std::auto_ptr<InfoTree> tr;
+    std::auto_ptr<movie_root::InfoTree> tr;
 
-    if (! VM::isInitialized()) {
+    if (!_stage) {
         return tr;
     }
 
-    tr.reset(new InfoTree());
+    tr.reset(new movie_root::InfoTree());
 
     // Top nodes for the tree:
     // 1. VM information
     // 2. "Stage" information
     // 3. ...
 
-    InfoTree::iterator topIter = tr->begin();
-    InfoTree::iterator firstLevelIter;
+    movie_root::InfoTree::iterator topIter = tr->begin();
+    movie_root::InfoTree::iterator firstLevelIter;
 
     VM& vm = _stage->getVM();
 
@@ -1119,14 +1119,14 @@ Gui::getMovieInfo() const
     /// VM top level
     //
     os << "SWF " << vm.getSWFVersion();
-    topIter = tr->insert(topIter, StringPair("VM version", os.str()));
+    topIter = tr->insert(topIter, std::make_pair("VM version", os.str()));
 
     // This short-cut is to avoid a bug in movie_root's getMovieInfo,
     // which relies on the availability of a _rootMovie for doing
     // it's work, while we don't set it if we didn't start..
     // 
     if (! _started) {
-        topIter = tr->insert(topIter, StringPair("Stage properties", 
+        topIter = tr->insert(topIter, std::make_pair("Stage properties", 
                     "not constructed yet"));
         return tr;
     }
@@ -1137,7 +1137,7 @@ Gui::getMovieInfo() const
     //
     /// Mouse entities
     //
-    topIter = tr->insert(topIter, StringPair("Mouse Entities", ""));
+    topIter = tr->insert(topIter, std::make_pair("Mouse Entities", ""));
 
     const DisplayObject* ch;
     ch = stage.getActiveEntityUnderPointer();
@@ -1147,7 +1147,8 @@ Gui::getMovieInfo() const
            << " - depth:" << ch->get_depth()
            << " - useHandCursor:" << ch->allowHandCursor()
            << ")";
-    	firstLevelIter = tr->append_child(topIter, StringPair("Active entity under mouse pointer", ss.str()));
+    	firstLevelIter = tr->append_child(topIter, 
+                std::make_pair("Active entity under mouse pointer", ss.str()));
     }
 
     ch = stage.getEntityUnderPointer();
@@ -1156,7 +1157,8 @@ Gui::getMovieInfo() const
         ss << ch->getTarget() << " (" + typeName(*ch) 
            << " - depth:" << ch->get_depth()
            << ")";
-	firstLevelIter = tr->append_child(topIter, StringPair("Topmost entity under mouse pointer", ss.str()));
+        firstLevelIter = tr->append_child(topIter, 
+            std::make_pair("Topmost entity under mouse pointer", ss.str()));
     }
     
     ch = stage.getDraggingCharacter();
@@ -1164,15 +1166,16 @@ Gui::getMovieInfo() const
         std::stringstream ss;
         ss << ch->getTarget() << " (" + typeName(*ch) 
            << " - depth:" << ch->get_depth() << ")";
-    	firstLevelIter = tr->append_child(topIter, StringPair("Dragging character: ", ss.str()));
+    	firstLevelIter = tr->append_child(topIter,
+                std::make_pair("Dragging character: ", ss.str()));
     }
 
     //
     /// GC row
     //
-    topIter = tr->insert(topIter, StringPair("GC Statistics", ""));
+    topIter = tr->insert(topIter, std::make_pair("GC Statistics", ""));
     GC::CollectablesCount cc;
-    GC::get().countCollectables(cc);
+    _stage->gc().countCollectables(cc);
     
     const std::string lbl = "GC managed ";
     for (GC::CollectablesCount::iterator i=cc.begin(), e=cc.end(); i!=e; ++i) {
@@ -1180,7 +1183,7 @@ Gui::getMovieInfo() const
         std::ostringstream ss;
         ss << i->second;
         firstLevelIter = tr->append_child(topIter,
-                            StringPair(lbl + typ, ss.str()));
+                    std::make_pair(lbl + typ, ss.str()));
     }
 
     tr->sort(firstLevelIter.begin(), firstLevelIter.end());
@@ -1272,14 +1275,6 @@ Gui::addFlashVars(Gui::VariableMap& from)
     }
 }
 
-void
-Gui::addScriptableVar(const std::string &name, const std::string &value)
-{
-    log_debug("Adding scriptable variable \"%s\" = %s",
-              name, value);
-    _scriptableVars[name] = value;
-}
-
 void
 Gui::setMovieDefinition(movie_definition* md)
 {
@@ -1307,8 +1302,8 @@ void
 Gui::setQuality(Quality q)
 {
     if (!_stage) {
-	log_error("Gui::setQuality called before a movie_root was available");
-	return;
+        log_error("Gui::setQuality called before a movie_root was available");
+        return;
     }
     _stage->setQuality(q);
 }
@@ -1349,50 +1344,6 @@ Gui::callCallback(int fd)
 
     f();
 }
-
-void
-ScreenShotter::saveImage(const std::string& id) const
-{
-    // Replace all "%f" in the filename with the frameAdvance.
-    std::string outfile(_fileName);
-    boost::replace_all(outfile, "%f", id);
-    
-    FILE* f = std::fopen(outfile.c_str(), "wb");
-    if (f) {
-        boost::shared_ptr<IOChannel> t(new tu_file(f, true));
-        _renderer->renderToImage(t, GNASH_FILETYPE_PNG);
-    } else {
-        log_error("Failed to open screenshot file \"%s\"!", outfile);
-    }
-}
-
-void
-ScreenShotter::screenShot(size_t frameAdvance)
-{
-    // Save an image if an spontaneous screenshot was requested or the
-    // frame is in the list of requested frames.
-    if (_immediate || std::binary_search(_frames.begin(), _frames.end(),
-                frameAdvance)) {
-        saveImage(boost::lexical_cast<std::string>(frameAdvance));
-        _immediate = false;
-    }
-}
-
-void
-ScreenShotter::last() const
-{
-    if (_last) {
-	saveImage("last");
-    }
-}
-
-void
-ScreenShotter::setFrames(const FrameList& frames)
-{
-    _frames = frames;
-    std::sort(_frames.begin(), _frames.end());
-}
-
 // end of namespace
 }
 
