@@ -1,6 +1,7 @@
 // Time.h: clock and local time functions for Gnash
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -22,10 +23,10 @@
 
 #include <boost/cstdint.hpp>
 #include "dsodefs.h"
-#include <sys/time.h> // for struct timezone
 
-namespace clocktime
-{
+namespace gnash {
+
+namespace clocktime {
     /// Wall clock timer, returns current POSIX time in milliseconds.
 	DSOEXPORT boost::uint64_t getTicks();
 
@@ -34,6 +35,7 @@ namespace clocktime
 	/// cannot be regarded as reliable.
 	DSOEXPORT boost::int32_t getTimeZoneOffset(double time);
 
-}
+} // namespace clocktime
+} // namespace gnash
 
 #endif
