@@ -1,7 +1,7 @@
 // GnashImage.h: Base class for reading image data in Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -28,164 +28,178 @@
 #include <boost/noncopyable.hpp>
 #include <boost/cstdint.hpp>
 #include <boost/scoped_array.hpp>
-#include <memory> // for auto_ptr
+#include <memory> 
 
-#include "FileTypes.h"
+#include "GnashEnums.h"
 #include "log.h"
 #include "dsodefs.h"
 
-
 // Forward declarations
 namespace gnash {
     class IOChannel;
-    class JpegImageInput;
 }
 
-namespace gnash
-{
+namespace gnash {
+
+/// Image handling functions and classes.
+namespace image {
 
 /// The types of images handled in Gnash.
 enum ImageType
 {
     GNASH_IMAGE_INVALID,
-    GNASH_IMAGE_RGB,
-    GNASH_IMAGE_RGBA,
-    GNASH_IMAGE_ALPHA
+    TYPE_RGB,
+    TYPE_RGBA
 };
 
+/// The locations of images handled in Gnash.
+enum ImageLocation
+{
+    GNASH_IMAGE_CPU = 1,
+    GNASH_IMAGE_GPU
+};
+
+inline size_t
+numChannels(ImageType t)
+{
+    switch (t) {
+        case TYPE_RGBA:
+            return 4;
+        case TYPE_RGB:
+            return 3;
+        default:
+            std::abort();
+    }
+}
 
 /// Base class for different types of bitmaps
 //
-/// @todo document layout of the image, like pixel data
-///       order in the raw array (rows or columns first?)
-///
-class DSOEXPORT GnashImage
+/// 1. Bytes are packed in RGB(A) order.
+/// 2. Rowstride is equal to channels * width
+class DSOEXPORT GnashImage : boost::noncopyable
 {
 public:
 
-    /// Copy an GnashImage.
-    GnashImage(const GnashImage& o) throw (std::bad_alloc)
-        :
-        _type(o._type),
-        _size(o.size()),
-        _width(o.width()),
-        _height(o.height()),
-        _pitch(o._pitch),
-        _data(new boost::uint8_t[_size])
-    {
-        update(o);
+    typedef boost::uint8_t value_type;
+    typedef boost::scoped_array<value_type> container_type;
+    typedef value_type* iterator;
+    typedef const value_type* const_iterator;
+
+    virtual ~GnashImage() {}
+
+    /// Return the ImageType of the image.
+    //
+    /// This saves guessing when dynamic_cast is used.
+    ImageType type() const {
+        return _type;
     }
+
+    /// Return the ImageLocation of the image.
+    //
+    /// This saves guessing when dynamic_cast is used.
+    ImageLocation location() const {
+        return _location;
+    }
+
+    /// Get the size of the image buffer
+    //
+    /// @return     The size of the buffer in bytes
+    size_t size() const {
+        return stride() * _height;
+    }
+
+    /// Get the pitch of the image buffer
+    //
+    /// @return     The rowstride of the buffer in bytes
+    virtual size_t stride() const {
+        return _width * channels();
+    }
+
+    /// Get the number of channels
+    //
+    /// @return     The number of channels
+    size_t channels() const {
+        return numChannels(_type);
+    }
+
+    /// Get the image's width
+    //
+    /// @return     The image's width in pixels.
+    size_t width() const {
+        return _width;
+    }
+
+    /// Get the image's width
+    //
+    /// @return     The image's height in pixels.
+    size_t height() const {
+        return _height;
+    }
+
+    /// Copy image data from a buffer.
+    //
+    /// Note that this buffer MUST have the same rowstride and type, or
+    /// unexpected things will happen. In general, it is only safe to copy
+    /// from another GnashImage or unexpected things will happen. 
+    ///
+    /// @param data     buffer to copy data from.
+    void update(const_iterator data);
+
+    /// Copy image data from another image data
+    //
+    /// Note that this buffer must have the same rowstride and type
+    ///
+    /// @param from     image to copy data from.
+    void update(const GnashImage& from);
     
-    /// \brief Construct an GnashImage from a data buffer,
-    /// taking ownership of the data.
+    /// Access the raw data.
+    virtual iterator begin() {
+        return _data.get();
+    }
+
+    /// Access the raw data
+    virtual const_iterator begin() const {
+        return _data.get();
+    }
+
+    /// An iterator to the end of the data.
+    iterator end() {
+        return begin() + size();
+    }
+
+    /// An iterator to the end of the data.
+    const_iterator end() const {
+        return begin() + size();
+    }
+
+protected:
+
+    /// Construct a GnashImage from a data buffer, taking ownership of the data.
     //
     /// @param data     The raw image data. This class takes ownership.
     /// @param width    The width of the image in pixels.
     /// @param height   The height of the image in pixels.
     /// @param pitch    The pitch (rowstride) of the image in bytes.
     /// @param type     The ImageType of the image.
-    GnashImage(boost::uint8_t *data, int width, int height,
-            int pitch, ImageType type);
+    GnashImage(iterator data, size_t width, size_t height, ImageType type,
+            ImageLocation location = GNASH_IMAGE_CPU);
 
     /// Construct an empty GnashImage
     //
-    /// Once constructed, this image must be updated with update() before
-    /// use.
+    /// Note: there is an arbitrary limit of boost::int32_t::max bytes for the
+    /// total size of the bitmap constructed with this constructor.
     //
     /// @param width    The width of the image in pixels.
     /// @param height   The height of the image in pixels.
-    /// @param pitch    The pitch (rowstride) of the image in bytes.
     /// @param type     The ImageType of the image.
-    GnashImage(int width, int height, int pitch, ImageType type);
-
-    /// Return the ImageType of the image.
-    //
-    /// This saves guessing when dynamic_cast is used.
-    ImageType type() const { return _type; }
-
-    /// Get the size of the image buffer
-    //
-    /// @return     The size of the buffer in bytes
-    size_t size() const { return _size; }
-
-    /// Get the pitch of the image buffer
-    //
-    /// @return     The pitch of the buffer in bytes
-    size_t pitch() const { return _pitch; }
-
-    /// Get size of a single pixel
-    //
-    /// @return     The size of a single pixel in bytes.
-    size_t pixelSize() const
-    {
-        return _pitch / _width;
-    }
-
-    /// Get the image's width
-    //
-    /// @return     The image's width in pixels.
-    size_t width() const { return _width; }
-
-    /// Get the image's width
-    //
-    /// @return     The image's height in pixels.
-    size_t height() const { return _height; }
-
-    /// Copy image data from a buffer.
-    //
-    /// Note that this buffer MUST have the same _pitch, or unexpected things
-    /// will happen. In general, it is only safe to copy from another GnashImage
-    /// (or derivative thereof) or unexpected things will happen. 
-    ///
-    /// @param data buffer to copy data from.
-    ///
-    void update(const boost::uint8_t* data);
-
-    /// Copy image data from another image data
-    //
-    /// Note that this buffer MUST have the same _pitch and _type
-    /// or an assertion will fail.
-    ///
-    /// @param from image to copy data from.
-    ///
-    void update(const GnashImage& from);
-    
-    /// Get access to the underlying data
-    //
-    /// @return     A pointer to the raw image data.
-    virtual boost::uint8_t* data() { return _data.get(); }
-
-    /// Get read-only access to the underlying data
-    //
-    /// @return     A read-only pointer to the raw image data.
-    virtual const boost::uint8_t* data() const { return _data.get(); }
-
-    /// Get a pointer to a given row
-    //
-    /// @param y    The index of the required row.
-    /// @return     A pointer to the first byte of the specified row.
-    boost::uint8_t* scanline(size_t y);
-
-    /// Get a read-only pointer to a given row
-    //
-    /// @param y    The index of the required row.
-    /// @return     A read-only pointer to the first byte of the specified
-    ///             row.
-    DSOEXPORT const boost::uint8_t* scanlinePointer(size_t y) const;
-
-    virtual ~GnashImage() {}
-
-    /// Clone the GnashImage.
-    //
-    /// @return     A copy of the GnashImage, owned by the caller.
-    virtual std::auto_ptr<GnashImage> clone() = 0;
-
-protected:
+    GnashImage(size_t width, size_t height, ImageType type,
+               ImageLocation location = GNASH_IMAGE_CPU);
 
+    /// The type of the image: RGBA or RGB.
     const ImageType _type;
 
-    /// Size of image buffer in bytes.
-    const size_t _size;
+    /// Image data location (CPU or GPU)
+    const ImageLocation _location;
 
     /// Width of image, in pixels
     const size_t _width;
@@ -193,61 +207,44 @@ protected:
     /// Height of image, in pixels
     const size_t _height;
 
-    /// Byte offset from one row to the next
-    //
-    /// This is basically width in bytes of each line.
-    /// For example, in an alpha image type this is equal to _width
-    /// while for an RGB this is 3 times the _width.
-    const size_t _pitch;
-
-    /// Data bytes, geometry defined by members below
-    boost::scoped_array<boost::uint8_t> _data;
+    /// Data if held in this class
+    container_type _data;
 
 };
 
-/// 24-bit RGB image.  Packed data, red byte first (RGBRGB...)
+/// 24-bit RGB bitmap
+//
+/// Channels are in RGB order.
 class DSOEXPORT ImageRGB : public GnashImage
 {
-
 public:
 
-    ImageRGB(int width, int height);
+    /// Create an empty RGB image with uninitialized data.
+    ImageRGB(size_t width, size_t height);
 
-    ImageRGB(const ImageRGB& o)
+    /// Create an ImageRGB taking ownership of the data.
+    ImageRGB(iterator data, size_t width, size_t height)
         :
-        GnashImage(o)
+        GnashImage(data, width, height, TYPE_RGB)
     {}
 
-    ImageRGB(boost::uint8_t* data, int width, int height, int stride)
-        :
-        GnashImage(data, width, height, stride, GNASH_IMAGE_RGB)
-    {}
-
-    ~ImageRGB();
-
-    virtual std::auto_ptr<GnashImage> clone()
-    {
-        return std::auto_ptr<GnashImage>(new ImageRGB(*this));
-    };
-
+    virtual ~ImageRGB();
 };
 
-/// 32-bit RGBA image.  Packed data, red byte first (RGBARGBA...)
+/// 32-bit RGBA bitmap
+//
+/// Channels are in RGBA order.
 class DSOEXPORT ImageRGBA : public GnashImage
 {
 
 public:
 
-    ImageRGBA(int width, int height);
+    /// Create an empty RGB image with uninitialized data.
+    ImageRGBA(size_t width, size_t height);
 
-    ImageRGBA(const ImageRGBA& o)
+    ImageRGBA(iterator data, size_t width, size_t height)
         :
-        GnashImage(o)
-    {}
-
-    ImageRGBA(boost::uint8_t* data, int width, int height, int stride)
-        :
-        GnashImage(data, width, height, stride, GNASH_IMAGE_RGBA)
+        GnashImage(data, width, height, TYPE_RGBA)
     {}
     
     ~ImageRGBA();
@@ -256,54 +253,27 @@ public:
     //
     /// TODO: move in base class ?
     ///
-    void setPixel(size_t x, size_t y, boost::uint8_t r, boost::uint8_t g, boost::uint8_t b, boost::uint8_t a);
-
-    void mergeAlpha(const boost::uint8_t* alphaData, const size_t bufferLength);
-
-    virtual std::auto_ptr<GnashImage> clone()
-    {
-        return std::auto_ptr<GnashImage>(new ImageRGBA(*this));
-    };
-
-};
-
-/// 8-bit alpha image.
-class DSOEXPORT alpha : public GnashImage
-{
-public:
-    alpha(int width, int height);
-
-    alpha(const alpha& o)
-        :
-        GnashImage(o)
-    {}
-
-    ~alpha();
-
-    virtual std::auto_ptr<GnashImage> clone()
-    {
-        return std::auto_ptr<GnashImage>(new alpha(*this));
-    };
-
+    void setPixel(size_t x, size_t y, value_type r, value_type g, value_type b,
+            value_type a);
 };
 
 /// The base class for reading image data. 
-class ImageInput : boost::noncopyable
+class Input : boost::noncopyable
 {
-
 public:
 
-    /// Construct an ImageInput object to read from an IOChannel.
+    /// Construct an Input object to read from an IOChannel.
     //
     /// @param in   The stream to read data from. Ownership is shared
-    ///             between caller and ImageInput, so it is freed
+    ///             between caller and Input, so it is freed
     ///             automatically when the last owner is destroyed.
-    ImageInput(boost::shared_ptr<IOChannel> in) :
+    Input(boost::shared_ptr<IOChannel> in)
+        :
         _inStream(in),
         _type(GNASH_IMAGE_INVALID)
     {}
 
-    virtual ~ImageInput() {}
+    virtual ~Input() {}
 
     /// Begin processing the image data.
     virtual void read() = 0;
@@ -360,24 +330,25 @@ protected:
 };
 
 // Base class for writing image data.
-class ImageOutput : boost::noncopyable
+class Output : boost::noncopyable
 {
 
 public:
 
-    /// Construct an ImageOutput for writing to an IOChannel
+    /// Construct an Output for writing to an IOChannel
     //
     /// @param out      The gnash::IOChannel to write the image to. Ownership
     ///                 is shared.
     /// @param width    The width of the resulting image
     /// @param height   The height of the resulting image.
-    ImageOutput(boost::shared_ptr<IOChannel> out, size_t width, size_t height) :
+    Output(boost::shared_ptr<IOChannel> out, size_t width, size_t height)
+        :
         _width(width),
         _height(height),
         _outStream(out)
-        {}
+    {}
 
-    virtual ~ImageOutput() {}
+    virtual ~Output() {}
     
     /// Write RGB image data using the parameters supplied at construction.
     //
@@ -394,16 +365,17 @@ public:
 
     /// Write the given image to the given IOChannel in a specified format.
     //
-    /// @param type     The image format to write in (see libcore/gnash.h)
+    /// @param type     The image format to write in (see GnashEnums.h)
     /// @param out      The IOChannel to write to.
     /// @param image    The image to write.
-    /// @param quality  The quality of the image output (not used for all
-    ///                 formats)
+    /// @param quality  The quality of the image output, from 0..100. Values
+    ///                 outside this range will be clamped to the minimum or
+    ///                 maxium value. The quality is not used for all
+    ///                 formats.
     DSOEXPORT static void writeImageData(FileType type,
             boost::shared_ptr<gnash::IOChannel> out, const GnashImage& image,
             int quality);
 
-
 protected:
 
     const size_t _width;
@@ -414,8 +386,32 @@ protected:
 
 };
 
+/// Get a pointer to a given row of any image.
+//
+/// @param row    The index of the required row.
+/// @return     A pointer to the first byte of the specified row.
+inline GnashImage::iterator
+scanline(GnashImage& im, size_t row)
+{
+    assert(row < im.height());
+    return im.begin() + im.stride() * row;
+}
+
+/// Get a read-only pointer to a given row of any image.
+//
+/// @param y    The index of the required row.
+/// @return     A read-only pointer to the first byte of the specified row.
+inline GnashImage::const_iterator
+scanline(const GnashImage& im, size_t row)
+{
+    assert(row < im.height());
+    return im.begin() + im.stride() * row;
+}
+
+DSOEXPORT void mergeAlpha(ImageRGBA& im, GnashImage::const_iterator alphaData,
+        const size_t bufferLength);
+
+} // namespace image
 } // namespace gnash
 
-
-
 #endif
