@@ -1,6 +1,7 @@
 // extension.cpp:  Read and enable plug-in extensions to Flash, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -21,31 +22,35 @@
 #include "gnashconfig.h"
 #endif
 
-// #if defined(_WIN32) || defined(WIN32)
-// # define lock(lib_mutex);
-// # define scoped_lock ;
-// #define PLUGINSDIR "./"	//hack
-// #define USE_DIRENT 1
-// #else
-// # include <boost/detail/lightweight_mutex.hpp>
-//   using boost::detail::lightweight_mutex;
-// # define scoped_lock lightweight_mutex::scoped_lock
-//   static lightweight_mutex lib_mutex;
-// #endif
-
-#include <ltdl.h>
+#if defined(WIN32) || defined(_WIN32)
+#define LIBLTDL_DLL_IMPORT 1
+#endif
 #include <cstring>
 #include <iostream>
 #include <sys/types.h>
 
 #include "log.h"
+#include "ltdl.h"
 #include "sharedlib.h"
 #include "extension.h"
-#include "as_object.h"
 
-#if HAVE_DIRENT_H || WIN32==1	// win32 hack
+namespace gnash {
+    class as_object;
+}
+
+#if defined(WIN32) || defined(_WIN32)
+int        lt_dlsetsearchpath   (const char *search_path);
+int        lt_dlinit           (void);
+void *     lt_dlsym            (lt_dlhandle handle, const char *name);
+const char *lt_dlerror         (void);
+int        lt_dlclose          (lt_dlhandle handle);
+int        lt_dlmakeresident   (lt_dlhandle handle);
+lt_dlhandle lt_dlopenext       (const char *filename);
+#endif
+
+#if HAVE_DIRENT_H || WIN32==1    // win32 hack
 # include <dirent.h>
-# define NAMLEN(dirent) strlen((dirent)->d_name)
+# define NAMLEN(dirent) std::strlen((dirent)->d_name)
 #else
 # define dirent direct
 # define NAMLEN(dirent) (dirent)->d_namlen
@@ -60,7 +65,6 @@
 # endif
 #endif
 
-using namespace std;
 namespace gnash {
 
 Extension::Extension() 
@@ -70,18 +74,21 @@ Extension::Extension()
 //     return lt_dlmutex_register (gnash_mutex_lock, gnash_mutex_unlock,
 //                                 gnash_mutex_seterror, gnash_mutex_geterror);
 #endif
-    char *env = getenv ("GNASH_PLUGINS");
-    if (env == 0) {
+    char *env = std::getenv("GNASH_PLUGINS");
+    if (!env) {
         _pluginsdir = PLUGINSDIR;
-    } else {
+    }
+    else {
         _pluginsdir = env;
     }
 
     log_debug("Plugins path: %s", _pluginsdir);
-    lt_dlsetsearchpath(_pluginsdir);
+#ifdef HAVE_LTDL
+    lt_dlsetsearchpath(_pluginsdir.c_str());
+#endif
 }
 
-Extension::Extension(const char *dir)
+Extension::Extension(const std::string& dir)
 {
 //    GNASH_REPORT_FUNCTION;
 #ifdef LT_DLMUTEX
@@ -89,7 +96,9 @@ Extension::Extension(const char *dir)
 //                                 gnash_mutex_seterror, gnash_mutex_geterror);
 #endif
     _pluginsdir = dir;
-    lt_dlsetsearchpath(_pluginsdir);
+#ifdef HAVE_LTDL
+    lt_dlsetsearchpath(_pluginsdir.c_str());
+#endif
 }
 
 Extension::~Extension()
@@ -98,61 +107,59 @@ Extension::~Extension()
 }
 
 bool
-Extension::scanAndLoad(const char *dir, as_object &obj)
+Extension::scanAndLoad(const std::string& dir, as_object &obj)
 {
 //    GNASH_REPORT_FUNCTION;
     
-    lt_dlsetsearchpath(_pluginsdir);
+#ifdef HAVE_LTDL
+    lt_dlsetsearchpath(_pluginsdir.c_str());
+#endif
     _pluginsdir = dir;
     
     return scanAndLoad(obj);
 }
 
 bool
-Extension::scanAndLoad(as_object &obj)
+Extension::scanAndLoad(as_object& where)
 {
 //    GNASH_REPORT_FUNCTION;
-    string mod;
     
-    if (_modules.size() == 0) {
+    if (_modules.empty()) {
         scanDir(_pluginsdir);
     }
     
-    vector<string>::iterator it;
+    std::vector<std::string>::iterator it;
     for (it = _modules.begin(); it != _modules.end(); it++) {
-        mod = *(it);
-        log_security(_("Loading module: %s"), mod.c_str());
-        SharedLib sl;
-        initModule(mod.c_str(), obj);
-    }   
-		return true;
+        const std::string& mod = *it;
+        log_security(_("Loading module: %s from %s"), mod, _pluginsdir);
+        initModule(mod, where);
+    }
+    return true;
 }
 
 bool
-Extension::initModule(const char *module, as_object &obj)
+Extension::initModule(const std::string& module, as_object &where)
 {
-//    GNASH_REPORT_FUNCTION;
 
-    SharedLib::initentry *symptr;
     SharedLib *sl;
-    string symbol;
+    std::string symbol(module);
 
-    log_security(_("Initializing module: \"%s\""), module);
+    log_security(_("Initializing module: \"%s\" from %s"), symbol, _pluginsdir);
     
-    symbol = module;
     if (_plugins[module] == 0) {
-        sl = new SharedLib(module);
+        sl = new SharedLib(_pluginsdir + "/" + module, "GNASH_PLUGINS");
         sl->openLib();
         _plugins[module] = sl;
     } else {
         sl = _plugins[module];
     }
     
-    symbol += "_class_init";
-    symptr = sl->getInitEntry(symbol.c_str());
+    symbol.append("_class_init");
+    
+    SharedLib::initentry *symptr = sl->getInitEntry(symbol);
 
     if (symptr) {    
-        symptr(obj);
+        symptr(where);
     } else {
         log_error(_("Couldn't get class_init symbol"));
     }
@@ -161,31 +168,32 @@ Extension::initModule(const char *module, as_object &obj)
 }
 
 bool
-Extension::initModuleWithFunc(const char *module, const char *func,
-	as_object &obj)
+Extension::initModuleWithFunc(const std::string& module,
+        const std::string& func, as_object &obj)
 {
-	SharedLib::initentry *symptr;
-	SharedLib *sl;
+    GNASH_REPORT_FUNCTION;
 
-	log_security(_("Initializing module: \"%s\""), module);
+    SharedLib *sl;
 
-	if (_plugins[module] == 0) {
-		sl = new SharedLib(module);
-		sl->openLib();
-		_plugins[module] = sl;
-	} else {
-		sl = _plugins[module];
-	}
+    log_security(_("Initializing module: \"%s\""), module);
 
-	symptr = sl->getInitEntry(func);
+    if (_plugins[module] == 0) {
+        sl = new SharedLib(module);
+        sl->openLib();
+        _plugins[module] = sl;
+    } else {
+        sl = _plugins[module];
+    }
 
-	if (symptr) {
-		symptr(obj);
-	} else {
-		log_error(_("Couldn't get class_init symbol: \"%s\""), func);
-	}
+    SharedLib::initentry *symptr = sl->getInitEntry(func);
 
-	return true;
+    if (symptr) {
+        symptr(obj);
+    } else {
+        log_error(_("Couldn't get class_init symbol: \"%s\""), func);
+    }
+
+    return true;
 }
 
 bool
@@ -193,76 +201,60 @@ Extension::scanDir()
 {
 //    GNASH_REPORT_FUNCTION;
     scanDir(_pluginsdir);
-		return true;
+    return true;
 }
 
 bool
-Extension::scanDir(const char *dirlist)
+Extension::scanDir(const std::string& dirlist)
 {
-//    GNASH_REPORT_FUNCTION;
+    GNASH_REPORT_FUNCTION;
     
-    int i;
-    struct dirent *entry;
-    //string::size_type pos;
-    char *dirlistcopy;
-    char *dir;
-    char *suffix = 0;
+    Tok t(dirlist, Sep(":"));
+    for (Tok::iterator i = t.begin(), e = t.end(); i != e; ++i) {
 
-//    scoped_lock lock(lib_mutex);
+        const std::string& dir = *i;
 
-    dirlistcopy = strdup(dirlist);
-    
-    dir = strtok(dirlistcopy, ":");
-    if (dir == NULL) {
-        dir = dirlistcopy;
-    }
-
-            
-    while (dir) {
         log_debug(_("Scanning directory \"%s\" for plugins"), dir);
-        DIR *library_dir = opendir(dir);
+        DIR *libdir = opendir(dir.c_str());
 
-        if (library_dir == NULL) {
+        if (!libdir) {
             log_error(_("Can't open directory %s"), dir);
             return false;
-        }
+        }   
         
-        entry = readdir(library_dir);
-        for (i=0; entry>0; i++) {
-            // We only want shared libraries than end with the suffix, otherwise
+        struct dirent *entry;
+
+        while ((entry = readdir(libdir)) != NULL) {
+            // We only want shared libraries that end with the suffix, otherwise
             // we get all the duplicates.
-            entry = readdir(library_dir);
+            std::string name(entry->d_name);
 
-            if (entry <= NULL) { // All done
-                continue;
-            }
-
-            if (strncmp(entry->d_name, ".", 1) == 0) {
+            // Hidden files.
+            if (name.at(0) == '.') {
                 continue;
             }            
-            
-            suffix = strrchr(entry->d_name, '.');
-            if (suffix == 0) {
-                continue;
-            }
+                
+            const std::string::size_type pos = name.find_last_of('.');
+ 
+            if (pos == std::string::npos) continue;
+ 
+            const std::string suffix = name.substr(pos);
+            name.erase(pos);
 
-            log_debug(_("Gnash Plugin name: %s"), entry->d_name);
-            
-            if (strcmp(suffix, ".so") == 0) {
-                *suffix = 0;
-                log_debug(_("Gnash Plugin name: %s"), entry->d_name);
-                _modules.push_back(entry->d_name);
-            } else {
+            if (suffix == ".so") {
+                log_debug(_("Gnash Plugin name: %s"), name);
+                _modules.push_back(name);
+            }
+            else {
                 continue;
             }
         }
         
-        if (closedir(library_dir) != 0) {
+        if (closedir(libdir) != 0) {
             return false;
         }
-        dir = strtok(NULL, ":");
     }
-	return true;
+    return true;
 }
 
 void
@@ -270,10 +262,10 @@ Extension::dumpModules()
 {
     GNASH_REPORT_FUNCTION;
     
-    cerr << _modules.size() << " plugin(s) for Gnash installed" << endl;    
-    vector<string>::iterator it;
+    std::cerr << _modules.size() << " plugin(s) for Gnash installed" << std::endl;    
+    std::vector<std::string>::iterator it;
     for (it = _modules.begin(); it != _modules.end(); it++) {
-        cerr << "Module name is: \"" << *(it) << "\"" << endl;
+        std::cerr << "Module name is: \"" << *(it) << "\"" << std::endl;
     }
 }
 
