@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -21,12 +21,11 @@
 
 #include "Range2d.h"
 
-#include <list>
 #include <vector>
-#include <boost/bind.hpp>
 #include <iterator>
 #include <algorithm>
-#include <iostream>
+#include <ostream>
+#include <boost/cstdint.hpp>
 
 namespace gnash {
 
@@ -652,7 +651,7 @@ inline bool snaptest(const geometry::Range2d<T>& range1,
 } // namespace geometry
 
 /// Standard snapping 2d ranges type for invalidated bounds calculation    
-typedef geometry::SnappingRanges2d<float> InvalidatedRanges;
+typedef geometry::SnappingRanges2d<boost::int32_t> InvalidatedRanges;
 
 } //namespace gnash
 
