@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -24,9 +24,7 @@
 #include "Movie.h" // for inheritance
 
 // Forward declarations
-namespace gnash
-{
-    class GnashImage;
+namespace gnash {
     class DisplayObject;
 }
 
