@@ -1,6 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -24,30 +24,34 @@
 #include "gnashconfig.h" // USE_SWFTREE
 #endif
 
-#include "smart_ptr.h" // GNASH_USE_GC
-#include "event_id.h" // for inlines
-#include "as_object.h" // for inheritance
-#include "SWFRect.h" // for composition (invalidated bounds)
-#include "SWFMatrix.h" // for composition
-#include "cxform.h" // for composition
-#include "dsodefs.h" //for DSOEXPORT
-#include "log.h"
-#include "snappingrange.h"
-#ifdef USE_SWFTREE
-# include "tree.hh"
-#endif
-
+#include <vector>
 #include <map>
 #include <string>
 #include <cassert>
 #include <boost/cstdint.hpp> // For C99 int types
 #include <boost/noncopyable.hpp>
 
+#include "ObjectURI.h" 
+#include "GC.h"
+#include "Transform.h"
+#include "event_id.h" 
+#include "SWFRect.h"
+#include "SWFMatrix.h"
+#include "SWFCxForm.h"
+#include "dsodefs.h" 
+#include "snappingrange.h"
+#ifdef USE_SWFTREE
+# include "tree.hh"
+#endif
+
+
 //#define DEBUG_SET_INVALIDATED 1
 
 // Forward declarations
 namespace gnash {
     class MovieClip;
+    class movie_root;
+    class fn_call;
     class Movie;
     class ExecutableCode;
     class action_buffer;
@@ -55,6 +59,11 @@ namespace gnash {
     class StaticText;
     class InteractiveObject;
     class Renderer;
+    class as_object;
+    class as_value;
+    class as_environment;
+    class DisplayObject;
+    class KeyVisitor;
     namespace SWF {
         class TextRecord;
     }
@@ -75,7 +84,7 @@ bool isReferenceable(const DisplayObject& d);
 /// @param obj      The DisplayObject whose property should be set
 /// @param val      An as_value representing the new value of the property.
 ///                 Some values may be rejected.
-bool setDisplayObjectProperty(DisplayObject& obj, string_table::key key,
+bool setDisplayObjectProperty(DisplayObject& obj, const ObjectURI& uri,
         const as_value& val);
 
 /// Get special properties
@@ -83,10 +92,10 @@ bool setDisplayObjectProperty(DisplayObject& obj, string_table::key key,
 /// This gets the magic properties of DisplayObjects and handles special
 /// MovieClip properties such as DisplayList members.
 //
-/// @param key      The string table key of the property to get.
+/// @param key      The uri of the property to get.
 /// @param obj      The DisplayObject whose property should be got
 /// @param val      An as_value to be set to the value of the property.
-bool getDisplayObjectProperty(DisplayObject& obj, string_table::key key,
+bool getDisplayObjectProperty(DisplayObject& obj, const ObjectURI& uri,
         as_value& val);
 
 /// Get a property by its numeric index.
@@ -114,6 +123,21 @@ void setIndexedProperty(size_t index, DisplayObject& o, const as_value& val);
 /// @param to       The DisplayObject to copy to.
 void copyMatrix(const DisplayObject& from, DisplayObject& to);
 
+/// Get concatenated SWFMatrix (all ancestor transforms and our SWFMatrix)
+//
+/// Maps from our local space into "world" space
+/// (i.e. root movie space).
+//
+/// @param includeRoot      Whether the transform of the Stage (_root)
+///                         should be concatenated. This is required to be
+///                         false for pointInBounds.
+SWFMatrix getWorldMatrix(const DisplayObject& d, bool includeRoot = true);
+
+/// Get concatenated color transform of a DisplayObject
+//
+/// Maps from our local space into normal color space.
+SWFCxForm getWorldCxForm(const DisplayObject& d);
+
 /// DisplayObject is the base class for all DisplayList objects.
 //
 /// It represents a single active element in a movie. This class does not
@@ -205,9 +229,6 @@ public:
     static const int noClipDepthValue = -1000000;
 
     /// Return a reference to the variable scope of this DisplayObject.
-    //
-    /// TODO: make const/return const& ?
-    ///
     virtual as_environment& get_environment() {
         // MovieClip must override this
         // and any other DisplayObject will have
@@ -218,18 +239,16 @@ public:
 
     /// Enumerate any non-proper properties
     //
-    /// This function is called by enumerateProperties(as_environment&) 
-    /// to allow for enumeration of properties that are derived from the
-    /// DisplayObject type, e.g. DisplayList members.
+    /// This function allows enumeration of properties that are
+    /// derived from the DisplayObject type, e.g. DisplayList members.
     ///
     /// The default implementation adds nothing
-    ///
-    virtual void enumerateNonProperties(as_environment&) const {}
+    virtual void visitNonProperties(KeyVisitor&) const {}
 
     /// \brief
     /// Return the parent of this DisplayObject, or NULL if
     /// the DisplayObject has no parent.
-    DisplayObject* get_parent() const
+    DisplayObject* parent() const
     {
         return _parent;
     }
@@ -245,15 +264,15 @@ public:
 
     virtual MovieClip* to_movie() { return 0; }
 
-    int get_depth() const { return m_depth; }
+    int get_depth() const { return _depth; }
 
-    void  set_depth(int d) { m_depth = d; }
+    void set_depth(int d) { _depth = d; }
 
     /// Get sound volume for this DisplayObject
     int getVolume() const { return _volume; }
 
     /// Set sound volume for this DisplayObject
-    void setVolume(int vol) { _volume=vol; }
+    void setVolume(int vol) { _volume = vol; }
 
     /// Get concatenated sound volume for this DisplayObject
     //
@@ -268,9 +287,11 @@ public:
     virtual int getDefinitionVersion() const {
         return -1;
     }
+    
+    const Transform& transform() const {
+        return _transform;
+    }
 
-    /// Get local transform SWFMatrix for this DisplayObject
-    const SWFMatrix& getMatrix() const { return m_matrix; }
 
     /// Set local transform SWFMatrix for this DisplayObject
     //
@@ -279,7 +300,7 @@ public:
     /// @param updateCache if true, updates the cache values
     ///        from the SWFMatrix (only if SWFMatrix != current SWFMatrix)
     ///
-    void setMatrix(const SWFMatrix& m, bool updateCache=false);
+    void setMatrix(const SWFMatrix& m, bool updateCache = false);
 
     /// Set the xscale value of current SWFMatrix
     //
@@ -327,33 +348,24 @@ public:
     ///
     virtual void setHeight(double height);
 
-    const cxform& get_cxform() const { return m_color_transform; }
-
-    void set_cxform(const cxform& cx) 
+    void setCxForm(const SWFCxForm& cx) 
     {       
-        if (cx != m_color_transform) {
-            set_invalidated(__FILE__, __LINE__);
-            m_color_transform = cx;
+        if (_transform.colorTransform != cx) {
+            set_invalidated();
+            _transform.colorTransform = cx;
         }
     }
 
-    void concatenate_cxform(const cxform& cx) {
-        m_color_transform.concatenate(cx);
-    }
+    boost::uint16_t get_ratio() const { return _ratio; }
 
-    void concatenateMatrix(const SWFMatrix& m) { m_matrix.concatenate(m); }
-
-    int get_ratio() const { return m_ratio; }
-
-    void set_ratio(int r)
-    {
-        if (r != m_ratio) set_invalidated(__FILE__, __LINE__); 
-        m_ratio = r;       
+    void set_ratio(boost::uint16_t r) {
+        if (r != _ratio) set_invalidated(); 
+        _ratio = r;       
     }
 
     /// Returns the clipping depth (if any) of this DisplayObject.
     /// The parameter tells us to use the DisplayObject as a mask for
-    /// all the objects contained in the display list from m_depth
+    /// all the objects contained in the display list from _depth
     /// to m_clipping_depth inclusive.
     /// 
     /// The value returned by get_clip_depth() is only valid when isMaskLayer()
@@ -411,39 +423,12 @@ public:
     ///
     void setMask(DisplayObject* mask);
 
-    /// Returns true if this DisplayObject is a mask (either layer or
-    /// dynamic mask)
-    bool isMask() const
-    {
-        return isDynamicMask() || isMaskLayer();
-    }
-
     /// Set DisplayObject name, initializing the original target member
-    void set_name(const std::string& name)
-    {
-        _name = name;
+    void set_name(const ObjectURI& uri) {
+        _name = uri;
     }
 
-    const std::string& get_name() const { return _name; }
-
-    /// \brief
-    /// Get our concatenated SWFMatrix (all our ancestor transforms,
-    /// times our SWFMatrix). 
-    ///
-    /// Maps from our local space into "world" space
-    /// (i.e. root movie space).
-    //
-    /// @param includeRoot      Whether the transform of the Stage (_root)
-    ///                         should be concatenated. This is required to be
-    ///                         false for pointInBounds.
-    DSOEXPORT SWFMatrix getWorldMatrix(bool includeRoot = true) const;
-
-    /// \brief
-    /// Get our concatenated color transform (all our ancestor transforms,
-    /// times our cxform). 
-    ///
-    /// Maps from our local space into normal color space.
-    virtual cxform get_world_cxform() const;
+    const ObjectURI& get_name() const { return _name; }
 
     /// Get the built-in function handlers code for the given event
     //
@@ -479,7 +464,7 @@ public:
     /// Render the DisplayObject.
     //
     /// All DisplayObjects must have a display() function.
-	virtual void display(Renderer& renderer) = 0;
+	virtual void display(Renderer& renderer, const Transform& xform) = 0;
 
     /// Search for StaticText objects
     //
@@ -502,7 +487,7 @@ public:
     bool pointInBounds(boost::int32_t x, boost::int32_t y) const
     {
         SWFRect bounds = getBounds();
-        SWFMatrix wm = getWorldMatrix(false);
+        const SWFMatrix wm = getWorldMatrix(*this, false);
         wm.transform(bounds);
         return bounds.point_test(x, y);
     }
@@ -528,8 +513,8 @@ public:
     ///
     virtual bool pointInVisibleShape(boost::int32_t x, boost::int32_t y) const
     {
-        if ( ! visible() ) return false;
-        if ( isMask() ) return false;
+        if (!visible()) return false;
+        if (isDynamicMask() || isMaskLayer()) return false;
         return pointInShape(x, y);
     }
 
@@ -543,7 +528,7 @@ public:
     /// against this DisplayObject's parent.
     ///
     virtual Movie* get_root() const {
-        return get_parent()->get_root();
+        return parent()->get_root();
     }
 
     /// Return the _root ActionScript property of this DisplayObject.
@@ -572,19 +557,7 @@ public:
     /// In ActionScript 1.0, everything seems to be CASE
     /// INSENSITIVE.
     ///
-    virtual as_object* pathElement(string_table::key key);
-
-    /// Advance this DisplayObject to next frame.
-    //
-    /// Character advancement is only meaningful for sprites
-    /// and sprite containers (button DisplayObjects) because
-    /// sprites are the only DisplayObjects that have frames.
-    /// 
-    /// Frame advancement include execution of all control tags.
-    /// 
-    virtual void advance()
-    {
-    }
+    virtual as_object* pathElement(const ObjectURI& uri);
 
     /// \brief
     /// Return true if PlaceObjects tag are allowed to move
@@ -657,7 +630,6 @@ public:
     //
     /// notifyEvent(id) will be called by execution of the queued
     /// action
-    ///
     void queueEvent(const event_id& id, int lvl);
 
     /// Return true if an handler for the given event is defined
@@ -667,7 +639,6 @@ public:
     /// this in a non-virtual function. Main use for this method
     /// is for being called by ::unload() to verify an Unload handler
     /// is available.
-    ///
     bool hasEventHandler(const event_id& id) const;
 
 	/// DisplayObjects are not a mouse entity by default.
@@ -682,7 +653,6 @@ public:
     /// point and is not the DisplayObject being dragged or any of its childs.
     //
     /// Point coordinates in global twips.
-    ///
     virtual const DisplayObject* findDropTarget(boost::int32_t x, 
             boost::int32_t y, DisplayObject* dragging) const
     {
@@ -693,7 +663,20 @@ public:
         return 0;
     }
 
-    /// @}
+    /// Return whether this DisplayObject has been invalidated or not
+    bool invalidated() const {
+        return _invalidated;
+    }
+
+    /// Return whether this DisplayObject has and invalidated child or not
+    bool childInvalidated() const {
+        return _child_invalidated;
+    }
+
+    /// Notify a change in the DisplayObject's appearance.
+    virtual void update() {
+        set_invalidated();
+    }
 
     /// \brief
     /// This function marks the DisplayObject as being modified in aspect
@@ -731,7 +714,6 @@ public:
     /// not need to redraw itself completely. This function will 
     /// recursively inform all its parents of the change.
     void set_child_invalidated();
-    
 
     /// Clear invalidated flag and reset m_old_invalidated_bounds to null.
     ///
@@ -746,8 +728,8 @@ public:
     /// prevent the parent to be informed when this DisplayObject (or a
     /// child) is invalidated again (see set_invalidated() recursion).
     void clear_invalidated() {
-        m_invalidated = false;
-        m_child_invalidated = false;        
+        _invalidated = false;
+        _child_invalidated = false;        
         m_old_invalidated_ranges.setNull();
     }
     
@@ -767,7 +749,7 @@ public:
     ///
     /// It is used to determine what area needs to be re-rendered.
     /// The coordinates are world coordinates (in TWIPS).
-    /// Only instances with m_invalidated flag set are checked unless
+    /// Only instances with _invalidated flag set are checked unless
     /// force is set.
     ///
     virtual void add_invalidated_bounds(InvalidatedRanges& ranges, bool force);
@@ -787,7 +769,8 @@ public:
     /// If you override the method remember to call saveOriginalTarget()
     /// as the first thing.
     ///
-    virtual void stagePlacementCallback(as_object* = 0)
+    /// This handles all ActionScript construction and initialization events.
+    virtual void construct(as_object* /*init*/ = 0)
     {
         saveOriginalTarget();
     }
@@ -803,14 +786,15 @@ public:
     /// @return true if any onUnload event handler was defined
     ///                 by either this or any child DisplayObjects, false
     ///                 otherwise.
-    ///
-    virtual bool unload();
+    bool unload();
 
     /// Accept a loaded Movie
     virtual void getLoadedMovie(Movie* newMovie);
 
     /// Return true if this DisplayObject was unloaded from the stage
-    bool unloaded() const { return _unloaded; }
+    bool unloaded() const {
+        return _unloaded;
+    }
 
     /// Mark this DisplayObject as destroyed
     //
@@ -877,8 +861,7 @@ public:
     bool DSOEXPORT allowHandCursor() const;
 
 #ifdef USE_SWFTREE
-    typedef std::pair<std::string, std::string> StringPair; 
-    typedef tree<StringPair> InfoTree; 
+    typedef tree<std::pair<std::string, std::string> > InfoTree; 
     /// Append DisplayObject info in the tree
     //
     /// @param tr
@@ -888,14 +871,12 @@ public:
     /// The iterator to append info to.
     ///
     /// @return iterator the appended subtree
-    ///
-    // TODO: use a typedef for tree<StringPair> ?
     virtual InfoTree::iterator getMovieInfo(InfoTree& tr,
             InfoTree::iterator it);
 #endif
     
     /// Used to assign a name to unnamed instances
-    std::string getNextUnnamedInstanceName();
+    ObjectURI getNextUnnamedInstanceName();
 
     enum BlendMode
     {
@@ -957,7 +938,9 @@ public:
         return _yscale;
     }
 
-    as_object* object() const;
+    as_object* object() const {
+        return _object;
+    }
 
     /// Getter-setter for blendMode.
     static as_value blendMode(const fn_call& fn);
@@ -975,9 +958,29 @@ public:
     virtual void markOwnResources() const {}
 
 protected:
+    
+    /// Render a dynamic mask for a specified DisplayObject
+    //
+    /// Dynamic masks are rendered out-of-turn when the object they are masking
+    /// is drawn. 
+    //
+    /// A MaskRenderer object should be constructed at the beginning of
+    /// relevant display() functions; it then takes care of rendering the
+    /// mask with the appropriate transform and cleaning up afterwards.
+    class MaskRenderer
+    {
+    public:
+        MaskRenderer(Renderer& r, const DisplayObject& o);
+        ~MaskRenderer();
+    private:
+        Renderer& _renderer;
+        DisplayObject* _mask;
+    };
+
+    virtual bool unloadChildren() { return false; }
 
     /// Get the movie_root to which this DisplayObject belongs.
-    movie_root& stage() {
+    movie_root& stage() const {
         return _stage;
     }
 
@@ -992,7 +995,6 @@ protected:
         _origTarget=getTarget();
     }
 
-
     const Events& get_event_handlers() const
     {
         return _event_handlers;
@@ -1001,7 +1003,7 @@ protected:
     void set_event_handlers(const Events& copyfrom);
 
     /// Name of this DisplayObject (if any)
-    std::string _name;
+    ObjectURI _name; 
 
     DisplayObject* _parent;
 
@@ -1011,21 +1013,6 @@ protected:
     ///
     as_object* getPathElementSeparator(string_table::key key);
 
-    /// \brief
-    /// Set when the visual aspect of this particular DisplayObject or movie
-    /// has been changed and redrawing is necessary.    
-    //
-    /// This is initialized to true as the initial state for
-    /// any DisplayObject is the "invisible" state (it wasn't there)
-    /// so it starts in invalidated mode.
-    ///
-    bool m_invalidated;
-
-    /// Just like m_invalidated but set when a child is invalidated instead
-    /// of this DisplayObject instance. m_invalidated and m_child_invalidated
-    /// can be set at the same time. 
-    bool m_child_invalidated;
-
     /// \brief
     /// Bounds of this DisplayObject instance before first invalidation
     /// since last call to clear_invalidated().
@@ -1048,24 +1035,24 @@ private:
     /// Register a DisplayObject masked by this instance
     void setMaskee(DisplayObject* maskee);
 
-    /// Build the _target member recursive on parent
-    std::string computeTargetPath() const;
-
     /// The as_object to which this DisplayObject is attached.
     as_object* _object;
 
     /// The movie_root to which this DisplayObject belongs.
     movie_root& _stage;
 
-    int m_depth;
-    cxform m_color_transform;
-    SWFMatrix m_matrix;
+    Transform _transform;
+    
+    Events _event_handlers;
 
     /// Cache values for ActionScript access.
     /// NOTE: not all DisplayObjects need this, just the
     ///       ones which are ActionScript-referenceable
     double _xscale, _yscale, _rotation;
 
+    /// The depth of this DisplayObject.
+    boost::int32_t _depth;
+
     /// Volume control associated to this DisplayObject
     //
     /// This is used by Sound objects
@@ -1076,15 +1063,8 @@ private:
     ///
     int _volume;
 
-    int m_ratio;
+    boost::uint16_t _ratio;
     int m_clip_depth;
-    Events _event_handlers;
-
-    /// Set to yes when this instance has been unloaded
-    bool _unloaded;
-
-    /// This flag should be set to true by a call to destroy()
-    bool _destroyed;
 
     /// The DisplayObject masking this instance (if any)
     DisplayObject* _mask;
@@ -1110,8 +1090,61 @@ private:
 
     bool _dynamicallyCreated;
 
+    /// Set to yes when this instance has been unloaded
+    bool _unloaded;
+
+    /// This flag should be set to true by a call to destroy()
+    bool _destroyed;
+
+    /// \brief
+    /// Set when the visual aspect of this particular DisplayObject or movie
+    /// has been changed and redrawing is necessary.    
+    //
+    /// This is initialized to true as the initial state for
+    /// any DisplayObject is the "invisible" state (it wasn't there)
+    /// so it starts in invalidated mode.
+    ///
+    bool _invalidated;
+
+    /// Just like _invalidated but set when a child is invalidated instead
+    /// of this DisplayObject instance. _invalidated and _child_invalidated
+    /// can be set at the same time. 
+    bool _child_invalidated;
+
+
 };
 
+/// Get local transform SWFMatrix for this DisplayObject
+inline const SWFMatrix&
+getMatrix(const DisplayObject& o)
+{ 
+    return o.transform().matrix;
+}
+
+inline const SWFCxForm&
+getCxForm(const DisplayObject& o) 
+{
+    return o.transform().colorTransform;
+}
+
+inline SWFMatrix
+getWorldMatrix(const DisplayObject& d, bool includeRoot)
+{
+    SWFMatrix m = d.parent() ?
+        getWorldMatrix(*d.parent(), includeRoot) : SWFMatrix();
+
+    if (d.parent() || includeRoot) m.concatenate(getMatrix(d));
+    return m;
+}
+
+inline SWFCxForm
+getWorldCxForm(const DisplayObject& d)
+{
+    SWFCxForm cx = d.parent() ? getWorldCxForm(*d.parent()) : SWFCxForm();
+    cx.concatenate(getCxForm(d));
+    return cx;
+}
+
 inline bool
 isReferenceable(const DisplayObject& d)
 {
