@@ -1,7 +1,7 @@
 // Shape.h: Shape DisplayObject implementation for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -58,7 +58,7 @@ public:
 	    assert(_def);
 	}
 
-	virtual void display(Renderer& renderer);
+	virtual void display(Renderer& renderer, const Transform& xform);
 
     virtual SWFRect getBounds() const {
         return _def ? _def->bounds() : _shape->getBounds();
