@@ -1,6 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -37,7 +37,6 @@ namespace gnash {
 /// An interval timer.
 //
 /// This is constructed when _global.setInterval() is called.
-/// Instances of this class will be stored in the movie_root singleton.
 ///
 /// A timer has a function to call, a context in which to call it, a
 /// list of arguments and an interval specifying how often the function must be
@@ -97,7 +96,7 @@ public:
     ///
     /// @param runOnce
     ///     If true the interval will run only once. False if omitted.
-    Timer(as_object* obj, string_table::key methodName, unsigned long ms,
+    Timer(as_object* obj, const ObjectURI& methodName, unsigned long ms,
             const fn_call::Args& args, bool runOnce = false);
 
     /// Clear the timer, ready for reuse
@@ -140,7 +139,6 @@ public:
     ///
     void executeAndReset();
 
-#ifdef GNASH_USE_GC
     /// Mark all reachable resources (for GC)
     //
     /// Resources reachable from Timer are:
@@ -150,7 +148,6 @@ public:
     ///    - Target object (_object)
     ///
     void markReachableResources() const;
-#endif // GNASH_USE_GC
 
 private:
 
@@ -186,7 +183,7 @@ private:
     /// an intrusive pointer
     as_function* _function;
 
-    string_table::key _methodName;
+    ObjectURI _methodName;
 
     /// Context for the function call. Will be used as 'this' pointer.
     as_object* _object;
