@@ -1,6 +1,7 @@
 // VirtualClock.h -- virtual clock for gnash core lib
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -19,12 +20,6 @@
 #ifndef GNASH_VIRTUAL_CLOCK_H
 #define GNASH_VIRTUAL_CLOCK_H
 
-// What's the policy for when to include config.h ? --strk Dec 7 2007;
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
-#include <memory> // for InterruptableVirtualClock
 #include <cassert> // for InterruptableVirtualClock
 
 namespace gnash
