@@ -1,7 +1,7 @@
 // Mouse.cpp:  ActionScript "Mouse" input device class, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -25,7 +25,6 @@
 #include "fn_call.h"
 #include "Global_as.h"
 #include "smart_ptr.h" // for boost intrusive_ptr
-#include "builtin_function.h" // need builtin_function
 #include "NativeFunction.h" 
 #include "VM.h" // for registerNative
 #include "AsBroadcaster.h" // for initializing self as a broadcaster
@@ -92,7 +91,8 @@ as_value
 mouse_hide(const fn_call& fn)
 {
     movie_root& m = getRoot(fn);
-    const int success = (m.callInterface("Mouse.hide") == "true") ? 1 : 0;
+    const int success =
+        m.callInterface<bool>(HostMessage(HostMessage::SHOW_MOUSE, false));
 
     // returns 1 if mouse was visible before call.
     return as_value(success);
@@ -105,7 +105,8 @@ as_value
 mouse_show(const fn_call& fn)
 {
     movie_root& m = getRoot(fn);
-    const int success = (m.callInterface("Mouse.show") == "true") ? 1 : 0;
+    const int success = 
+        m.callInterface<bool>(HostMessage(HostMessage::SHOW_MOUSE, true));
 
     // returns 1 if Mouse was visible before call.
     return as_value(success);
