@@ -1,7 +1,7 @@
 // NetStream.cpp:  ActionScript class for streaming audio/video, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -18,16 +18,19 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
+#include "NetStream_as.h"
 
+#include <functional>
+#include <algorithm>
+#include <boost/cstdint.hpp>
+#include <boost/thread/mutex.hpp>
 
 #include "RunResources.h"
-#include "NetStream_as.h"
 #include "CharacterProxy.h"
-#include "smart_ptr.h" // GNASH_USE_GC
+#include "smart_ptr.h"
 #include "log.h"
 #include "fn_call.h"
 #include "Global_as.h"
-#include "builtin_function.h"
 #include "NativeFunction.h"
 #include "GnashException.h"
 #include "NetConnection_as.h"
@@ -39,8 +42,8 @@
 #include "MediaHandler.h"
 #include "StreamProvider.h"
 #include "sound_handler.h"
-#include "GnashSystemNetHeaders.h"
 #include "AMFConverter.h"
+#include "AMF.h"
 
 // Define the following macro to have status notification handling debugged
 //#define GNASH_DEBUG_STATUS
@@ -71,7 +74,14 @@ namespace {
     as_value netstream_send(const fn_call& fn);
 
     void attachNetStreamInterface(as_object& o);
-    
+
+    /// Transform the volume by the requested amount
+    //
+    /// @param data     The data to transform
+    /// @param size     The length of the array
+    /// @param volume   The volume in percent.
+    void adjustVolume(boost::int16_t* data, size_t size, int volume);
+
     // TODO: see where this can be done more centrally.
     void executeTag(const SimpleBuffer& _buffer, as_object& thisPtr);
 }
@@ -195,7 +205,7 @@ NetStream_as::newFrameReady()
     return false;
 }
 
-std::auto_ptr<GnashImage>
+std::auto_ptr<image::GnashImage>
 NetStream_as::get_video()
 {
     boost::mutex::scoped_lock lock(image_mutex);
@@ -263,7 +273,7 @@ NetStream_as::getStatusObject(StatusCode code)
     // Enumerable and deletable.
     const int flags = 0;
 
-    as_object* o = getGlobal(owner()).createObject();
+    as_object* o = createObject(getGlobal(owner()));
     o->init_member("code",  info.first, flags);
     o->init_member("level", info.second, flags);
 
@@ -273,19 +283,16 @@ NetStream_as::getStatusObject(StatusCode code)
 void
 NetStream_as::setAudioController(DisplayObject* ch)
 {
-    _audioController.reset(new CharacterProxy(ch));
+    _audioController.reset(new CharacterProxy(ch, getRoot(owner())));
 }
 
-#ifdef GNASH_USE_GC
 void
 NetStream_as::markReachableResources() const
 {
     if (_netCon) _netCon->setReachable();
-    if (_statusHandler) _statusHandler->setReachable();
     if (_audioController) _audioController->setReachable();
     if (_invalidatedVideoCharacter) _invalidatedVideoCharacter->setReachable();
 }
-#endif // GNASH_USE_GC
 
 void
 NetStream_as::stopAdvanceTimer()
@@ -299,28 +306,19 @@ NetStream_as::startAdvanceTimer()
     getRoot(owner()).addAdvanceCallback(this);
 }
 
-
-// AS-volume adjustment
-void adjust_volume(boost::int16_t* data, int size, int volume)
-{
-    for (int i=0; i < size*0.5; i++) {
-        data[i] = data[i] * volume/100;
-    }
-}
-
-
 NetStream_as::~NetStream_as()
 {
-    // close will also detach from sound handler
-    close();
+    // TODO: have this done by ~BufferedAudioQueue ?
+    _audioStreamer.cleanAudioQueue();
+    _audioStreamer.detachAuxStreamer();
 }
 
 
 void NetStream_as::pause(PauseMode mode)
 {
     log_debug("::pause(%d) called ", mode);
-    switch ( mode )
-    {
+    switch (mode) {
+
         case pauseModeToggle:
             if (_playHead.getState() == PlayHead::PLAY_PAUSED) {
                 unpausePlayback();
@@ -339,7 +337,8 @@ void NetStream_as::pause(PauseMode mode)
 
 }
 
-void NetStream_as::close()
+void
+NetStream_as::close()
 {
 
     // Delete any samples in the audio queue.
@@ -392,13 +391,11 @@ NetStream_as::play(const std::string& c_url)
     url = c_url;
 
     // Remove any "mp3:" prefix. Maybe should use this to mark as audio-only
-    if (url.compare(0, 4, std::string("mp3:")) == 0)
-    {
+    if (url.compare(0, 4, std::string("mp3:")) == 0) {
         url = url.substr(4);
     }
 
-    if (url.empty())
-    {
+    if (url.empty()) {
         log_error("Couldn't load URL %s", c_url);
         return;
     }
@@ -406,13 +403,12 @@ NetStream_as::play(const std::string& c_url)
     // Reset any previously active playback
     close();
 
-    log_security( _("Connecting to movie: %s"), url );
+    log_security(_("Connecting to movie: %s"), url);
 
     _inputStream = _netCon->getStream(url); 
 
     // We need to start playback
-    if (!startPlayback())
-    {
+    if (!startPlayback()) {
         log_error("NetStream.play(%s): failed starting playback", c_url);
         return;
     }
@@ -427,9 +423,9 @@ void
 NetStream_as::initVideoDecoder(const media::VideoInfo& info)
 {
     // Caller should check these:
-    assert ( _mediaHandler ); 
-    assert ( !_videoInfoKnown );
-    assert ( !_videoDecoder.get() );
+    assert (_mediaHandler); 
+    assert (!_videoInfoKnown);
+    assert (!_videoDecoder.get());
 
     _videoInfoKnown = true; 
 
@@ -440,12 +436,13 @@ NetStream_as::initVideoDecoder(const media::VideoInfo& info)
                 "video consumer");
         _playHead.setVideoConsumerAvailable();
     }
-    catch (MediaException& e) {
+    catch (const MediaException& e) {
         log_error("NetStream: Could not create Video decoder: %s", e.what());
 
         // This is important enough to let the user know.
         movie_root& m = getRoot(owner());
-        m.errorInterface(e.what());
+        m.callInterface(HostMessage(HostMessage::NOTIFY_ERROR,
+                std::string(e.what())));
     }
 
 }
@@ -468,12 +465,14 @@ NetStream_as::initAudioDecoder(const media::AudioInfo& info)
                 "audio consumer");
         _playHead.setAudioConsumerAvailable();
     }
-    catch (MediaException& e) {
-        log_error("Could not create Audio decoder: %s", e.what());
+    catch (const MediaException& e) {
+        const std::string& err = e.what();
+
+        log_error("Could not create Audio decoder: %s", err);
 
         // This is important enough to let the user know.
         movie_root& m = getRoot(owner());
-        m.errorInterface(e.what());
+        m.callInterface(HostMessage(HostMessage::NOTIFY_ERROR, err));
     }
 
 }
@@ -494,8 +493,7 @@ NetStream_as::startPlayback()
     // status notifications to be received (e.g. streamNotFound).
     startAdvanceTimer();
 
-    if ( ! _inputStream.get() )
-    {
+    if (!_inputStream.get()) {
         log_error(_("Gnash could not get stream '%s' from NetConnection"),
                 url);
         setStatus(streamNotFound);
@@ -505,8 +503,7 @@ NetStream_as::startPlayback()
     assert(_inputStream->tell() == static_cast<std::streampos>(0));
     inputPos = 0;
 
-    if (!_mediaHandler)
-    {
+    if (!_mediaHandler) {
         LOG_ONCE( log_error(_("No Media handler registered, can't "
             "parse NetStream input")) );
         return false;
@@ -514,8 +511,7 @@ NetStream_as::startPlayback()
     m_parser = _mediaHandler->createMediaParser(_inputStream);
     assert(!_inputStream.get());
 
-    if ( ! m_parser.get() )
-    {
+    if (!m_parser.get()) {
         log_error(_("Unable to create parser for NetStream input"));
         // not necessarily correct, the stream might have been found...
         setStatus(streamNotFound);
@@ -556,24 +552,23 @@ NetStream_as::startPlayback()
 }
 
 
-std::auto_ptr<GnashImage> 
+std::auto_ptr<image::GnashImage> 
 NetStream_as::getDecodedVideoFrame(boost::uint32_t ts)
 {
     assert(_videoDecoder.get()); 
 
-    std::auto_ptr<GnashImage> video;
+    std::auto_ptr<image::GnashImage> video;
 
     assert(m_parser.get());
-    if ( ! m_parser.get() )
-    {
+    if (!m_parser.get()) {
         log_error("getDecodedVideoFrame: no parser available");
         return video; 
     }
 
     boost::uint64_t nextTimestamp;
     bool parsingComplete = m_parser->parsingCompleted();
-    if ( ! m_parser->nextVideoFrameTimestamp(nextTimestamp) )
-    {
+    if (!m_parser->nextVideoFrameTimestamp(nextTimestamp)) {
+
 #ifdef GNASH_DEBUG_DECODING
         log_debug("getDecodedVideoFrame(%d): "
             "no more video frames in input "
@@ -582,8 +577,8 @@ NetStream_as::getDecodedVideoFrame(boost::uint32_t ts)
             ts, parsingComplete);
 #endif 
 
-        if ( parsingComplete )
-        {
+        if (parsingComplete && m_parser->isBufferEmpty()) {
+
             decodingStatus(DEC_STOPPED);
 #ifdef GNASH_DEBUG_STATUS
             log_debug("getDecodedVideoFrame setting playStop status "
@@ -595,8 +590,7 @@ NetStream_as::getDecodedVideoFrame(boost::uint32_t ts)
         return video;
     }
 
-    if ( nextTimestamp > ts )
-    {
+    if (nextTimestamp > ts) {
 #ifdef GNASH_DEBUG_DECODING
         log_debug("%p.getDecodedVideoFrame(%d): next video frame is in "
                 "the future (%d)", this, ts, nextTimestamp);
@@ -606,173 +600,157 @@ NetStream_as::getDecodedVideoFrame(boost::uint32_t ts)
     }
 
     // Loop until a good frame is found
-        while ( 1 )
-        {
-            video = decodeNextVideoFrame();
-            if ( ! video.get() )
-            {
-                log_error("nextVideoFrameTimestamp returned true (%d), "
-                    "but decodeNextVideoFrame returned null, "
-                    "I don't think this should ever happen", nextTimestamp);
-                break;
-            }
-
-            if ( ! m_parser->nextVideoFrameTimestamp(nextTimestamp) )
-            {
-                // the one we decoded was the last one
-#ifdef GNASH_DEBUG_DECODING
-                log_debug("%p.getDecodedVideoFrame(%d): last video frame decoded "
-                    "(should set playback status to STOP?)", this, ts);
-#endif 
-                break;
-            }
-            if ( nextTimestamp > ts )
-            {
-                // the next one is in the future, we'll return this one.
-#ifdef GNASH_DEBUG_DECODING
-                log_debug("%p.getDecodedVideoFrame(%d): "
-                    "next video frame is in the future, "
-                    "we'll return this one",
-                    this, ts);
-#endif 
-                break; 
-            }
+    while (1) {
+        video = decodeNextVideoFrame();
+        if (!video.get()) {
+            log_error("nextVideoFrameTimestamp returned true (%d), "
+                "but decodeNextVideoFrame returned null, "
+                "I don't think this should ever happen", nextTimestamp);
+            break;
         }
 
+        if (!m_parser->nextVideoFrameTimestamp(nextTimestamp)) {
+            // the one we decoded was the last one
+#ifdef GNASH_DEBUG_DECODING
+            log_debug("%p.getDecodedVideoFrame(%d): last video frame decoded "
+                "(should set playback status to STOP?)", this, ts);
+#endif 
+            break;
+        }
+        if (nextTimestamp > ts) {
+            // the next one is in the future, we'll return this one.
+#ifdef GNASH_DEBUG_DECODING
+            log_debug("%p.getDecodedVideoFrame(%d): "
+                "next video frame is in the future, "
+                "we'll return this one",
+                this, ts);
+#endif 
+            break; 
+        }
+    }
+
+    return video;
+}
+
+std::auto_ptr<image::GnashImage> 
+NetStream_as::decodeNextVideoFrame()
+{
+    std::auto_ptr<image::GnashImage> video;
+
+    if (!m_parser.get()) {
+        log_error("decodeNextVideoFrame: no parser available");
+        return video; 
+    }
+
+    std::auto_ptr<media::EncodedVideoFrame> frame = m_parser->nextVideoFrame(); 
+    if (!frame.get()) {
+#ifdef GNASH_DEBUG_DECODING
+        log_debug("%p.decodeNextVideoFrame(): "
+            "no more video frames in input",
+            this);
+#endif 
         return video;
     }
 
-    std::auto_ptr<GnashImage> 
-    NetStream_as::decodeNextVideoFrame()
-    {
-        std::auto_ptr<GnashImage> video;
+    assert(_videoDecoder.get()); 
+    
+    // everything we push, we'll pop too..
+    assert(!_videoDecoder->peek()); 
 
-        if ( ! m_parser.get() )
-        {
-            log_error("decodeNextVideoFrame: no parser available");
-            return video; 
-        }
-
-        std::auto_ptr<media::EncodedVideoFrame> frame = m_parser->nextVideoFrame(); 
-        if ( ! frame.get() )
-        {
-#ifdef GNASH_DEBUG_DECODING
-            log_debug("%p.decodeNextVideoFrame(): "
-                "no more video frames in input",
-                this);
-#endif 
-            return video;
-        }
-
-        assert( _videoDecoder.get() ); 
-        
-        // everything we push, we'll pop too..
-        assert( ! _videoDecoder->peek() ); 
-
-        _videoDecoder->push(*frame);
-        video = _videoDecoder->pop();
-        if ( ! video.get() )
-        {
-            // TODO: tell more about the failure
-            log_error(_("Error decoding encoded video frame in NetStream input"));
-        }
-
-        return video;
+    _videoDecoder->push(*frame);
+    video = _videoDecoder->pop();
+    if (!video.get()) {
+        // TODO: tell more about the failure
+        log_error(_("Error decoding encoded video frame in NetStream input"));
     }
 
-    BufferedAudioStreamer::CursoredBuffer*
-    NetStream_as::decodeNextAudioFrame()
-    {
-        assert ( m_parser.get() );
+    return video;
+}
 
-        std::auto_ptr<media::EncodedAudioFrame> frame = m_parser->nextAudioFrame(); 
-        if ( ! frame.get() )
-        {
+BufferedAudioStreamer::CursoredBuffer*
+NetStream_as::decodeNextAudioFrame()
+{
+    assert (m_parser.get());
+
+    std::auto_ptr<media::EncodedAudioFrame> frame = m_parser->nextAudioFrame(); 
+    if (!frame.get()) {
 #ifdef GNASH_DEBUG_DECODING
-            log_debug("%p.decodeNextAudioFrame: "
-                "no more video frames in input",
-                this);
+        log_debug("%p.decodeNextAudioFrame: "
+            "no more video frames in input",
+            this);
 #endif
-            return 0;
-        }
+        return 0;
+    }
 
-        // TODO: make the buffer cursored later ?
-        BufferedAudioStreamer::CursoredBuffer* raw =
-            new BufferedAudioStreamer::CursoredBuffer();
-        raw->m_data = _audioDecoder->decode(*frame, raw->m_size);
+    // TODO: make the buffer cursored later ?
+    BufferedAudioStreamer::CursoredBuffer* raw =
+        new BufferedAudioStreamer::CursoredBuffer();
+    raw->m_data = _audioDecoder->decode(*frame, raw->m_size);
 
-        // TODO: let the sound_handler do this .. sounds cleaner
-        if ( _audioController ) 
-        {
-            DisplayObject* ch = _audioController->get();
-            if ( ch )
-            {
-                int vol = ch->getWorldVolume();
-                if ( vol != 100 )
-                {
-                    // NOTE: adjust_volume assumes samples 
-                    // are 16 bits in size, and signed.
-                    // Size is still given in bytes..
-                    adjust_volume(reinterpret_cast<boost::int16_t*>(raw->m_data),
-                            raw->m_size, vol);
-                }
+    // TODO: let the sound_handler do this .. sounds cleaner
+    if (_audioController) {
+        DisplayObject* ch = _audioController->get();
+        if (ch) {
+            const int vol = ch->getWorldVolume();
+            if (vol != 100) {
+                // NOTE: adjust_volume assumes samples 
+                // are 16 bits in size, and signed.
+                // Size is still given in bytes..
+                adjustVolume(reinterpret_cast<boost::int16_t*>(raw->m_data),
+                        raw->m_size / 2, vol);
             }
         }
+    }
 
 #ifdef GNASH_DEBUG_DECODING
-        log_debug("NetStream_as::decodeNextAudioFrame: "
-            "%d bytes of encoded audio "
-            "decoded to %d bytes",
-            frame->dataSize,
-            raw->m_size);
+    log_debug("NetStream_as::decodeNextAudioFrame: "
+        "%d bytes of encoded audio "
+        "decoded to %d bytes",
+        frame->dataSize,
+        raw->m_size);
 #endif 
 
-        raw->m_ptr = raw->m_data;
+    raw->m_ptr = raw->m_data;
 
-        return raw;
+    return raw;
+}
+
+void
+NetStream_as::seek(boost::uint32_t posSeconds)
+{
+    GNASH_REPORT_FUNCTION;
+
+    // We'll mess with the input here
+    if ( ! m_parser.get() )
+    {
+        log_debug("NetStream_as::seek(%d): no parser, no party", posSeconds);
+        return;
     }
 
-    bool NetStream_as::decodeMediaFrame()
+    // Don't ask me why, but NetStream_as::seek() takes seconds...
+    boost::uint32_t pos = posSeconds*1000;
+
+    // We'll pause the clock source and mark decoders as buffering.
+    // In this way, next advance won't find the source time to 
+    // be a lot of time behind and chances to get audio buffer
+    // overruns will reduce.
+    // ::advance will resume the playbackClock if DEC_BUFFERING...
+    //
+    _playbackClock->pause();
+
+    // Seek to new position
+    boost::uint32_t newpos = pos;
+    if ( ! m_parser->seek(newpos) )
     {
-        return false;
-    }
-
-    void
-    NetStream_as::seek(boost::uint32_t posSeconds)
-    {
-        GNASH_REPORT_FUNCTION;
-
-        // We'll mess with the input here
-        if ( ! m_parser.get() )
-        {
-            log_debug("NetStream_as::seek(%d): no parser, no party", posSeconds);
-            return;
-        }
-
-        // Don't ask me why, but NetStream_as::seek() takes seconds...
-        boost::uint32_t pos = posSeconds*1000;
-
-        // We'll pause the clock source and mark decoders as buffering.
-        // In this way, next advance won't find the source time to 
-        // be a lot of time behind and chances to get audio buffer
-        // overruns will reduce.
-        // ::advance will resume the playbackClock if DEC_BUFFERING...
-        //
-        _playbackClock->pause();
-
-        // Seek to new position
-        boost::uint32_t newpos = pos;
-        if ( ! m_parser->seek(newpos) )
-        {
 #ifdef GNASH_DEBUG_STATUS
-            log_debug("Setting invalidTime status");
+        log_debug("Setting invalidTime status");
 #endif
-            setStatus(invalidTime);
-            // we won't be *BUFFERING*, so resume now
-            _playbackClock->resume(); 
-            return;
-        }
-        log_debug("m_parser->seek(%d) returned %d", pos, newpos);
+        setStatus(invalidTime);
+        // we won't be *BUFFERING*, so resume now
+        _playbackClock->resume(); 
+        return;
+    }
+    log_debug("m_parser->seek(%d) returned %d", pos, newpos);
 
         // cleanup audio queue, so won't be consumed while seeking
     _audioStreamer.cleanAudioQueue();
@@ -802,7 +780,7 @@ NetStream_as::parseNextChunk()
 void
 NetStream_as::refreshAudioBuffer()
 {
-    assert ( m_parser.get() );
+    assert (m_parser.get());
 
 #ifdef GNASH_DEBUG_DECODING
     // bufferLength() would lock the mutex (which we already hold),
@@ -813,8 +791,7 @@ NetStream_as::refreshAudioBuffer()
         parserTime > playHeadTime ? parserTime-playHeadTime : 0;
 #endif
 
-    if ( _playHead.getState() == PlayHead::PLAY_PAUSED )
-    {
+    if (_playHead.getState() == PlayHead::PLAY_PAUSED) {
 #ifdef GNASH_DEBUG_DECODING
         log_debug("%p.refreshAudioBuffer: doing nothing as playhead "
                 "is paused - bufferLength=%d/%d", this, bufferLength(),
@@ -823,8 +800,7 @@ NetStream_as::refreshAudioBuffer()
         return;
     }
 
-    if ( _playHead.isAudioConsumed() ) 
-    {
+    if (_playHead.isAudioConsumed()) {
 #ifdef GNASH_DEBUG_DECODING
         log_debug("%p.refreshAudioBuffer: doing nothing "
             "as current position was already decoded - "
@@ -853,8 +829,8 @@ NetStream_as::pushDecodedAudioFrames(boost::uint32_t ts)
 {
     assert(m_parser.get());
 
-    if ( ! _audioDecoder.get() )
-    {
+    if (!_audioDecoder.get()) {
+
         // There are 3 possible reasons for _audioDecoder to not be here:
         //
         // 1: The stream does contain audio but we were unable to find
@@ -863,10 +839,9 @@ NetStream_as::pushDecodedAudioFrames(boost::uint32_t ts)
         // 2: The stream does contain audio but we didn't try to construct
         //    a decoder for it yet.
         //
-        // 3: The stream does NOT contain audio yet
+        // 3: The stream does not contain audio yet
 
-        if ( _audioInfoKnown )
-        {
+        if (_audioInfoKnown) {
             // case 1: we saw the audio info already,
             //         but couldn't construct a decoder
 
@@ -877,8 +852,7 @@ NetStream_as::pushDecodedAudioFrames(boost::uint32_t ts)
         }
 
         media::AudioInfo* audioInfo = m_parser->getAudioInfo();
-        if ( ! audioInfo )
-        {
+        if (!audioInfo) {
             // case 3: no audio found yet
             return;
         }
@@ -889,8 +863,7 @@ NetStream_as::pushDecodedAudioFrames(boost::uint32_t ts)
         initAudioDecoder(*audioInfo);
 
         // Don't go ahead if audio decoder construction failed
-        if ( ! _audioDecoder.get() )
-        {
+        if (!_audioDecoder.get()) {
             // TODO: we should still flush any existing Audio frame
             //       in the encoded queue...
             //       (or rely on next call)
@@ -903,8 +876,7 @@ NetStream_as::pushDecodedAudioFrames(boost::uint32_t ts)
     bool consumed = false;
 
     boost::uint64_t nextTimestamp;
-    while ( 1 )
-    {
+    while (1) {
 
         // FIXME: use services of BufferedAudioStreamer for this
         boost::mutex::scoped_lock lock(_audioStreamer._audioQueueMutex);
@@ -964,8 +936,8 @@ NetStream_as::pushDecodedAudioFrames(boost::uint32_t ts)
 
         const unsigned int bufferLimit = 20;
         unsigned int bufferSize = _audioStreamer._audioQueue.size();
-        if ( bufferSize > bufferLimit )
-        {
+        if (bufferSize > bufferLimit) {
+
             // we won't buffer more then 'bufferLimit' frames in the queue
             // to avoid ending up with a huge queue which will take some
             // time before being consumed by audio mixer, but still marked
@@ -992,8 +964,7 @@ NetStream_as::pushDecodedAudioFrames(boost::uint32_t ts)
         lock.unlock();
 
         bool parsingComplete = m_parser->parsingCompleted();
-        if ( ! m_parser->nextAudioFrameTimestamp(nextTimestamp) )
-        {
+        if (!m_parser->nextAudioFrameTimestamp(nextTimestamp)) {
 #ifdef GNASH_DEBUG_DECODING
             log_debug("%p.pushDecodedAudioFrames(%d): "
                 "no more audio frames in input "
@@ -1001,23 +972,23 @@ NetStream_as::pushDecodedAudioFrames(boost::uint32_t ts)
                 this, ts, parsingComplete);
 #endif 
 
-            if ( parsingComplete )
-            {
+            if (parsingComplete) {
                 consumed = true;
-                decodingStatus(DEC_STOPPED);
+                if (m_parser->isBufferEmpty()) {
+                    decodingStatus(DEC_STOPPED);
 #ifdef GNASH_DEBUG_STATUS
-                log_debug("pushDecodedAudioFrames setting playStop status "
-                        "(parsing complete and nextAudioFrameTimestamp "
-                        "returned false)");
+                    log_debug("pushDecodedAudioFrames setting playStop status "
+                              "(parsing complete and nextAudioFrameTimestamp "
+                              "returned false)");
 #endif
-                setStatus(playStop);
+                    setStatus(playStop);
+                }
             }
 
             break;
         }
 
-        if ( nextTimestamp > ts )
-        {
+        if (nextTimestamp > ts) {
 #ifdef GNASH_DEBUG_DECODING
             log_debug("%p.pushDecodedAudioFrames(%d): "
                 "next audio frame is in the future (%d)",
@@ -1030,8 +1001,7 @@ NetStream_as::pushDecodedAudioFrames(boost::uint32_t ts)
         }
 
         BufferedAudioStreamer::CursoredBuffer* audio = decodeNextAudioFrame();
-        if ( ! audio )
-        {
+        if (!audio) {
             // Well, it *could* happen, why not ?
             log_error("nextAudioFrameTimestamp returned true (%d), "
                 "but decodeNextAudioFrame returned null, "
@@ -1039,10 +1009,9 @@ NetStream_as::pushDecodedAudioFrames(boost::uint32_t ts)
             break;
         }
 
-        if ( ! audio->m_size )
-        {
+        if (!audio->m_size) {
             // Don't bother pushing an empty frame
-            // to the audio queue...
+            // to the audio Queue...
             log_debug("pushDecodedAudioFrames(%d): Decoded audio frame "
                     "contains no samples");
             delete audio;
@@ -1063,8 +1032,7 @@ NetStream_as::pushDecodedAudioFrames(boost::uint32_t ts)
 
     // If we consumed audio of current position, feel free to advance
     // if needed, resuming playbackClock too...
-    if ( consumed )
-    {
+    if (consumed) {
         // resume the playback clock, assuming the
         // only reason for it to be paused is we
         // put in pause mode due to buffer overrun
@@ -1085,10 +1053,9 @@ NetStream_as::pushDecodedAudioFrames(boost::uint32_t ts)
 void
 NetStream_as::refreshVideoFrame(bool alsoIfPaused)
 {
-    assert ( m_parser.get() );
+    assert (m_parser.get());
 
-    if ( ! _videoDecoder.get() )
-    {
+    if (!_videoDecoder.get()) {
         // There are 3 possible reasons for _videoDecoder to not be here:
         //
         // 1: The stream does contain video but we were unable to find
@@ -1097,11 +1064,10 @@ NetStream_as::refreshVideoFrame(bool alsoIfPaused)
         // 2: The stream does contain video but we didn't try to construct
         //    a decoder for it yet.
         //
-        // 3: The stream does NOT contain video yet
+        // 3: The stream does not contain video yet
         //
 
-        if ( _videoInfoKnown )
-        {
+        if (_videoInfoKnown) {
             // case 1: we saw the video info already,
             //         but couldn't construct a decoder
 
@@ -1112,8 +1078,7 @@ NetStream_as::refreshVideoFrame(bool alsoIfPaused)
         }
 
         media::VideoInfo* videoInfo = m_parser->getVideoInfo();
-        if ( ! videoInfo )
-        {
+        if (!videoInfo) {
             // case 3: no video found yet
             return;
         }
@@ -1124,8 +1089,7 @@ NetStream_as::refreshVideoFrame(bool alsoIfPaused)
         initVideoDecoder(*videoInfo);
 
         // Don't go ahead if video decoder construction failed
-        if ( ! _videoDecoder.get() )
-        {
+        if (!_videoDecoder.get()) {
             // TODO: we should still flush any existing Video frame
             //       in the encoded queue...
             //       (or rely on next call)
@@ -1169,7 +1133,7 @@ NetStream_as::refreshVideoFrame(bool alsoIfPaused)
 #endif 
 
     // Get next decoded video frame from parser, will have the lowest timestamp
-    std::auto_ptr<GnashImage> video = getDecodedVideoFrame(curPos);
+    std::auto_ptr<image::GnashImage> video = getDecodedVideoFrame(curPos);
 
     // to be decoded or we're out of data
     if (!video.get())
@@ -1551,20 +1515,18 @@ BufferedAudioStreamer::fetch(boost::int16_t* samples, unsigned int nSamples, boo
             break;
         }
 
-        CursoredBuffer* samples = _audioQueue.front();
+        CursoredBuffer& samples = _audioQueue.front();
 
-        assert( ! (samples->m_size%2) ); 
-        int n = std::min<int>(samples->m_size, len);
-        std::copy(samples->m_ptr, samples->m_ptr+n, stream);
+        assert( ! (samples.m_size%2) ); 
+        int n = std::min<int>(samples.m_size, len);
+        std::copy(samples.m_ptr, samples.m_ptr+n, stream);
 
         stream += n;
-        samples->m_ptr += n;
-        samples->m_size -= n;
+        samples.m_ptr += n;
+        samples.m_size -= n;
         len -= n;
 
-        if (samples->m_size == 0)
-        {
-            delete samples;
+        if (samples.m_size == 0) {
             _audioQueue.pop_front();
         }
 
@@ -1601,9 +1563,6 @@ void
 BufferedAudioStreamer::cleanAudioQueue()
 {
     boost::mutex::scoped_lock lock(_audioQueueMutex);
-
-    deleteChecked(_audioQueue.begin(), _audioQueue.end());
-
     _audioQueue.clear();
 }
 
@@ -1620,7 +1579,7 @@ netstream_new(const fn_call& fn)
     if (fn.nargs) {
 
         NetConnection_as* nc;
-        if (isNativeType(fn.arg(0).to_object(getGlobal(fn)), nc)) {
+        if (isNativeType(toObject(fn.arg(0), getVM(fn)), nc)) {
             ns->setNetCon(nc);
         }
         else {
@@ -1654,10 +1613,9 @@ netstream_pause(const fn_call& fn)
     
     // mode: -1 ==> toogle, 0==> pause, 1==> play
     NetStream_as::PauseMode mode = NetStream_as::pauseModeToggle;
-    if (fn.nargs > 0)
-    {
-        mode = fn.arg(0).to_bool() ? NetStream_as::pauseModePause :
-                                     NetStream_as::pauseModeUnPause;
+    if (fn.nargs > 0) {
+        mode = toBool(fn.arg(0), getVM(fn)) ? NetStream_as::pauseModePause :
+                                              NetStream_as::pauseModeUnPause;
     }
     
     // Toggle pause mode
@@ -1670,19 +1628,17 @@ netstream_play(const fn_call& fn)
 {
     NetStream_as* ns = ensure<ThisIsNative<NetStream_as> >(fn);
 
-    if (!fn.nargs)
-    {
+    if (!fn.nargs) {
         IF_VERBOSE_ASCODING_ERRORS(
-        log_aserror(_("NetStream_as play needs args"));
+            log_aserror(_("NetStream_as play needs args"));
         );
         return as_value();
     }
 
-    if ( ! ns->isConnected() )
-    {
+    if (!ns->isConnected()) {
         IF_VERBOSE_ASCODING_ERRORS(
-        log_aserror(_("NetStream.play(%s): stream is not connected"),
-            fn.arg(0));
+            log_aserror(_("NetStream.play(%s): stream is not connected"),
+                fn.arg(0));
         );
         return as_value();
     }
@@ -1697,9 +1653,8 @@ netstream_seek(const fn_call& fn)
 {
     NetStream_as* ns = ensure<ThisIsNative<NetStream_as> >(fn);
     boost::uint32_t time = 0;
-    if (fn.nargs > 0)
-    {
-        time = static_cast<boost::uint32_t>(fn.arg(0).to_number());
+    if (fn.nargs > 0) {
+        time = static_cast<boost::uint32_t>(toNumber(fn.arg(0), getVM(fn)));
     }
     ns->seek(time);
 
@@ -1709,22 +1664,18 @@ netstream_seek(const fn_call& fn)
 as_value
 netstream_setbuffertime(const fn_call& fn)
 {
-
-    //GNASH_REPORT_FUNCTION;
-
     NetStream_as* ns = ensure<ThisIsNative<NetStream_as> >(fn);
 
     // TODO: should we do anything if given no args ?
     //       are we sure setting bufferTime to 0 is what we have to do ?
     double time = 0;
-    if (fn.nargs > 0)
-    {
-        time = fn.arg(0).to_number();
+    if (fn.nargs > 0) {
+        time = toNumber(fn.arg(0), getVM(fn));
     }
 
     // TODO: don't allow a limit < 100 
 
-    ns->setBufferTime(boost::uint32_t(time*1000));
+    ns->setBufferTime(boost::uint32_t(time * 1000));
 
     return as_value();
 }
@@ -1812,8 +1763,7 @@ as_value
 netstream_bytesloaded(const fn_call& fn)
 {
     NetStream_as* ns = ensure<ThisIsNative<NetStream_as> >(fn);
-    if ( ! ns->isConnected() )
-    {
+    if (!ns->isConnected()) {
         return as_value();
     }
     long ret = ns->bytesLoaded();
@@ -1825,8 +1775,7 @@ as_value
 netstream_bytestotal(const fn_call& fn)
 {
     NetStream_as* ns = ensure<ThisIsNative<NetStream_as> >(fn);
-    if ( ! ns->isConnected() )
-    {
+    if (!ns->isConnected()) {
         return as_value();
     }
     long ret = ns->bytesTotal();
@@ -1838,8 +1787,7 @@ as_value
 netstream_currentFPS(const fn_call& fn)
 {
     NetStream_as* ns = ensure<ThisIsNative<NetStream_as> >(fn);
-    if ( ! ns->isConnected() )
-    {
+    if (!ns->isConnected()) {
         return as_value();
     }
 
@@ -1919,41 +1867,41 @@ attachNetStreamInterface(as_object& o)
 void
 executeTag(const SimpleBuffer& _buffer, as_object& thisPtr)
 {
-
 	const boost::uint8_t* ptr = _buffer.data();
 	const boost::uint8_t* endptr = ptr + _buffer.size();
 
-	if (endptr - ptr < 2) {
-		log_error("Premature end of AMF in NetStream metatag");
-		return;
-	}
-	const boost::uint16_t length = ntohs((*(boost::uint16_t *)ptr) & 0xffff);
-	ptr += 2;
+    std::string funcName;
 
-	if (endptr - ptr < length) {
-		log_error("Premature end of AMF in NetStream metatag");
-		return;
-	}
+    try {
+        funcName = amf::readString(ptr, endptr);
+    }
+    catch (const amf::AMFException&) {
+        log_error("Invalid AMF data in FLV tag");
+        return;
+    }
 
-	const std::string funcName(reinterpret_cast<const char*>(ptr), length); 
-	ptr += length;
-
-	log_debug("funcName: %s", funcName);
-
-	string_table& st = getStringTable(thisPtr);
-	string_table::key funcKey = st.find(funcName);
+	VM& vm = getVM(thisPtr);
+	const ObjectURI& funcKey = getURI(vm, funcName);
 
     amf::Reader rd(ptr, endptr, getGlobal(thisPtr));
 
 	as_value arg;
 	if (!rd(arg)) {
-		log_error("Could not convert FLV metatag to as_value, but will try "
-                "passing it anyway. It's an %s", arg);
+		log_error("Could not convert FLV metatag to as_value, passing "
+                "undefined");
 	}
 
 	log_debug("Calling %s(%s)", funcName, arg);
 	callMethod(&thisPtr, funcKey, arg);
 }
 
+// AS-volume adjustment
+void
+adjustVolume(boost::int16_t* data, size_t size, int volume)
+{
+    std::transform(data, data + size, data,
+            boost::bind(std::multiplies<double>(), volume / 100.0, _1));
+}
+
 } // anonymous namespace
 } // gnash namespace
