@@ -1,7 +1,7 @@
 // string.cpp:  ActionScript "String" class, for Gnash.
 //
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,11 +17,18 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
+#include "String_as.h"
+
+#include <boost/algorithm/string/case_conv.hpp>
+#include <algorithm>
+#include <locale>
+#include <stdexcept>
+
+#include "SWFCtype.h"
 #include "smart_ptr.h"
 #include "fn_call.h"
 #include "Global_as.h"
 #include "as_object.h"
-#include "builtin_function.h" // need builtin_function
 #include "NativeFunction.h" 
 #include "log.h"
 #include "as_value.h"
@@ -30,15 +37,9 @@
 #include "VM.h" 
 #include "namedStrings.h"
 #include "utf8.h"
-#include "String_as.h"
 #include "GnashNumeric.h"
 #include "Global_as.h"
 
-#include <boost/algorithm/string/case_conv.hpp>
-#include <algorithm>
-#include <locale>
-#include <stdexcept>
-
 namespace gnash {
 
 // Forward declarations
@@ -111,7 +112,7 @@ string_class_init(as_object& where, const ObjectURI& uri)
     VM& vm = getVM(where);
     Global_as& gl = getGlobal(where);
 
-    as_object* proto = gl.createObject();
+    as_object* proto = createObject(gl);
     as_object* cl = vm.getNative(251, 0);
     cl->init_member(NSV::PROP_PROTOTYPE, proto);
     proto->init_member(NSV::PROP_CONSTRUCTOR, cl);
@@ -178,13 +179,13 @@ string_slice(const fn_call& fn)
 
     if (!checkArgs(fn, 1, 2, "String.slice()")) return as_value();
 
-    size_t start = validIndex(wstr, toInt(fn.arg(0)));
+    size_t start = validIndex(wstr, toInt(fn.arg(0), getVM(fn)));
 
     size_t end = wstr.length();
 
     if (fn.nargs >= 2)
     {
-        end = validIndex(wstr, toInt(fn.arg(1)));
+        end = validIndex(wstr, toInt(fn.arg(1), getVM(fn)));
 
     } 
 
@@ -260,7 +261,7 @@ string_split(const fn_call& fn)
         // SWF5
         if (fn.nargs > 1 && !fn.arg(1).is_undefined())
         {
-            int limit = toInt(fn.arg(1));
+            int limit = toInt(fn.arg(1), getVM(fn));
             if (limit < 1)
             {
                 // Return empty array.
@@ -293,7 +294,7 @@ string_split(const fn_call& fn)
         // the delimiter is defined.
         if (fn.nargs > 1 && !fn.arg(1).is_undefined())
         {
-            int limit = toInt(fn.arg(1));
+            int limit = toInt(fn.arg(1), getVM(fn));
             if (limit < 1) {
                 // Return empty array if 
                 return as_value(array);
@@ -351,7 +352,7 @@ string_lastIndexOf(const fn_call& fn)
     int start = str.size();
 
     if (fn.nargs >= 2) {
-        start = toInt(fn.arg(1));
+        start = toInt(fn.arg(1), getVM(fn));
     }
     
     if (start < 0) {
@@ -384,13 +385,13 @@ string_substr(const fn_call& fn)
 
     if (!checkArgs(fn, 1, 2, "String.substr()")) return as_value(str);
     
-    int start = validIndex(wstr, toInt(fn.arg(0)));
+    int start = validIndex(wstr, toInt(fn.arg(0), getVM(fn)));
 
     int num = wstr.length();
 
     if (fn.nargs >= 2 && !fn.arg(1).is_undefined())
     {
-        num = toInt(fn.arg(1));
+        num = toInt(fn.arg(1), getVM(fn));
         if ( num < 0 )
         {
             if ( -num <= start ) num = 0;
@@ -424,7 +425,7 @@ string_substring(const fn_call& fn)
 
     const as_value& s = fn.arg(0);
 
-    int start = toInt(s);
+    int start = toInt(s, getVM(fn));
     int end = wstr.size();
 
     if (s.is_undefined() || start < 0) {
@@ -436,7 +437,7 @@ string_substring(const fn_call& fn)
     }
 
     if (fn.nargs >= 2 && !fn.arg(1).is_undefined()) {
-        int num = toInt(fn.arg(1));
+        int num = toInt(fn.arg(1), getVM(fn));
 
         if (num < 0) {
             num = 0;
@@ -486,21 +487,19 @@ string_indexOf(const fn_call& fn)
     if (fn.nargs >= 2)
     {
         const as_value& saval = fn.arg(1); // start arg val
-        int start_arg = toInt(saval);
-        if ( start_arg > 0 ) start = (size_t) start_arg;
-    else
-    {
-        IF_VERBOSE_ASCODING_ERRORS(
-        if ( start_arg < 0 )
-        {
-            log_aserror("String.indexOf(%s, %s): second argument casts to invalid offset (%d)",
-                tfarg, saval, start_arg);
+        int start_arg = toInt(saval, getVM(fn));
+        if (start_arg > 0) start = (size_t) start_arg;
+        else {
+            IF_VERBOSE_ASCODING_ERRORS(
+                if (start_arg < 0) {
+                    log_aserror("String.indexOf(%s, %s): second argument casts "
+                        "to invalid offset (%d)", tfarg, saval, start_arg);
+                }
+            );
         }
-        );
-    }
     }
 
-    size_t pos = wstr.find(toFind, start);
+    const size_t pos = wstr.find(toFind, start);
 
     if (pos == std::wstring::npos) {
         return as_value(-1);
@@ -524,11 +523,11 @@ string_fromCharCode(const fn_call& fn)
         for (unsigned int i = 0; i < fn.nargs; i++)
         {
             // Maximum 65535, as with all DisplayObject codes.
-            boost::uint16_t c = static_cast<boost::uint16_t>(toInt(fn.arg(i)));
+            const boost::uint16_t c = 
+                static_cast<boost::uint16_t>(toInt(fn.arg(i), getVM(fn)));
             
             // If more than 255, push 'overflow' byte.
-            if (c > 255)
-            {
+            if (c > 255) {
                 str.push_back(static_cast<unsigned char>(c >> 8));
             }
 
@@ -544,7 +543,8 @@ string_fromCharCode(const fn_call& fn)
 
     for (unsigned int i = 0; i < fn.nargs; i++)
     {
-        boost::uint16_t c = static_cast<boost::uint16_t>(toInt(fn.arg(i)));
+        const boost::uint16_t c = 
+            static_cast<boost::uint16_t>(toInt(fn.arg(i), getVM(fn)));
         if (c == 0) break;
         wstr.push_back(c);
     }
@@ -578,7 +578,7 @@ string_charCodeAt(const fn_call& fn)
         }
     )
 
-    size_t index = static_cast<size_t>(fn.arg(0).to_number());
+    size_t index = static_cast<size_t>(toNumber(fn.arg(0), getVM(fn)));
 
     if (index >= wstr.length()) {
         as_value rv;
@@ -600,7 +600,7 @@ string_charAt(const fn_call& fn)
     if (!checkArgs(fn, 1, 1, "String.charAt()")) return as_value("");
 
     // to_int() makes this safe from overflows.
-    const size_t index = static_cast<size_t>(toInt(fn.arg(0)));
+    const size_t index = static_cast<size_t>(toInt(fn.arg(0), getVM(fn)));
 
     size_t currentIndex = 0;
 
@@ -633,37 +633,9 @@ string_toUpperCase(const fn_call& fn)
 
     std::wstring wstr = utf8::decodeCanonicalString(str, version);
 
-    // If this is the C locale, the conversion will be wrong.
-    // Most other locales are correct. FIXME: get this to
-    // work regardless of user's current settings.
-    std::locale currentLocale;
-    try
-    {
-        currentLocale = std::locale("");
-    }
-    catch (std::runtime_error& e)
-    {
-        currentLocale = std::locale::classic();
-    }
-
-    if (currentLocale == std::locale::classic()) {
-        LOG_ONCE(
-            log_error(_("Your locale probably can't convert non-ascii "
-            "DisplayObjects to upper case. Using a UTF8 locale may fix this."));
-        );
-    }
-
 #if !defined(__HAIKU__) && !defined(__amigaos4__)
-    try {
-        boost::to_upper(wstr, currentLocale);
-    }
-    catch (std::bad_cast&)
-        {
-            // std::use_facet<wchar_t>(currentLocale) called from
-            // boost::to_upper may throw std::bad_cast
-            log_error(_("bad_cast caught"));
-            std::abort();
-        }
+    static const std::locale swfLocale((std::locale()), new SWFCtype());
+    boost::to_upper(wstr, swfLocale);
 #else
     size_t l = wstr.size();
     for (size_t i = 0; i < l; ++i) {
@@ -687,38 +659,9 @@ string_toLowerCase(const fn_call& fn)
 
     std::wstring wstr = utf8::decodeCanonicalString(str, version);
 
-    // If this is the C locale, the conversion will be wrong.
-    // Most other locales are correct. FIXME: get this to
-    // work regardless of user's current settings.
-    std::locale currentLocale;
-    try
-    {
-        currentLocale = std::locale("");
-    }
-    catch (std::runtime_error& e)
-    {
-        currentLocale = std::locale::classic();
-    }
-
-    if (currentLocale == std::locale::classic())
-    {
-        LOG_ONCE( 
-            log_error(_("Your locale probably can't convert non-ascii "
-                "DisplayObjects to lower case. Using a UTF8 locale may fix this"));
-        );
-    }
-
 #if !defined(__HAIKU__) && !defined(__amigaos4__)
-    try {
-        boost::to_lower(wstr, currentLocale);
-    }
-    catch (std::bad_cast&)
-    {
-        // std::use_facet<wchar_t>(currentLocale) called from
-        // boost::to_lower may throw std::bad_cast
-        log_error(_("bad_cast caught"));
-        std::abort();
-    }
+    static const std::locale swfLocale((std::locale()), new SWFCtype());
+    boost::to_lower(wstr, swfLocale);
 #else
     size_t l = wstr.size();
     for (size_t i = 0; i < l; ++i) {
