@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -21,22 +22,33 @@
 #define GNASH_STRING_H
 
 #include <string>
+#include "Relay.h"
 
 namespace gnash {
 
 class as_object;
+struct ObjectURI;
+class Global_as;
 
-// Initialize the global String class
-void string_class_init(as_object& global);
+class String_as : public Relay
+{
 
-/// Return a String instance (possibibly NULL!)
-//
-/// This function will use the native String constructor in SWF5, but
-/// any function registered by user as the _global.String for SWF6 and higher.
-/// In the second case, not finding a proper constructor might result in
-/// returning the NULL object.
-///
-boost::intrusive_ptr<as_object> init_string_instance(const std::string& val);
+public:
+
+    explicit String_as(const std::string& s);
+
+    const std::string& value() {
+        return _string;
+    }
+
+private:
+    std::string _string;
+};
+
+/// Initialize the global String class
+void string_class_init(as_object& global, const ObjectURI& uri);
+
+void registerStringNative(as_object& global);
 
 }
 
