@@ -1,12 +1,12 @@
-// xml.cpp:  XML markup language support, for Gnash.
-// 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
-// 
+// XML_as.cpp:  ActionScript "XMLDocument" class, for Gnash.
+//
+//   Copyright (C) 2009, 2010, 2011 Free Software Foundation, Inc.
+//
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
-// 
+//
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -17,23 +17,22 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-// The XML parsing algorithms are based on swfdec's parsing model.
-
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
 
 #include "log.h"
-#include "as_function.h" // for as_function
+#include "as_function.h" //for as_function
 #include "fn_call.h"
+#include "Global_as.h"
 
 #include "LoadableObject.h"
 #include "XMLNode_as.h"
 #include "XML_as.h"
-#include "builtin_function.h"
+#include "NativeFunction.h"
 #include "VM.h"
 #include "namedStrings.h"
 #include "StringPredicates.h"
+#include "smart_ptr.h" // for boost intrusive_ptr
+#include "GnashException.h" // for ActionException
+#include "Object.h"
 
 #include <string>
 #include <sstream>
@@ -45,85 +44,62 @@
 
 
 namespace gnash {
-  
-//#define DEBUG_MEMORY_ALLOCATION 1
 
-// Define this to enable verbosity of XML loads
-//#define DEBUG_XML_LOADS 1
-
-// Define this to enable verbosity of XML parsing
-//#define DEBUG_XML_PARSE 1
-// Forward declarations.
+// Forward declarations
 namespace {
-    as_object* getXMLInterface();
-    void attachXMLInterface(as_object& o);
-    void attachXMLProperties(as_object& o);
 
     as_value xml_new(const fn_call& fn);
     as_value xml_createElement(const fn_call& fn);
     as_value xml_createTextNode(const fn_call& fn);
-    as_value xml_getBytesLoaded(const fn_call& fn);
-    as_value xml_getBytesTotal(const fn_call& fn);
     as_value xml_parseXML(const fn_call& fn);
-    as_value xml_ondata(const fn_call& fn);
+    as_value xml_onData(const fn_call& fn);
+    as_value xml_onLoad(const fn_call& fn);
     as_value xml_xmlDecl(const fn_call& fn);
     as_value xml_docTypeDecl(const fn_call& fn);
     as_value xml_escape(const fn_call& fn);
+    as_value xml_loaded(const fn_call& fn);
+    as_value xml_status(const fn_call& fn);
 
-    bool textAfterWhitespace(const std::string& xml,
-            std::string::const_iterator& it);
-    bool textMatch(const std::string& xml, std::string::const_iterator& it,
+    typedef XML_as::xml_iterator xml_iterator;
+
+    bool textAfterWhitespace(xml_iterator& it, xml_iterator end);
+    bool textMatch(xml_iterator& it, xml_iterator end,
             const std::string& match, bool advance = true);
-    bool parseNodeWithTerminator(const std::string& xml,
-            std::string::const_iterator& it, const std::string& terminator,
-            std::string& content);
+    bool parseNodeWithTerminator( xml_iterator& it, xml_iterator end,
+            const std::string& terminator, std::string& content);
+	
+	
+    typedef std::map<std::string, std::string> Entities;
+    const Entities& getEntities();
+
+    void attachXMLProperties(as_object& o);
+	void attachXMLInterface(as_object& o);
+
 }
 
-XML_as::XML_as() 
+
+XML_as::XML_as(as_object& object) 
     :
-    as_object(getXMLInterface()),
-    _loaded(-1), 
+    XMLNode_as(getGlobal(object)),
+    _loaded(XML_LOADED_UNDEFINED), 
     _status(XML_OK)
 {
-#ifdef DEBUG_MEMORY_ALLOCATION
-    log_debug(_("Creating XML data at %p"), this);
-#endif
-
-    attachXMLProperties(*this);
+    setObject(&object);
 }
 
-
 // Parse the ASCII XML string into an XMLNode tree
-XML_as::XML_as(const std::string& xml)
+XML_as::XML_as(as_object& object, const std::string& xml)
     :
-    as_object(getXMLInterface()),
-    _loaded(-1), 
+    XMLNode_as(getGlobal(object)),
+    _loaded(XML_LOADED_UNDEFINED), 
     _status(XML_OK)
 {
-#ifdef DEBUG_MEMORY_ALLOCATION
-    log_debug(_("Creating XML data at %p"), this);
-#endif
-
+    setObject(&object);
     parseXML(xml);
 }
 
-const XML_as::Entities&
-XML_as::getEntities()
-{
-
-    static Entities entities = boost::assign::map_list_of
-        ("&amp;", "&")
-        ("&quot;", "\"")
-        ("&lt;", "<")
-        ("&gt;", ">")
-        ("&apos;", "'");
-
-    return entities;
-
-}
-
 void
-XML_as::escape(std::string& text)
+escapeXML(std::string& text)
 {
     const Entities& ent = getEntities();
 
@@ -135,16 +111,18 @@ XML_as::escape(std::string& text)
 }
 
 void
-XML_as::unescape(std::string& text)
+unescapeXML(std::string& text)
 {
     const Entities& ent = getEntities();
 
     for (Entities::const_iterator i = ent.begin(), e = ent.end();
-            i != e; ++i)
-    {
+            i != e; ++i) {
         boost::replace_all(text, i->first, i->second);
     }
 
+    // Additionally, the &nbsp; entity is unescaped (but never escaped).
+    // Note we do this as UTF-8, which is most likely wrong for SWF5.
+    boost::replace_all(text, "&nbsp;", "\xc2\xa0");
 }
 
 void
@@ -156,125 +134,72 @@ XML_as::toString(std::ostream& o, bool encode) const
     XMLNode_as::toString(o, encode);
 }
 
-bool
-XML_as::get_member(string_table::key name, as_value *val,
-        string_table::key nsname)
-{
-    if (name == NSV::PROP_STATUS) 
-    {
-        val->set_int(_status);
-        return true;
-    }
-    else if (name == NSV::PROP_LOADED)
-    {
-        if ( _loaded < 0 ) val->set_undefined();
-        else val->set_bool(_loaded);
-        return true;
-    }
-
-    return as_object::get_member(name, val, nsname);
-}
-
-bool
-XML_as::set_member(string_table::key name, const as_value& val, 
-    string_table::key nsname, bool ifFound)
-{
-    if (name == NSV::PROP_STATUS)
-    {
-        // TODO: this should really be a proper property (see XML.as)
-        if ( ! val.is_number() )
-        {
-            _status = static_cast<ParseStatus>(
-                    std::numeric_limits<boost::int32_t>::min());
-        }
-        else
-        {
-            unsigned int statusNumber = static_cast<int>(val.to_number());
-            _status = static_cast<ParseStatus>(statusNumber);
-        }
-        return true;
-    }
-    else if (name == NSV::PROP_LOADED)
-    {
-        // TODO: this should really be a proper property
-        bool b = val.to_bool();
-        if ( b ) _loaded = 1;
-        else _loaded = 0;
-        return true;
-    }
-
-    return as_object::set_member(name, val, nsname, ifFound);
-}
-
 void
-XML_as::parseAttribute(XMLNode_as* node, const std::string& xml,
-        std::string::const_iterator& it, Attributes& attributes)
+XML_as::parseAttribute(XMLNode_as* node, xml_iterator& it,
+        const xml_iterator end, Attributes& attributes)
 {
 
     const std::string terminators("\r\t\n >=");
 
-    std::string::const_iterator end = std::find_first_of(it, xml.end(),
+    xml_iterator ourend = std::find_first_of(it, end,
             terminators.begin(), terminators.end());
 
-    if (end == xml.end()) {
+    if (ourend == end) {
         _status = XML_UNTERMINATED_ELEMENT;
         return;
     }
-    std::string name(it, end);
+    std::string name(it, ourend);
     
     if (name.empty()) {
         _status = XML_UNTERMINATED_ELEMENT;
         return;
     }
 
-    // Point iterator to the character after the name.
-    it = end;
+    // Point iterator to the DisplayObject after the name.
+    it = ourend;
 
     // Skip any whitespace before the '='. If we reach the end of the string
     // or don't find an '=', it's a parser error.
-    if (!textAfterWhitespace(xml, it) || *it != '=') {
+    if (!textAfterWhitespace(it, end) || *it != '=') {
         _status = XML_UNTERMINATED_ELEMENT;
         return;
     }
 
-    // Point to the character after the '='
+    // Point to the DisplayObject after the '='
     ++it;
 
     // Skip any whitespace. If we reach the end of the string, or don't find
     // a " or ', it's a parser error.
-    if (!textAfterWhitespace(xml, it) || (*it != '"' && *it != '\'')) {
+    if (!textAfterWhitespace(it, end) || (*it != '"' && *it != '\'')) {
         _status = XML_UNTERMINATED_ELEMENT;
         return;
     }
 
-    // Find the end of the attribute, looking for the opening character,
+    // Find the end of the attribute, looking for the opening DisplayObject,
     // as long as it's not escaped. We begin one after the present position,
-    // which should be the opening character. We want to remember what the
+    // which should be the opening DisplayObject. We want to remember what the
     // iterator is pointing to for a while, so don't advance it.
-    end = it;
+    ourend = it;
     do {
-        ++end;
-        end = std::find(end, xml.end(), *it);
-    } while (end != xml.end() && *(end - 1) == '\\');
+        ++ourend;
+        ourend = std::find(ourend, end, *it);
+    } while (ourend != end && *(ourend - 1) == '\\');
 
-    if (end == xml.end()) {
+    if (ourend == end) {
         _status = XML_UNTERMINATED_ATTRIBUTE;
         return;
     }
     ++it;
 
-    std::string value(it, end);
+    std::string value(it, ourend);
 
     // Replace entities in the value.
-    unescape(value);
+    unescapeXML(value);
 
-    //log_debug("adding attribute to node %s: %s, %s", node->nodeName(),
-    //        name, value);
-
-    // We've already checked that end != xml.end(), so we can advance at 
+    // We've already checked that ourend != end, so we can advance at 
     // least once.
-    it = end;
-    // Advance past the last attribute character
+    it = ourend;
+    // Advance past the last attribute DisplayObject
     ++it;
 
     // Handle namespace. This is set once only for each node, and is also
@@ -294,12 +219,11 @@ XML_as::parseAttribute(XMLNode_as* node, const std::string& xml,
 /// Parse and set the docTypeDecl. This is stored without any validation and
 /// with the same case as in the parsed XML.
 void
-XML_as::parseDocTypeDecl(const std::string& xml,
-        std::string::const_iterator& it)
+XML_as::parseDocTypeDecl(xml_iterator& it, const xml_iterator end)
 {
 
-    std::string::const_iterator end;
-    std::string::const_iterator current = it; 
+    xml_iterator ourend;
+    xml_iterator current = it; 
 
     std::string::size_type count = 1;
 
@@ -307,32 +231,32 @@ XML_as::parseDocTypeDecl(const std::string& xml,
     while (count) {
 
         // Find the next closing bracket after the current position.
-        end = std::find(current, xml.end(), '>');
-        if (end == xml.end()) {
+        ourend = std::find(current, end, '>');
+        if (ourend == end) {
             _status = XML_UNTERMINATED_DOCTYPE_DECL;
             return;
         }
         --count;
 
         // Count any opening brackets in between.
-        count += std::count(current, end, '<');
-        current = end;
+        count += std::count(current, ourend, '<');
+        current = ourend;
         ++current;
     }
 
-    const std::string content(it, end);
+    const std::string content(it, ourend);
     std::ostringstream os;
     os << '<' << content << '>';
     _docTypeDecl = os.str();
-    it = end + 1;
+    it = ourend + 1;
 }
 
 
 void
-XML_as::parseXMLDecl(const std::string& xml, std::string::const_iterator& it)
+XML_as::parseXMLDecl(xml_iterator& it, const xml_iterator end)
 {
     std::string content;
-    if (!parseNodeWithTerminator(xml, it, "?>", content))
+    if (!parseNodeWithTerminator(it, end, "?>", content))
     {
         _status = XML_UNTERMINATED_XML_DECL;
         return;
@@ -348,10 +272,9 @@ XML_as::parseXMLDecl(const std::string& xml, std::string::const_iterator& it)
 
 // The iterator should be pointing to the first char after the '<'
 void
-XML_as::parseTag(XMLNode_as*& node, const std::string& xml,
-    std::string::const_iterator& it)
+XML_as::parseTag(XMLNode_as*& node, xml_iterator& it,
+        const xml_iterator end)
 {
-    //log_debug("Processing node: %s", node->nodeName());
 
     bool closing = (*it == '/');
     if (closing) ++it;
@@ -359,11 +282,11 @@ XML_as::parseTag(XMLNode_as*& node, const std::string& xml,
     // These are for terminating the tag name, not (necessarily) the tag.
     const std::string terminators("\r\n\t >");
 
-    std::string::const_iterator endName = std::find_first_of(it, xml.end(),
-            terminators.begin(), terminators.end());
+    xml_iterator endName = std::find_first_of(it, end, terminators.begin(),
+            terminators.end());
 
     // Check that one of the terminators was found; otherwise it's malformed.
-    if (endName == xml.end()) {
+    if (endName == end) {
         _status = XML_UNTERMINATED_ELEMENT;
         return;
     }
@@ -386,15 +309,14 @@ XML_as::parseTag(XMLNode_as*& node, const std::string& xml,
 
     if (!closing) {
 
-        XMLNode_as* childNode = new XMLNode_as;
+        XMLNode_as* childNode = new XMLNode_as(_global);
         childNode->nodeNameSet(tagName);
         childNode->nodeTypeSet(Element);
 
-        //log_debug("created childNode with name %s", childNode->nodeName());
         // Skip to the end of any whitespace after the tag name
         it = endName;
 
-        if (!textAfterWhitespace(xml, it)) {
+        if (!textAfterWhitespace(it, end)) {
             _status = XML_UNTERMINATED_ELEMENT;
            return;
         }
@@ -403,16 +325,16 @@ XML_as::parseTag(XMLNode_as*& node, const std::string& xml,
         // '>'
         // Attributes are added in reverse order and without any duplicates.
         Attributes attributes;
-        while (it != xml.end() && *it != '>' && _status == XML_OK)
+        while (it != end && *it != '>' && _status == XML_OK)
         {
-            if (xml.end() - it > 1 && std::equal(it, it + 2, "/>")) break;
+            if (end - it > 1 && std::equal(it, it + 2, "/>")) break;
 
             // This advances the iterator
-            parseAttribute(childNode, xml, it, attributes);
+            parseAttribute(childNode, it, end, attributes);
 
             // Skip any whitespace. If we reach the end of the string,
             // it's malformed.
-            if (!textAfterWhitespace(xml, it)) {
+            if (!textAfterWhitespace(it, end)) {
                 _status = XML_UNTERMINATED_ELEMENT;
                 return;
             }
@@ -437,10 +359,9 @@ XML_as::parseTag(XMLNode_as*& node, const std::string& xml,
 
     // If we reach here, this is a closing tag.
 
-    it = std::find(endName, xml.end(), '>');
+    it = std::find(endName, end, '>');
 
-    if (it == xml.end())
-    {
+    if (it == end) {
        _status = XML_UNTERMINATED_ELEMENT;
        return;
     }
@@ -471,60 +392,55 @@ XML_as::parseTag(XMLNode_as*& node, const std::string& xml,
 }
 
 void
-XML_as::parseText(XMLNode_as* node, const std::string& xml, 
-        std::string::const_iterator& it)
+XML_as::parseText(XMLNode_as* node, xml_iterator& it,
+        const xml_iterator end)
 {
-    std::string::const_iterator end = std::find(it, xml.end(), '<');
-    std::string content(it, end);
+    xml_iterator ourend = std::find(it, end, '<');
+    std::string content(it, ourend);
     
-    it = end;
+    it = ourend;
 
     if (ignoreWhite() && 
         content.find_first_not_of("\t\r\n ") == std::string::npos) return;
 
-    XMLNode_as* childNode = new XMLNode_as;
+    XMLNode_as* childNode = new XMLNode_as(_global);
 
     childNode->nodeTypeSet(XMLNode_as::Text);
 
     // Replace any entitites.
-    unescape(content);
+    unescapeXML(content);
 
     childNode->nodeValueSet(content);
     node->appendChild(childNode);
 
-    //log_debug("appended text node: %s", content);
 }
 
-
-
 void
-XML_as::parseComment(XMLNode_as* /*node*/, const std::string& xml, 
-        std::string::const_iterator& it)
+XML_as::parseComment(XMLNode_as* /*node*/, xml_iterator& it,
+        const xml_iterator end)
 {
-    //log_debug("discarding comment node");
 
     std::string content;
 
-    if (!parseNodeWithTerminator(xml, it, "-->", content)) {
+    if (!parseNodeWithTerminator(it, end, "-->", content)) {
         _status = XML_UNTERMINATED_COMMENT;
         return;
     }
     // Comments are discarded at least up to SWF8
-    
 }
 
 void
-XML_as::parseCData(XMLNode_as* node, const std::string& xml, 
-        std::string::const_iterator& it)
+XML_as::parseCData(XMLNode_as* node, xml_iterator& it,
+        const xml_iterator end)
 {
     std::string content;
 
-    if (!parseNodeWithTerminator(xml, it, "]]>", content)) {
+    if (!parseNodeWithTerminator(it, end, "]]>", content)) {
         _status = XML_UNTERMINATED_CDATA;
         return;
     }
 
-    XMLNode_as* childNode = new XMLNode_as;
+    XMLNode_as* childNode = new XMLNode_as(_global);
     childNode->nodeValueSet(content);
     childNode->nodeTypeSet(Text);
     node->appendChild(childNode);
@@ -536,48 +452,47 @@ XML_as::parseCData(XMLNode_as* node, const std::string& xml,
 void
 XML_as::parseXML(const std::string& xml)
 {
-    GNASH_REPORT_FUNCTION; 
-    if (xml.empty())
-    {
+
+    if (xml.empty()) {
         log_error(_("XML data is empty"));
         return;
     }
 
     // Clear current data
     clear(); 
-    _status = XML_OK;
 
-    std::string::const_iterator it = xml.begin();
+    xml_iterator it = xml.begin();
+    const xml_iterator end = xml.end();
     XMLNode_as* node = this;
 
-    while (it != xml.end() && _status == XML_OK)
+    while (it != end && _status == XML_OK)
     {
         if (*it == '<')
         {
             ++it;
-            if (textMatch(xml, it, "!DOCTYPE", false))
+            if (textMatch(it, end, "!DOCTYPE", false))
             {
                 // We should not advance past the DOCTYPE label, as
                 // the case is preserved.
-                parseDocTypeDecl(xml, it);
+                parseDocTypeDecl(it, end);
             }
-            else if (textMatch(xml, it, "?xml", false))
+            else if (textMatch(it, end, "?xml", false))
             {
                 // We should not advance past the xml label, as
                 // the case is preserved.
-                parseXMLDecl(xml, it);
+                parseXMLDecl(it, end);
             }
-            else if (textMatch(xml, it, "!--"))
+            else if (textMatch(it, end, "!--"))
             {
-                parseComment(node, xml, it);
+                parseComment(node, it, end);
             }
-            else if (textMatch(xml, it, "![CDATA["))
+            else if (textMatch(it, end, "![CDATA["))
             {
-                parseCData(node, xml, it);
+                parseCData(node, it, end);
             }
-            else parseTag(node, xml, it);
+            else parseTag(node, it, end);
         }
-        else parseText(node, xml, it);
+        else parseText(node, it, end);
     }
 
     // If everything parsed correctly, check that we've got back to the
@@ -588,75 +503,87 @@ XML_as::parseXML(const std::string& xml)
 
 }
 
-bool
-XML_as::onLoad()
-{
-    log_debug(_("%s: FIXME: onLoad Default event handler"), __FUNCTION__);
-
-    return(_loaded);
-}
-
-
 void
 XML_as::clear()
 {
     // TODO: should set childs's parent to NULL ?
-    _children.clear();
+    clearChildren();
     _docTypeDecl.clear();
     _xmlDecl.clear();
+    _status = XML_OK;
 }
 
 bool
-XML_as::ignoreWhite() const
+XML_as::ignoreWhite() 
 {
 
-    string_table::key propnamekey = _vm.getStringTable().find("ignoreWhite");
+    // TODO: use NSV:
+    const ObjectURI& propnamekey =
+        getURI(getVM(_global), "ignoreWhite");
     as_value val;
-    if (!const_cast<XML_as*>(this)->get_member(propnamekey, &val)) {
+
+    as_object* obj = object();
+
+    if (!obj->get_member(propnamekey, &val)) {
         return false;
     }
-    return val.to_bool();
+    return toBool(val, getVM(*obj));
 }
 
-
-// extern (used by Global.cpp)
+// XML.prototype is assigned after the class has been constructed, so it
+// replaces the original prototype and does not have a 'constructor'
+// property.
 void
-XML_as::init(as_object& global)
+xml_class_init(as_object& where, const ObjectURI& uri)
 {
 
-    static boost::intrusive_ptr<builtin_function> cl;
+    Global_as& gl = getGlobal(where);
+    as_object* cl = gl.createClass(&xml_new, 0);
 
-    if ( cl == NULL )
-    {
-        cl=new builtin_function(&xml_new, getXMLInterface());
+    as_function* ctor = getMember(gl, NSV::CLASS_XMLNODE).to_function();
+
+    if (ctor) {
+        // XML.prototype is an XMLNode(1, "");
+        fn_call::Args args;
+        args += 1, "";
+        as_object* proto =
+            constructInstance(*ctor, as_environment(getVM(where)), args);
+        attachXMLInterface(*proto);
+        cl->init_member(NSV::PROP_PROTOTYPE, proto);
     }
     
-    global.init_member("XML", cl.get());
+    where.init_member(uri, cl, as_object::DefaultFlags);
 
 }
 
 void
-XML_as::registerNative(as_object& global)
+registerXMLNative(as_object& where)
 {
-    VM& vm = global.getVM();
+    VM& vm = getVM(where);
     vm.registerNative(xml_escape, 100, 5);
-    vm.registerNative(xml_createElement, 253, 8);
-    vm.registerNative(xml_createTextNode, 253, 9);
-    vm.registerNative(xml_parseXML, 253, 10);
+    vm.registerNative(xml_createElement, 253, 10);
+    vm.registerNative(xml_createTextNode, 253, 11);
+    vm.registerNative(xml_parseXML, 253, 12);
 }
 
-///
-/// XML object AS interface.
-///
-
 namespace {
 
 void
-attachXMLProperties(as_object& /*o*/)
+attachXMLProperties(as_object& o)
 {
-    // if we use a proper member here hasOwnProperty() would return true
-    // but we want it to return false instead. See XML.as
-    //o.init_member("status", as_value(XML::sOK));
+
+    as_object* proto = o.get_prototype();
+    if (!proto) return;
+    const int flags = 0;
+    proto->init_member("contentType", "application/x-www-form-urlencoded",
+            flags);
+    proto->init_property("docTypeDecl", &xml_docTypeDecl, &xml_docTypeDecl,
+            flags);
+    proto->init_member("ignoreWhite", false, flags);
+    proto->init_property("loaded", xml_loaded, xml_loaded);
+    proto->init_property("status", xml_status, xml_status, flags);
+    proto->init_property("xmlDecl", &xml_xmlDecl, &xml_xmlDecl, flags);
+
 }
 
 
@@ -664,80 +591,101 @@ void
 attachXMLInterface(as_object& o)
 {
 
-    VM& vm = o.getVM();
+    VM& vm = getVM(o);
+    Global_as& gl = getGlobal(o);
 
     const int flags = 0;
 
     // No flags:
-    o.init_member("addRequestHeader", new builtin_function(
-                LoadableObject::loadableobject_addRequestHeader), flags);
-    o.init_member("createElement", vm.getNative(253, 8), flags);
-    o.init_member("createTextNode", vm.getNative(253, 9), flags);
-    o.init_member("getBytesLoaded", 
-            new builtin_function(xml_getBytesLoaded), flags);
-    o.init_member("getBytesTotal", 
-            new builtin_function(xml_getBytesTotal), flags);
+    o.init_member("createElement", vm.getNative(253, 10), flags);
+    o.init_member("createTextNode", vm.getNative(253, 11), flags);
     o.init_member("load", vm.getNative(301, 0), flags);
-    o.init_member("parseXML", vm.getNative(253, 10), flags); 
+    
+    /// This handles getBytesLoaded, getBytesTotal, and addRequestHeader
+    attachLoadableInterface(o, flags);
+    
+    o.init_member("parseXML", vm.getNative(253, 12), flags); 
     o.init_member("send", vm.getNative(301, 1), flags);
     o.init_member("sendAndLoad", vm.getNative(301, 2), flags);
-    o.init_member("onData", new builtin_function(xml_ondata), flags);
+    o.init_member("onData", gl.createFunction(xml_onData), flags);
+    o.init_member("onLoad", gl.createFunction(xml_onLoad), flags);
 
-    o.init_property("xmlDecl", &xml_xmlDecl, &xml_xmlDecl, flags);
-    o.init_property("docTypeDecl", &xml_docTypeDecl, &xml_docTypeDecl, flags);
-}
-
-as_object*
-getXMLInterface()
-{
-    static boost::intrusive_ptr<as_object> o;
-    if ( o == NULL )
-    {
-        o = new as_object(XMLNode_as::getXMLNodeInterface());
-        attachXMLInterface(*o);
-    }
-    return o.get();
 }
 
 as_value
 xml_new(const fn_call& fn)
 {
-    boost::intrusive_ptr<XML_as> xml_obj;
-  
-    if ( fn.nargs > 0 )
-    {
-        if ( fn.arg(0).is_object() )
-        {
-            boost::intrusive_ptr<as_object> obj = fn.arg(0).to_object();
-            xml_obj = dynamic_cast<XML_as*>(obj.get());
-            if ( xml_obj )
-            {
-                log_debug(_("Cloned the XML object at %p"),
-                       (void *)xml_obj.get());
-                return as_value(xml_obj->cloneNode(true).get());
+
+    as_object* obj = ensure<ValidThis>(fn);
+
+    if (fn.nargs && !fn.arg(0).is_undefined()) {
+
+        // Copy constructor clones nodes.
+        if (fn.arg(0).is_object()) {
+            as_object* other = toObject(fn.arg(0), getVM(fn));
+            XML_as* xml;
+            if (isNativeType(other, xml)) {
+                as_object* clone = xml->cloneNode(true)->object();
+                attachXMLProperties(*clone);
+                return as_value(clone);
             }
         }
 
-        const std::string& xml_in = fn.arg(0).to_string();
-        if ( xml_in.empty() )
-        {
-            IF_VERBOSE_ASCODING_ERRORS(
-            log_aserror(_("First arg given to XML constructor (%s) "
-                    "evaluates to the empty string"), fn.arg(0));
-            );
-        }
-        else
-        {
-            xml_obj = new XML_as(xml_in);
-            return as_value(xml_obj.get());
-        }
+        const int version = getSWFVersion(fn);
+        const std::string& xml_in = fn.arg(0).to_string(version);
+        // It doesn't matter if the string is empty.
+        obj->setRelay(new XML_as(*obj, xml_in));
+        attachXMLProperties(*obj);
+        return as_value();
     }
 
-    xml_obj = new XML_as;
+    obj->setRelay(new XML_as(*obj));
+    attachXMLProperties(*obj);
 
-    return as_value(xml_obj.get());
+    return as_value();
 }
 
+/// This is attached to the prototype (an XMLNode) on construction of XML
+//
+/// It has the curious effect of giving the XML object an inherited 'loaded'
+/// property that fails when called on the prototype, because the prototype
+/// is of type XMLNode.
+as_value
+xml_loaded(const fn_call& fn)
+{
+    XML_as* ptr = ensure<ThisIsNative<XML_as> >(fn);
+
+    if (!fn.nargs) {
+        XML_as::LoadStatus ls = ptr->loaded();
+        if (ls == XML_as::XML_LOADED_UNDEFINED) return as_value();
+        return as_value(static_cast<bool>(ls));
+    }
+    ptr->setLoaded(
+            static_cast<XML_as::LoadStatus>(toBool(fn.arg(0), getVM(fn))));
+    return as_value();
+}
+
+as_value
+xml_status(const fn_call& fn)
+{
+    XML_as* ptr = ensure<ThisIsNative<XML_as> >(fn);
+    
+    if (!fn.nargs) {
+        return as_value(ptr->status());
+    }
+
+    const double status = toNumber(fn.arg(0), getVM(fn));
+    if (isNaN(status) ||
+            status > std::numeric_limits<boost::int32_t>::max() ||
+            status < std::numeric_limits<boost::int32_t>::min()) {
+
+        ptr->setStatus(static_cast<XML_as::ParseStatus>(
+                    std::numeric_limits<boost::int32_t>::min()));
+    }
+
+    ptr->setStatus(static_cast<XML_as::ParseStatus>(int(status)));
+    return as_value();
+}
 
 /// Only available as ASnative.
 as_value
@@ -746,7 +694,7 @@ xml_escape(const fn_call& fn)
     if (!fn.nargs) return as_value();
 
     std::string escaped = fn.arg(0).to_string();
-    XML_as::escape(escaped);
+    escapeXML(escaped);
     return as_value(escaped);
 }
 
@@ -765,11 +713,11 @@ xml_createElement(const fn_call& fn)
     if (fn.nargs > 0)
     {
         const std::string& text = fn.arg(0).to_string();
-        XMLNode_as *xml_obj = new XMLNode_as;
+        XMLNode_as *xml_obj = new XMLNode_as(getGlobal(fn));
         xml_obj->nodeNameSet(text);
         xml_obj->nodeTypeSet(XMLNode_as::Text);
 
-        return as_value(xml_obj);
+        return as_value(xml_obj->object());
         
     }
     else {
@@ -793,10 +741,10 @@ xml_createTextNode(const fn_call& fn)
 
     if (fn.nargs > 0) {
         const std::string& text = fn.arg(0).to_string();
-        XMLNode_as* xml_obj = new XMLNode_as;
+        XMLNode_as* xml_obj = new XMLNode_as(getGlobal(fn));
         xml_obj->nodeValueSet(text);
         xml_obj->nodeTypeSet(XMLNode_as::Text);
-        return as_value(xml_obj);
+        return as_value(xml_obj->object());
     }
     else {
         log_error(_("no text for text node creation"));
@@ -805,31 +753,11 @@ xml_createTextNode(const fn_call& fn)
 }
 
 
-as_value
-xml_getBytesLoaded(const fn_call& fn)
-{
-    boost::intrusive_ptr<XML_as> ptr = ensureType<XML_as>(fn.this_ptr);
-    long int ret = ptr->getBytesLoaded();
-    if ( ret < 0 ) return as_value();
-    else return as_value(ret);
-}
-
-
-as_value
-xml_getBytesTotal(const fn_call& fn)
-{
-    boost::intrusive_ptr<XML_as> ptr = ensureType<XML_as>(fn.this_ptr);
-    long int ret = ptr->getBytesTotal();
-    if ( ret < 0 ) return as_value();
-    else return as_value(ret);
-}
-
-
 as_value
 xml_parseXML(const fn_call& fn)
 {
 
-    boost::intrusive_ptr<XML_as> ptr = ensureType<XML_as>(fn.this_ptr);
+    XML_as* ptr = ensure<ThisIsNative<XML_as> >(fn);
 
     if (fn.nargs < 1)
     {
@@ -848,7 +776,7 @@ xml_parseXML(const fn_call& fn)
 as_value
 xml_xmlDecl(const fn_call& fn)
 {
-    boost::intrusive_ptr<XML_as> ptr = ensureType<XML_as>(fn.this_ptr);
+    XML_as* ptr = ensure<ThisIsNative<XML_as> >(fn);
 
     if (!fn.nargs)
     {
@@ -861,7 +789,7 @@ xml_xmlDecl(const fn_call& fn)
     // Setter
 
     const std::string& xml = fn.arg(0).to_string();
-    ptr->setDocTypeDecl(xml);
+    ptr->setXMLDecl(xml);
     
     return as_value();
 
@@ -870,7 +798,7 @@ xml_xmlDecl(const fn_call& fn)
 as_value
 xml_docTypeDecl(const fn_call& fn)
 {
-    boost::intrusive_ptr<XML_as> ptr = ensureType<XML_as>(fn.this_ptr);
+    XML_as* ptr = ensure<ThisIsNative<XML_as> >(fn);
 
     if (!fn.nargs)
     {
@@ -889,43 +817,48 @@ xml_docTypeDecl(const fn_call& fn)
 
 }
 
+/// XML.prototype has an empty onLoad function defined.
 as_value
-xml_ondata(const fn_call& fn)
+xml_onLoad(const fn_call& /*fn*/)
+{
+    return as_value();
+}
+
+as_value
+xml_onData(const fn_call& fn)
 {
 
-    as_object* thisPtr = fn.this_ptr.get();
+    as_object* thisPtr = fn.this_ptr;
     assert(thisPtr);
 
-    // See http://gitweb.freedesktop.org/?p=swfdec/swfdec.git;a=blob;f=libswfdec/swfdec_initialize.as
+    // See http://gitweb.freedesktop.org/?p=swfdec/swfdec.git;
+    // a=blob;f=libswfdec/swfdec_initialize.as
 
-    as_value src; src.set_null();
-    if ( fn.nargs ) src = fn.arg(0);
+    as_value src;
+    if (fn.nargs) src = fn.arg(0);
 
-    if ( ! src.is_null() )
-    {
+    if (!src.is_undefined()) {
         thisPtr->set_member(NSV::PROP_LOADED, true);
-        thisPtr->callMethod(NSV::PROP_PARSE_XML, src);
-        thisPtr->callMethod(NSV::PROP_ON_LOAD, true);
+        callMethod(thisPtr, NSV::PROP_PARSE_XML, src);
+        callMethod(thisPtr, NSV::PROP_ON_LOAD, true);
     }
-    else
-    {
-        thisPtr->set_member(NSV::PROP_LOADED, true);
-        thisPtr->callMethod(NSV::PROP_ON_LOAD, true);
+    else {
+        thisPtr->set_member(NSV::PROP_LOADED, false);
+        callMethod(thisPtr, NSV::PROP_ON_LOAD, false);
     }
 
     return as_value();
 }
 
 /// Case insensitive match of a string, returning false if there too few
-/// characters left or if there is no match. If there is a match, and advance
-/// is not false, the iterator points to the character after the match.
+/// DisplayObjects left or if there is no match. If there is a match, and advance
+/// is not false, the iterator points to the DisplayObject after the match.
 bool
-textMatch(const std::string& xml, std::string::const_iterator& it,
+textMatch(xml_iterator& it, const xml_iterator end,
         const std::string& match, bool advance)
 {
 
     const std::string::size_type len = match.length();
-    const std::string::const_iterator end = xml.end();
 
     if (static_cast<size_t>(end - it) < len) return false;
 
@@ -941,11 +874,11 @@ textMatch(const std::string& xml, std::string::const_iterator& it,
 /// @return true if there is text after the whitespace, false if we 
 ///         reach the end of the string.
 bool
-textAfterWhitespace(const std::string& xml, std::string::const_iterator& it)
+textAfterWhitespace(xml_iterator& it, const xml_iterator end)
 {
     const std::string whitespace("\r\t\n ");
-    while (it != xml.end() && whitespace.find(*it) != std::string::npos) ++it;
-    return (it != xml.end());
+    while (it != end && whitespace.find(*it) != std::string::npos) ++it;
+    return (it != end);
 }
 
 /// Parse a complete node up to a specified terminator.
@@ -953,33 +886,48 @@ textAfterWhitespace(const std::string& xml, std::string::const_iterator& it)
 /// @return     false if we reach the end of the text before finding the
 ///             terminator.
 /// @param it   The current position of the iterator. If the return is true,
-///             this points to the first character after the terminator 
+///             this points to the first DisplayObject after the terminator 
 ///             after return
 /// @param content  If the return is true, this is filled with the content of
 ///                 the tag.
 /// @param xml      The complete XML string.
 bool
-parseNodeWithTerminator(const std::string& xml,
-        std::string::const_iterator& it, const std::string& terminator,
-        std::string& content)
+parseNodeWithTerminator(xml_iterator& it, const xml_iterator end,
+        const std::string& terminator, std::string& content)
 {
-    std::string::const_iterator end = std::search(it, xml.end(),
-            terminator.begin(), terminator.end());
+    xml_iterator ourend = std::search(it, end, terminator.begin(),
+            terminator.end());
 
-    if (end == xml.end()) {
+    if (ourend == end) {
         return false;
     }
 
-    content = std::string(it, end);
-    it = end + terminator.length();
+    content = std::string(it, ourend);
+    it = ourend + terminator.length();
 
     return true;
 }
 
-} // anonymous namespace
-} // end of gnash namespace
+const Entities&
+getEntities()
+{
 
-// Local Variables:
+    static const Entities entities = boost::assign::map_list_of
+        ("&amp;", "&")
+        ("&quot;", "\"")
+        ("&lt;", "<")
+        ("&gt;", ">")
+        ("&apos;", "'");
+
+    return entities;
+
+}
+
+} // anonymous namespace 
+} // gnash namespace
+
+// local Variables:
 // mode: C++
 // indent-tabs-mode: t
 // End:
+
