@@ -1,7 +1,7 @@
 // ExternalInterface_as.h:  ActionScript "ExternalInterface" class, for Gnash.
 //
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -21,19 +21,24 @@
 #ifndef GNASH_ASOBJ_EXTERNALINTERFACE_H
 #define GNASH_ASOBJ_EXTERNALINTERFACE_H
 
-
-//#include <memory> // for auto_ptr
+namespace gnash {
+    class as_object;
+    struct ObjectURI;
+}
 
 namespace gnash {
 
-class as_object;
-class ObjectURI;
-
 /// Initialize the global ExternalInterface class
-void externalinterface_class_init(as_object& where, const ObjectURI& uri);
+void externalinterface_class_init(gnash::as_object& where,
+                                  const gnash::ObjectURI& uri);
 
+void registerExternalInterfaceNative(as_object& global);
 
-} // end of gnash namespace
+} // namespace gnash 
 
-// __GNASH_ASOBJ_EXTERNALINTERFACE_H__
 #endif
+
+// local Variables:
+// mode: C++
+// indent-tabs-mode: nil
+// End:
