@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -16,8 +16,8 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-#ifndef __GNASH_ASOBJ_BEVELFILTER_H__
-#define __GNASH_ASOBJ_BEVELFILTER_H__
+#ifndef GNASH_ASOBJ_BEVELFILTER_H
+#define GNASH_ASOBJ_BEVELFILTER_H
 
 namespace gnash {
 
@@ -28,7 +28,5 @@ struct ObjectURI;
 void bevelfilter_class_init(as_object& where, const ObjectURI& uri);
 
 } // end of gnash namespace
-
-// __GNASH_ASOBJ_BEVELFILTER_H__
 #endif
 
