@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -16,41 +16,31 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-#ifndef __GNASH_ASOBJ_BITMAPFILTER_H__
-#define __GNASH_ASOBJ_BITMAPFILTER_H__
+#ifndef GNASH_ASOBJ_BITMAPFILTER_H
+#define GNASH_ASOBJ_BITMAPFILTER_H
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
+#include "Global_as.h"
+
+namespace gnash {
+    class as_object;
+    struct ObjectURI;
+}
 
 namespace gnash {
 
-class as_object;
-
 /// Initialize the global BitmapFilter class
-void BitmapFilter_class_init(as_object& global);
+void bitmapfilter_class_init(as_object& where, const ObjectURI& uri);
 
-/// Get the interface, for inheritance.
-as_object *bitmapFilter_interface();
+void registerBitmapFilterNative(as_object& global);
+
+/// Convenience function only for BitmapFilter subclasses.
+//
+/// This implements the AS code necessary for defining subclasses of
+/// BitmapFilter in AS2.
+void registerBitmapClass(as_object& where, Global_as::ASFunction ctor,
+        Global_as::Properties p, const ObjectURI& uri);
 
 } // end of gnash namespace
 
-// __GNASH_ASOBJ_BITMAPFILTER_H__
 #endif
 
-#ifdef phelp_helper
-#ifndef easy_clone
-#define easy_clone(sp_name) \
-as_value \
-sp_name::bitmap_clone(const fn_call& fn) \
-{ \
-    boost::intrusive_ptr<sp_name> ptr = ensureType<sp_name>(fn.this_ptr); \
-    boost::intrusive_ptr<sp_name> obj = new sp_name(*ptr); \
-    boost::intrusive_ptr<as_object> r = obj; \
-    r->set_prototype(ptr->get_prototype()); \
-    r->copyProperties(*ptr); \
-\
-    return as_value(r); \
-}
-#endif /* easy_clone */
-#endif /* phelp_helper */
