@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -22,13 +22,12 @@
 namespace gnash {
 
 class as_object;
-class ObjectURI;
+struct ObjectURI;
 
 /// Initialize the global BlurFilter class
 void blurfilter_class_init(as_object& where, const ObjectURI& uri);
 
 } // end of gnash namespace
 
-// __GNASH_ASOBJ_BLURFILTER_H__
 #endif
 
