@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,13 +15,13 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
+#include "ColorMatrixFilter_as.h"
 
 #include "as_object.h"
-#include "ColorMatrixFilter.h"
 #include "VM.h"
-#include "builtin_function.h"
 #include "Global_as.h"
 #include "BitmapFilter_as.h"
+#include "Filters.h"
 
 namespace gnash {
 
@@ -69,7 +69,7 @@ colormatrixfilter_matrix(const fn_call& fn)
 as_value
 colormatrixfilter_new(const fn_call& fn)
 {
-    boost::intrusive_ptr<as_object> obj = ensure<ThisIs<as_object> >(fn);
+    as_object* obj = ensure<ValidThis>(fn);
     obj->setRelay(new ColorMatrixFilter_as);
     return as_value();
 }
