@@ -1,7 +1,7 @@
 // DisplacementMapFilter_as.cpp:  ActionScript "DisplacementMapFilter" class, for Gnash.
 //
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,15 +20,16 @@
 
 
 #include "DisplacementMapFilter_as.h"
-#include "as_object.h" // for inheritance
+
+#include <sstream>
+
+#include "as_object.h"
+#include "as_function.h" 
 #include "log.h"
 #include "fn_call.h"
 #include "Global_as.h"
-#include "smart_ptr.h" // for boost intrusive_ptr
-#include "builtin_function.h" // need builtin_function
 #include "BitmapFilter_as.h"
-
-#include <sstream>
+#include "Filters.h"
 
 namespace gnash {
 
@@ -186,7 +187,7 @@ displacementmapfilter_scaleY(const fn_call& fn)
 as_value
 displacementmapfilter_ctor(const fn_call& fn)
 {
-	boost::intrusive_ptr<as_object> obj = ensure<ThisIs<as_object> >(fn);
+	as_object* obj = ensure<ValidThis>(fn);
 	obj->setRelay(new DisplacementMapFilter_as);
     return as_value(); 
 }
