@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,13 +15,13 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
+#include "GradientGlowFilter_as.h"
 
+#include "Filters.h"
+#include "BitmapFilter_as.h"
 #include "as_object.h"
-#include "GradientGlowFilter.h"
 #include "VM.h"
 #include "Global_as.h"
-#include "BitmapFilter_as.h"
-#include "builtin_function.h"
 
 namespace gnash {
 
@@ -32,7 +32,6 @@ namespace {
     as_value gradientglowfilter_colors(const fn_call& fn);
     as_value gradientglowfilter_alphas(const fn_call& fn);
     as_value gradientglowfilter_ratios(const fn_call& fn);
-    as_value gradientglowfilter_shadowAlpha(const fn_call& fn);
     as_value gradientglowfilter_blurX(const fn_call& fn);
     as_value gradientglowfilter_blurY(const fn_call& fn);
     as_value gradientglowfilter_strength(const fn_call& fn);
@@ -98,7 +97,7 @@ gradientglowfilter_distance(const fn_call& fn)
         return as_value(ptr->m_distance );
     }
     
-    float sp_distance = fn.arg(0).to_number();
+    float sp_distance = toNumber(fn.arg(0), getVM(fn));
     ptr->m_distance = sp_distance;
     return as_value();
 }
@@ -110,7 +109,7 @@ gradientglowfilter_angle(const fn_call& fn)
     if (fn.nargs == 0) {
         return as_value(ptr->m_angle);
     }
-    double sp_angle = fn.arg(0).to_number();
+    double sp_angle = toNumber(fn.arg(0), getVM(fn));
     ptr->m_angle = sp_angle;
     return as_value();
 }
@@ -147,7 +146,7 @@ gradientglowfilter_blurX(const fn_call& fn)
     if (fn.nargs == 0) {
         return as_value(ptr->m_blurX );
     }
-    float sp_blurX = fn.arg(0).to_number ();
+    float sp_blurX = toNumber(fn.arg(0), getVM(fn));
     ptr->m_blurX = sp_blurX;
     return as_value();
 }
@@ -159,7 +158,7 @@ gradientglowfilter_blurY(const fn_call& fn)
     if (fn.nargs == 0) {
 		return as_value(ptr->m_blurY );
     }
-    float sp_blurY = fn.arg(0).to_number ();
+    float sp_blurY = toNumber(fn.arg(0), getVM(fn));
     ptr->m_blurY = sp_blurY;
     return as_value();
 }
@@ -171,7 +170,7 @@ gradientglowfilter_strength(const fn_call& fn)
     if (fn.nargs == 0) {
         return as_value(ptr->m_strength );
     }
-    float sp_strength = fn.arg(0).to_number ();
+    float sp_strength = toNumber(fn.arg(0), getVM(fn));
     ptr->m_strength = sp_strength;
     return as_value();
 }
@@ -183,7 +182,7 @@ gradientglowfilter_quality(const fn_call& fn)
     if (fn.nargs == 0) {
 		return as_value(ptr->m_quality );
     }
-    boost::uint8_t sp_quality = fn.arg(0).to_number ();
+    boost::uint8_t sp_quality = toNumber(fn.arg(0), getVM(fn));
     ptr->m_quality = sp_quality;
     return as_value();
 }
@@ -195,7 +194,7 @@ gradientglowfilter_knockout(const fn_call& fn)
     if (fn.nargs == 0) {
 		return as_value(ptr->m_knockout );
     }
-    bool sp_knockout = fn.arg(0).to_bool ();
+    const bool sp_knockout = toBool(fn.arg(0), getVM(fn));
     ptr->m_knockout = sp_knockout;
     return as_value();
 }
@@ -247,7 +246,7 @@ gradientglowfilter_type(const fn_call& fn)
 as_value
 gradientglowfilter_new(const fn_call& fn)
 {
-    boost::intrusive_ptr<as_object> obj = ensure<ThisIs<as_object> >(fn);
+    as_object* obj = ensure<ValidThis>(fn);
     obj->setRelay(new GradientGlowFilter_as);
     return as_value();
 }
