@@ -1,7 +1,7 @@
 // Rectangle_as.h:  ActionScript "Rectangle" class, for Gnash.
 //
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -32,9 +32,6 @@ struct ObjectURI;
 /// Initialize the global Rectangle class
 void rectangle_class_init(as_object& where, const ObjectURI& uri);
 
-/// Return a Rectangle instance (in case the core lib needs it)
-boost::intrusive_ptr<as_object> init_Rectangle_instance();
-
 } // end of gnash namespace
 
 // __GNASH_ASOBJ_RECTANGLE_H__
