@@ -1,6 +1,7 @@
 // filereferencelist_as.cpp:  ActionScript "FileReferenceList" class, for Gnash.
 //
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,20 +18,16 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
 
 #include "FileReferenceList_as.h"
-#include "as_object.h" // for inheritance
+
+#include "as_function.h" 
+#include "as_object.h"
 #include "log.h"
 #include "fn_call.h"
 #include "Global_as.h"
-#include "smart_ptr.h" // for boost intrusive_ptr
-#include "builtin_function.h" // need builtin_function
 #include "GnashException.h" // for ActionException
-#include "Object.h" // for AS inheritance
-#include "VM.h" // for addStatics
+#include "VM.h"
 
 #include <sstream>
 
@@ -51,18 +48,13 @@ attachFileReferenceListStaticInterface(as_object& /*o*/)
 static void
 attachFileReferenceListInterface(as_object& o)
 {
-    Global_as* gl = getGlobal(o);
-    o.init_member("addListener", gl->createFunction(filereferencelist_addListener));
-    o.init_member("browse", gl->createFunction(filereferencelist_browse));
-    o.init_member("removeListener", gl->createFunction(filereferencelist_removeListener));
+    Global_as& gl = getGlobal(o);
+    o.init_member("addListener", gl.createFunction(filereferencelist_addListener));
+    o.init_member("browse", gl.createFunction(filereferencelist_browse));
+    o.init_member("removeListener", gl.createFunction(filereferencelist_removeListener));
     o.init_property("fileList", filereferencelist_fileList_getset, filereferencelist_fileList_getset);
 }
 
-static void
-attachFileReferenceListStaticProperties(as_object& /*o*/)
-{
-   
-}
 
 static as_value
 filereferencelist_addListener(const fn_call& /*fn*/)
