@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -18,13 +19,13 @@
 #ifndef GNASH_ACTION_BUFFER_H
 #define GNASH_ACTION_BUFFER_H
 
+#include <string>
+#include <vector> 
 #include <boost/noncopyable.hpp>
-#include <boost/cstdint.hpp> // for boost::uint8_t
-#include <vector> // for composition
+#include <boost/cstdint.hpp> 
 
 #include "GnashException.h"
-#include "smart_ptr.h"
-#include "log.h" // For gettext macro
+#include "log.h"
 
 // Forward declarations
 namespace gnash {
@@ -34,13 +35,8 @@ namespace gnash {
 	class SWFStream; // for read signature
 }
 
-
 namespace gnash {
 
-class ActionExec;
-
-double convert_double_wacky(const void *p);
-
 /// A code segment.
 //
 /// This currently holds the actions in a memory
@@ -52,7 +48,6 @@ double convert_double_wacky(const void *p);
 class action_buffer : boost::noncopyable
 {
 public:
-	friend class ActionExec;
 
 	action_buffer(const movie_definition& md);
 
