@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,22 +15,20 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-
 #ifndef GNASH_FILTER_FACTORY_H
 #define GNASH_FILTER_FACTORY_H
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
 #include <vector>
-#include "BitmapFilter.h"
+#include <boost/shared_ptr.hpp>
+
+namespace gnash {
+    class SWFStream;
+    class BitmapFilter;
+}
 
 namespace gnash {
 
-class SWFStream;
-
-typedef std::vector<Filter> Filters;
+typedef std::vector<boost::shared_ptr<BitmapFilter> > Filters;
 
 class filter_factory
 {
