@@ -1,6 +1,6 @@
 //
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -72,7 +72,9 @@ namespace gnash {
     }
     class Font;
     class sound_sample;
-    class JpegImageInput;
+    namespace image {
+        class JpegInput;
+    }
 }
 
 namespace gnash
@@ -91,13 +93,6 @@ namespace gnash
 /// and SWFMovieDefinition (a top-level movie) to derive from
 /// a common class to allow tag_loaders to take a pointer
 /// to the base class to act on (consider PLACEOBJECT tags...).
-///
-/// This design is uncomfortable when it comes to programmatically
-/// created DisplayObjects, in that they do NOT have any *fixed* definition.
-/// A possible workaround to this would be not *requiring* DisplayObject
-/// instances to have an associated DefinitionTag. I'll work on this
-/// --strk 2006-12-05.
-///
 class movie_definition : public SWF::DefinitionTag
 {
 public:
@@ -225,10 +220,6 @@ public:
 		return false;
 	}
 
-	//
-	// For use during creation.
-	//
-
 	/// Returns 1 based index. Ex: if 1 then 1st frame as been fully loaded
 	virtual size_t	get_loading_frame() const = 0;
 
@@ -249,7 +240,7 @@ public:
 	/// This method is here to be called by DEFINEFONT tags loaders.
 	/// The default implementation does nothing.
 	///
-	virtual void add_font(int /*id*/, Font* /*ch*/)
+	virtual void add_font(int /*id*/, boost::intrusive_ptr<Font> /*ch*/)
 	{
 	}
 
@@ -280,11 +271,7 @@ public:
 	/// 	The tag to add in the list of executable tags for
 	/// 	the frame currently being loaded. Ownership is transferred
 	/// 	to the SWFMovieDefinition.
-	///
-	/// TODO: take an auto_ptr.
-	/// NOTE: the default implementation just let the ControlTag leak.
-	///
-	virtual void addControlTag(SWF::ControlTag* /*c*/)
+	virtual void addControlTag(boost::intrusive_ptr<SWF::ControlTag> /*c*/)
 	{
 	}
 
@@ -307,7 +294,7 @@ public:
 	/// *is* performed, and it is deleting the jpeg::input instance since
 	/// it is passed in an auto_ptr...
 	///
-	virtual void set_jpeg_loader(std::auto_ptr<JpegImageInput> /*j_in*/)
+	virtual void set_jpeg_loader(std::auto_ptr<image::JpegInput> /*j_in*/)
 	{
 	}
 
@@ -322,7 +309,7 @@ public:
 	///
 	/// NOTE: ownership of the returned object is NOT transferred
 	///
-	virtual JpegImageInput* get_jpeg_loader() const
+	virtual image::JpegInput* get_jpeg_loader() const
 	{
 		return NULL;
 	}
@@ -471,10 +458,13 @@ public:
 
 #endif
 protected:
+
     movie_definition(boost::uint16_t id = 0)
         :
         DefinitionTag(id)
     {}
+
+    virtual ~movie_definition() {}
 };
 
 } // namespace gnash
