@@ -1,6 +1,7 @@
 // sprite_definition.cpp:  for Gnash.
 //
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -16,49 +17,38 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
-#include "smart_ptr.h" // GNASH_USE_GC
+#include "RunResources.h"
+#include "smart_ptr.h" 
 #include "MovieClip.h"
 #include "sprite_definition.h"
 #include "ControlTag.h" // for dtor visibility
-#include "as_function.h" // for dtor visibility
-#include "SWFStream.h" // for use
+#include "as_function.h"
+#include "SWFStream.h"
 #include "GnashAlgorithm.h"
 #include "SWFParser.h"
+#include "namedStrings.h"
+#include "Global_as.h"
 
 #include <vector>
 #include <string>
 #include <cassert>
 
 
-// Define the following macro to get a dump the prototype 
-// members of classes registered to definitions.
-//#define DEBUG_REGISTER_CLASS 1
-
 namespace gnash {
 
 DisplayObject*
-sprite_definition::createDisplayObject(DisplayObject* parent, int id) const
+sprite_definition::createDisplayObject(Global_as& gl, DisplayObject* parent)
+    const
 {
-#ifdef DEBUG_REGISTER_CLASS
-	log_debug(_("Instantiating sprite_def %p"), (void*)this);
-#endif
-	MovieClip* si = new MovieClip(this, parent->get_root(), parent, id);
-	return si;
+    // Should not call MovieClip constructor (probably), but should
+    // attach MovieClip.prototype
+    as_object* obj = getObjectWithPrototype(gl, NSV::CLASS_MOVIE_CLIP);
+    DisplayObject* mc = new MovieClip(obj, this, parent->get_root(), parent);
+	return mc;
 }
 
 sprite_definition::~sprite_definition()
 {
-	// Release our playlist data.
-	for (PlayListMap::iterator i=m_playlist.begin(), e=m_playlist.end();
-            i != e; ++i)
-	{
-		PlayList& pl = i->second;
-        deleteAllChecked(pl);
-    }
 }
 
 /*private*/
@@ -119,8 +109,9 @@ sprite_definition::get_labeled_frame(const std::string& label,
 }
 
 sprite_definition::sprite_definition(movie_definition& m, SWFStream& in, 
-        const RunResources& runResources)
+        const RunResources& runResources, boost::uint16_t id)
 	:
+    movie_definition(id),
 	m_movie_def(m),
 	m_frame_count(0),
 	m_loading_frame(0),
@@ -130,29 +121,16 @@ sprite_definition::sprite_definition(movie_definition& m, SWFStream& in,
 	read(in, runResources);
 }
 
-/*
- * This function is not inlined to avoid having to include as_function.h
- * from sprite_definition.h. We need as_function.h for visibility of
- * as_function destructor by boost::intrusive_ptr
- */
 void
 sprite_definition::registerClass(as_function* the_class)
 {
 	registeredClass = the_class;
-#ifdef DEBUG_REGISTER_CLASS
-	log_debug(_("Registered class %p for sprite_def %p"), (void*)registeredClass.get(), (void*)this);
-	as_object* proto = registeredClass->getPrototype().get();
-	log_debug(_(" Exported interface: "));
-	proto->dump_members();
-#endif
 }
 
-#ifdef GNASH_USE_GC
 void
 sprite_definition::markReachableResources() const
 {
-	if ( registeredClass.get() ) registeredClass->setReachable();
+	if (registeredClass) registeredClass->setReachable();
 }
-#endif // GNASH_USE_GC
 
 } // namespace gnash
