@@ -1,6 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,6 +20,8 @@
 #ifndef GNASH_SWF_DEFINEFONTALIGNZONESTAG_H
 #define GNASH_SWF_DEFINEFONTALIGNZONESTAG_H
 
+#include "SWF.h"
+
 // Forward declarations
 namespace gnash {
 	class movie_definition;
@@ -30,7 +32,8 @@ namespace gnash {
 namespace gnash {
 namespace SWF {
 
-class DefineFontAlignZonesTag {
+class DefineFontAlignZonesTag
+{
 public:
 
 	enum {
@@ -56,10 +59,6 @@ private:
 
 };
 
-
-
-
-
 } // namespace gnash::SWF
 } // namespace gnash
 
