@@ -1,7 +1,7 @@
 // DefineTextTag.cpp:  Read StaticText definitions, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,10 +17,13 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-// Derived from text.cpp	-- Thatcher Ulrich <tu@tulrich.com> 2003
-
-#include "RunResources.h"
 #include "DefineTextTag.h"
+
+#include <algorithm>
+#include <numeric>
+
+#include "TypesParser.h"
+#include "RunResources.h"
 #include "SWFStream.h"
 #include "log.h"
 #include "SWF.h"
@@ -30,9 +33,7 @@
 #include "GnashAlgorithm.h"
 #include "Global_as.h"
 #include "movie_definition.h"
-
-#include <algorithm>
-#include <numeric>
+#include "Transform.h"
 
 namespace gnash {
 namespace SWF {
@@ -69,7 +70,7 @@ DefineTextTag::extractStaticText(std::vector<const TextRecord*>& to,
 
     /// Insert pointers to all our TextRecords into to.
     std::transform(_textRecords.begin(), _textRecords.end(),
-            std::back_inserter(to), CreatePointer<TextRecord>());
+            std::back_inserter(to), CreatePointer<const TextRecord>());
 
     /// Count the number of DisplayObjects in this definition's text records.
     numChars = std::accumulate(_textRecords.begin(), _textRecords.end(),
@@ -100,7 +101,7 @@ DefineTextTag::read(SWFStream& in, movie_definition&m, TagType tag)
 {
 	assert(tag == DEFINETEXT || tag == DEFINETEXT2);
 
-	_rect.read(in);
+	_rect = readRect(in);
 	_matrix = readSWFMatrix(in);
 
 	in.ensureBytes(2); // glyph_bits + advance_bits
@@ -122,14 +123,12 @@ DefineTextTag::read(SWFStream& in, movie_definition&m, TagType tag)
 }
 
 void
-DefineTextTag::display(Renderer& renderer, const StaticText& inst) const
+DefineTextTag::display(Renderer& renderer, const Transform& base) const
 {
+    Transform xform = base;
+    xform.matrix.concatenate(_matrix);
 
-    SWFMatrix mat = inst.getWorldMatrix();
-    mat.concatenate(_matrix);
-
-    TextRecord::displayRecords(renderer, mat, inst.get_world_cxform(),
-            _textRecords);
+    TextRecord::displayRecords(renderer, xform, _textRecords);
 }
 
 
