@@ -1,6 +1,6 @@
 // PlaceObject2Tag.cpp:  for Gnash.
 //
-//   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -22,6 +22,11 @@
 #endif
 
 #include "PlaceObject2Tag.h"
+
+#include <string>
+
+#include "TypesParser.h"
+#include "RunResources.h"
 #include "DisplayObject.h"
 #include "MovieClip.h"
 #include "swf_event.h"
@@ -29,17 +34,31 @@
 #include "SWFStream.h"
 #include "filter_factory.h"
 #include "GnashAlgorithm.h"
+#include "action_buffer.h"
 
 namespace gnash {
 namespace SWF {
 
+PlaceObject2Tag::PlaceObject2Tag(const movie_definition& def)
+    :
+    DisplayListTag(0),
+    m_has_flags2(0),
+    m_has_flags3(0),
+    _id(0),
+    _ratio(0),
+    m_clip_depth(0),
+    _blendMode(0),
+    _movie_def(def)
+{
+}
+
 void
 PlaceObject2Tag::readPlaceObject(SWFStream& in)
 {
     // Original place_object tag; very simple.
     in.ensureBytes(2 + 2);
     _id = in.read_u16();
-    m_depth = in.read_u16() + DisplayObject::staticDepthOffset;
+    _depth = in.read_u16() + DisplayObject::staticDepthOffset;
 
     // PlaceObject doesn't know about masks.
     m_clip_depth = DisplayObject::noClipDepthValue;
@@ -50,11 +69,11 @@ PlaceObject2Tag::readPlaceObject(SWFStream& in)
 
     if (in.tell() < in.get_tag_end_position())
     {
-        m_matrix.read(in);
+        m_matrix = readSWFMatrix(in);
         m_has_flags2 |= HAS_MATRIX_MASK;
         if (in.tell() < in.get_tag_end_position())
         {
-            m_color_transform.read_rgb(in);
+            m_color_transform = readCxFormRGB(in);
             m_has_flags2 |= HAS_CXFORM_MASK;
         }
     }
@@ -62,10 +81,10 @@ PlaceObject2Tag::readPlaceObject(SWFStream& in)
     IF_VERBOSE_PARSE
     (
             log_parse(_("  PLACEOBJECT: depth=%d(%d) char=%d"),
-            	m_depth, m_depth - DisplayObject::staticDepthOffset,
+            	_depth, _depth - DisplayObject::staticDepthOffset,
             	_id);
             if (hasMatrix()) log_parse("  SWFMatrix: %s", m_matrix);
-            if (hasCxform()) log_parse(_("  cxform: %s"), m_color_transform);
+            if (hasCxform()) log_parse(_("  SWFCxForm: %s"), m_color_transform);
     );
 
     
@@ -75,13 +94,12 @@ PlaceObject2Tag::readPlaceObject(SWFStream& in)
 void
 PlaceObject2Tag::readPlaceActions(SWFStream& in)
 {
-    int movie_version = _movie_def.get_version();
+    const int movie_version = _movie_def.get_version();
 
     in.ensureBytes(2);
     boost::uint16_t reserved = in.read_u16();
-    IF_VERBOSE_MALFORMED_SWF (
-        if ( reserved != 0 ) // must be 0
-        {
+    IF_VERBOSE_MALFORMED_SWF(
+        if (reserved != 0) {
             log_swferror(_("Reserved field in PlaceObject actions == "
                     "%u (expected 0)"), reserved);
         }
@@ -90,13 +108,11 @@ PlaceObject2Tag::readPlaceActions(SWFStream& in)
     boost::uint32_t all_event_flags;
 
     // The logical 'or' of all the following handlers.
-    if (movie_version >= 6)
-    {
+    if (movie_version >= 6) {
         in.ensureBytes(4);
         all_event_flags = in.read_u32();
     }
-    else
-    {
+    else {
         in.ensureBytes(2);
         all_event_flags = in.read_u16();        
     }
@@ -106,8 +122,7 @@ PlaceObject2Tag::readPlaceActions(SWFStream& in)
     );
 
     // Read swf_events.
-    for (;;)
-    {
+    for (;;) {
         // Handle SWF malformations locally, by just prematurely interrupting
         // parsing of action events.
         // TODO: a possibly improvement would be using local code for the
@@ -115,32 +130,28 @@ PlaceObject2Tag::readPlaceActions(SWFStream& in)
         //       call for itself plus a repeated useless function call for
         //       get_end_tag_position (which could be cached).
         //       
-        try
-        {
+        try {
             // Read event.
             in.align();
     
             boost::uint32_t flags;
-            if (movie_version >= 6)
-            {
+            if (movie_version >= 6) {
                 in.ensureBytes(4);
                 flags = in.read_u32();
             }
-            else
-            {
+            else {
                 in.ensureBytes(2);
                 flags = in.read_u16();        
             }
     
-            if (flags == 0) // no other events
-            {
+            // no other events
+            if (flags == 0) {
                 break;
             }
     
             in.ensureBytes(4);
             boost::uint32_t event_length = in.read_u32();
-            if ( in.get_tag_end_position() - in.tell() <  event_length )
-            {
+            if (in.get_tag_end_position() - in.tell() <  event_length) {
                 IF_VERBOSE_MALFORMED_SWF(
                 log_swferror(_("swf_event::read(), "
                     "even_length = %u, but only %lu bytes left "
@@ -151,20 +162,20 @@ PlaceObject2Tag::readPlaceActions(SWFStream& in)
                 break;
             }
     
-            boost::uint8_t ch = key::INVALID;
-    
-            if (flags & (1 << 17))  // has KeyPress event
-            {
+            key::code ch = key::INVALID;
+            
+            // has KeyPress event
+            if (flags & (1 << 17)) {
                 in.ensureBytes(1);
-                ch = in.read_u8();
-                event_length--;
+                ch = static_cast<key::code>(in.read_u8());
+                --event_length;
             }
     
             // Read the actions for event(s)
             // auto_ptr here prevents leaks on malformed swf
             std::auto_ptr<action_buffer> action(new action_buffer(_movie_def));
-            action->read(in, in.tell()+event_length);
-            _actionBuffers.push_back(action.release()); // take ownership
+            action->read(in, in.tell() + event_length);
+            _actionBuffers.push_back(action); 
     
             // If there is no end tag, action_buffer appends a null-terminator,
             // and fails this check. As action_buffer should check bounds, we
@@ -172,9 +183,7 @@ PlaceObject2Tag::readPlaceActions(SWFStream& in)
             //assert (action->size() == event_length);
     
             // 13 bits reserved, 19 bits used
-            const int total_known_events = 19;
-            static const event_id s_code_bits[total_known_events] =
-            {
+            static const event_id::EventCode s_code_bits[] = {
                 event_id::LOAD,
                 event_id::ENTER_FRAME,
                 event_id::UNLOAD,
@@ -194,53 +203,48 @@ PlaceObject2Tag::readPlaceActions(SWFStream& in)
                 event_id::DRAG_OVER,
     
                 event_id::DRAG_OUT,
-                event_id(event_id::KEY_PRESS, key::CONTROL),
+                event_id::KEY_PRESS,
                 event_id::CONSTRUCT
             };
+            const size_t total_known_events = arraySize(s_code_bits);
     
             // Let's see if the event flag we received is for an event
             // that we know of.
     
             // Integrity check: all reserved bits should be zero
-            if( flags >> total_known_events ) 
-            {
+            if (flags >> total_known_events) {
                 IF_VERBOSE_MALFORMED_SWF(
-                log_swferror(_("swf_event::read() -- unknown / unhandled "
-                        "event type received, flags = 0x%x"), flags);
+                    log_swferror(_("swf_event::read() -- unknown / unhandled "
+                            "event type received, flags = 0x%x"), flags);
                 );
             }
     
             // Aah! same action for multiple events !
-            for (int i = 0, mask = 1; i < total_known_events; i++, mask <<= 1)
-            {
-                if (flags & mask)
-                {
+            for (size_t i = 0, mask = 1; i < total_known_events; ++i, mask <<= 1) {
+
+                if (flags & mask) {
                     // Yes, swf_event stores a reference to an element in
                     // _actionBuffers. A case of remote ownership, but both
                     // swf_event and the actions are owned by this class,
                     // so shouldn't be a problem.
-                    action_buffer* thisAction = _actionBuffers.back();
-                    std::auto_ptr<swf_event> ev(
-                            new swf_event(s_code_bits[i], *thisAction));
+                    action_buffer& thisAction = _actionBuffers.back();
 
-                    IF_VERBOSE_PARSE (
-                    log_parse("---- actions for event %s", ev->event());
+                    const event_id id(s_code_bits[i], (i == 17 ? ch : key::INVALID));
+
+                    std::auto_ptr<swf_event> ev(new swf_event(id, thisAction));
+
+                    IF_VERBOSE_PARSE(
+                        log_parse("---- actions for event %s", ev->event());
                     );
     
-                    if (i == 17)    // has KeyPress event
-                    {
-                        ev->event().setKeyCode(ch);
-                    }
-    
-                    _eventHandlers.push_back(ev.release());
+                    _eventHandlers.push_back(ev);
                 }
             }
         }
-        catch (ParserException& what)
-        {
+        catch (const ParserException& what) {
             IF_VERBOSE_MALFORMED_SWF(
-            log_swferror(_("Unexpected end of tag while parsing PlaceObject "
-                    "tag events"));
+                log_swferror(_("Unexpected end of tag while parsing "
+                        "PlaceObject tag events"));
             );
             break;
         }
@@ -258,66 +262,59 @@ PlaceObject2Tag::readPlaceObject2(SWFStream& in)
     // PlaceObject2 specific flags
     m_has_flags2 = in.read_u8();
 
-    m_depth = in.read_u16()+DisplayObject::staticDepthOffset;
+    _depth = in.read_u16()+DisplayObject::staticDepthOffset;
 
-    if ( hasCharacter( ))
-    {
+    if (hasCharacter()) {
         in.ensureBytes(2);
         _id = in.read_u16();
     }
 
-    if ( hasMatrix() )
-    {
-        m_matrix.read(in);
+    if (hasMatrix()) {
+        m_matrix = readSWFMatrix(in);
     }
 
-    if ( hasCxform() )
-    {
-        m_color_transform.read_rgba(in);
+    if (hasCxform()) {
+        m_color_transform = readCxFormRGBA(in);
     }
 
-    if ( hasRatio() )
-    {
+    if (hasRatio()) {
         in.ensureBytes(2);
-        m_ratio = in.read_u16();
+        _ratio = in.read_u16();
     }
 
-    if ( hasName() ) 
-    {
+    if (hasName()) {
         in.read_string(m_name);
     }
 
-    if ( hasClipDepth() )
-    {
+    if (hasClipDepth()) {
         in.ensureBytes(2);
         m_clip_depth = in.read_u16() + DisplayObject::staticDepthOffset;
     }
-    else
-    {
+    else {
         m_clip_depth = DisplayObject::noClipDepthValue;
     }
 
-    if ( hasClipActions() )
-    {
+    if (hasClipActions()) {
         readPlaceActions(in);
     }
 
-    IF_VERBOSE_PARSE (
+    IF_VERBOSE_PARSE(
         log_parse(_("  PLACEOBJECT2: depth = %d (%d)"),
-            m_depth, m_depth-DisplayObject::staticDepthOffset);
-        if ( hasCharacter() ) log_parse(_("  char id = %d"), _id);
-        if ( hasMatrix() )
-        {
+            _depth, _depth-DisplayObject::staticDepthOffset);
+        if (hasCharacter()) log_parse(_("  char id = %d"), _id);
+        if (hasMatrix()) {
             log_parse(_("  SWFMatrix: %s"), m_matrix);
         }
-        if ( hasCxform() )
-        {
-            log_parse(_("  cxform: %s"), m_color_transform);
+        if (hasCxform()) {
+            log_parse(_("  SWFCxForm: %s"), m_color_transform);
         }
-        if ( hasRatio() ) log_parse(_("  ratio: %d"), m_ratio);
-        if ( hasName() ) log_parse(_("  name = %s"), m_name.c_str());
-        if ( hasClipDepth() ) log_parse(_("  clip_depth = %d (%d)"), m_clip_depth, m_clip_depth-DisplayObject::staticDepthOffset);
-        log_parse(_(" m_place_type: %d"), getPlaceType() );
+        if (hasRatio()) log_parse(_("  ratio: %d"), _ratio);
+        if (hasName()) log_parse(_("  name = %s"), m_name.c_str());
+        if (hasClipDepth()) {
+            log_parse(_("  clip_depth = %d (%d)"), m_clip_depth,
+                m_clip_depth-DisplayObject::staticDepthOffset);
+        }
+        log_parse(_(" m_place_type: %d"), getPlaceType());
     );
 
 }
@@ -339,7 +336,7 @@ PlaceObject2Tag::readPlaceObject3(SWFStream& in)
     boost::uint8_t bitmask = 0;
     std::string className;
 
-    m_depth = in.read_u16() + DisplayObject::staticDepthOffset;
+    _depth = in.read_u16() + DisplayObject::staticDepthOffset;
 
     // This is documented to be here, but real instances of 
     // tags with either className or hasImage defined are rare to
@@ -356,16 +353,16 @@ PlaceObject2Tag::readPlaceObject3(SWFStream& in)
     }
 
     if (hasMatrix()) {
-        m_matrix.read(in);
+        m_matrix = readSWFMatrix(in);
     }
 
     if (hasCxform()) {
-        m_color_transform.read_rgba(in);
+        m_color_transform = readCxFormRGBA(in);
     }
 
     if (hasRatio()) {
         in.ensureBytes(2);
-        m_ratio = in.read_u16();
+        _ratio = in.read_u16();
     }
     
     if (hasName()) {
@@ -380,21 +377,20 @@ PlaceObject2Tag::readPlaceObject3(SWFStream& in)
         m_clip_depth = DisplayObject::noClipDepthValue;
     }
 
-    if ( hasFilters() )
-    {
+    if (hasFilters()) {
         Filters v; // TODO: Attach the filters to the display object.
         filter_factory::read(in, true, &v);
 	    // at time of writing no renderer supports bitmap filters
 	    LOG_ONCE( log_unimpl("Bitmap filters") );
     }
 
-    if ( hasBlendMode() )
-    {
+    if (hasBlendMode()) {
         in.ensureBytes(1);
         _blendMode = in.read_u8();
+        LOG_ONCE(log_unimpl("Blend mode in PlaceObject tag"));
     }
 
-    if ( hasBitmapCaching() ) {
+    if (hasBitmapCaching()) {
         // cacheAsBitmap is a boolean value, so the flag itself ought to be
         // enough. Alexis' SWF reference is unsure about this, but suggests
         // reading a byte here. The official SWF format spec doesn't mention
@@ -407,21 +403,21 @@ PlaceObject2Tag::readPlaceObject3(SWFStream& in)
         // TODO: set object property
         in.ensureBytes(1);
         bitmask = in.read_u8();
-	    LOG_ONCE( log_unimpl("Bitmap caching") );
+	    LOG_ONCE(log_unimpl("Bitmap caching"));
     }
 
-    if ( hasClipActions() ) {
+    if (hasClipActions()) {
         readPlaceActions(in);
     }
 
     IF_VERBOSE_PARSE (
 
-        log_parse(_("  PLACEOBJECT3: depth = %d (%d)"), m_depth,
-            m_depth - DisplayObject::staticDepthOffset);
+        log_parse(_("  PLACEOBJECT3: depth = %d (%d)"), _depth,
+            _depth - DisplayObject::staticDepthOffset);
         if (hasCharacter()) log_parse(_("  char id = %d"), _id);
         if (hasMatrix()) log_parse(_("  SWFMatrix: %s"), m_matrix);
-        if (hasCxform()) log_parse(_("  cxform: %d"), m_color_transform);
-        if (hasRatio()) log_parse(_("  ratio: %d"), m_ratio);
+        if (hasCxform()) log_parse(_("  SWFCxForm: %d"), m_color_transform);
+        if (hasRatio()) log_parse(_("  ratio: %d"), _ratio);
         if (hasName()) log_parse(_("  name = %s"), m_name);
         if (hasClassName()) log_parse(_("  class name = %s"), className);
         if (hasClipDepth()) log_parse(_("  clip_depth = %d (%d)"),
@@ -435,63 +431,53 @@ PlaceObject2Tag::readPlaceObject3(SWFStream& in)
 void
 PlaceObject2Tag::read(SWFStream& in, TagType tag)
 {
-
-    m_TagType = tag;
-
-    if (tag == SWF::PLACEOBJECT)
-    {
+    if (tag == SWF::PLACEOBJECT) {
         readPlaceObject(in);
     }
-    else if ( tag == SWF::PLACEOBJECT2 )
-    {
+    else if (tag == SWF::PLACEOBJECT2) {
         readPlaceObject2(in);
     }
-    else
-    {
+    else {
         readPlaceObject3(in);
     }
 }
 
 /// Place/move/whatever our object in the given movie.
 void
-PlaceObject2Tag::execute(MovieClip* m, DisplayList& dlist) const
+PlaceObject2Tag::executeState(MovieClip* m, DisplayList& dlist) const
 {
-    switch ( getPlaceType() ) 
-    {
+    switch (getPlaceType()) {
       case PLACE:
           m->add_display_object(this, dlist);
-      break;
+          break;
 
       case MOVE:
           m->move_display_object(this, dlist);
-      break;
+          break;
 
       case REPLACE:
           m->replace_display_object(this, dlist);
-      break;
+          break;
 
       case REMOVE:
 		  m->remove_display_object(this, dlist);
-      break;
+          break;
     }
 }
 
 
 PlaceObject2Tag::~PlaceObject2Tag()
 {
-    deleteAllChecked(_eventHandlers);
-    deleteAllChecked(_actionBuffers);
 }
 
-/* public static */
 void
 PlaceObject2Tag::loader(SWFStream& in, TagType tag, movie_definition& m,
         const RunResources& /*r*/)
 {
-    assert(tag == SWF::PLACEOBJECT || tag == SWF::PLACEOBJECT2 || tag == SWF::PLACEOBJECT3);
+    assert(tag == SWF::PLACEOBJECT || tag == SWF::PLACEOBJECT2 ||
+            tag == SWF::PLACEOBJECT3);
 
-    // TODO: who owns and is going to remove this tag ?
-    PlaceObject2Tag* ch = new PlaceObject2Tag(m);
+    boost::intrusive_ptr<PlaceObject2Tag> ch(new PlaceObject2Tag(m));
     ch->read(in, tag);
 
     m.addControlTag(ch);
