@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,20 +15,16 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-
 #ifndef GNASH_SWF_PLACEOBJECT2TAG_H
 #define GNASH_SWF_PLACEOBJECT2TAG_H
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
+#include <string>
+#include <boost/ptr_container/ptr_vector.hpp>
 
 #include "DisplayListTag.h" // for inheritance
 #include "SWF.h" // for TagType definition
 #include "SWFMatrix.h" // for composition
-#include "cxform.h" // for composition 
-#include "DisplayObject.h" // BlendMode enum
-#include <vector>
+#include "SWFCxForm.h" // for composition 
 
 // Forward declarations
 namespace gnash {
@@ -78,7 +74,7 @@ namespace SWF {
 /// m_matrix:
 /// The SWFMatrix transform to apply to the newly created instance.
 ///
-/// m_ratio
+/// _ratio
 ///
 /// m_clip_depth:
 /// If != DisplayObject::noClipDepthValue, mark the created instance
@@ -89,22 +85,10 @@ class PlaceObject2Tag : public DisplayListTag
 {
 public:
 
-    typedef std::vector<action_buffer*> ActionBuffers;
-    typedef std::vector<swf_event*> EventHandlers;
+    typedef boost::ptr_vector<action_buffer> ActionBuffers;
+    typedef boost::ptr_vector<swf_event> EventHandlers;
 
-    PlaceObject2Tag(const movie_definition& def)
-        :
-        DisplayListTag(0), // why is it 0 here and -1 for RemoveObjectTag ??
-        m_TagType(0),
-        m_has_flags2(0),
-        m_has_flags3(0),
-        _id(0),
-        m_ratio(0),
-        m_clip_depth(0),
-        _blendMode(0),
-        _movie_def(def)
-    {
-    }
+    PlaceObject2Tag(const movie_definition& def);
 
     ~PlaceObject2Tag();
 
@@ -112,7 +96,7 @@ public:
     void read(SWFStream& in, TagType tag);
 
     /// Place/move/whatever our object in the given movie.
-    void execute(MovieClip* m, DisplayList& dlist) const;
+    void executeState(MovieClip* m, DisplayList& dlist) const;
 
     static void loader(SWFStream& in, TagType tag, movie_definition& m,
             const RunResources& r);
@@ -120,12 +104,13 @@ public:
     int getPlaceType() const { 
         return m_has_flags2 & (HAS_CHARACTER_MASK | MOVE_MASK);
     } 
-    int getRatio()     const { return m_ratio; }
+
+    boost::uint16_t getRatio() const { return _ratio; }
     int getClipDepth() const { return m_clip_depth; }
-    int getID()        const { return _id; }
+    boost::uint16_t getID() const { return _id; }
     const std::string& getName() const { return m_name; }
-    const SWFMatrix& getMatrix()    const { return m_matrix; }
-    const cxform& getCxform()    const { return m_color_transform; }
+    const SWFMatrix& getMatrix() const { return m_matrix; }
+    const SWFCxForm& getCxform() const { return m_color_transform; }
     const EventHandlers& getEventHandlers() const { return _eventHandlers; }
     
     bool hasClipActions() const { return m_has_flags2 & HAS_CLIP_ACTIONS_MASK; }
@@ -136,7 +121,7 @@ public:
     bool hasMatrix()      const { return m_has_flags2 & HAS_MATRIX_MASK; }
     bool hasCharacter()   const { return m_has_flags2 & HAS_CHARACTER_MASK; }
 
-    bool hasImage()         const { return m_has_flags3 & HAS_IMAGE_MASK; }
+    bool hasImage() const { return m_has_flags3 & HAS_IMAGE_MASK; }
 
     bool hasClassName() const {
         return m_has_flags3 & HAS_CLASS_NAME_MASK;
@@ -163,13 +148,25 @@ public:
     }
 
 private:
-    int m_TagType;
+
+    // read SWF::PLACEOBJECT 
+    void readPlaceObject(SWFStream& in);
+
+    // read placeObject2 actions
+    void readPlaceActions(SWFStream& in);
+
+    // read SWF::PLACEOBJECT2 
+    void readPlaceObject2(SWFStream& in);
+
+    // read SWF::PLACEOBJECT3
+    void readPlaceObject3(SWFStream& in);
+
     boost::uint8_t m_has_flags2;
     boost::uint8_t m_has_flags3;
     boost::uint16_t _id;
-    cxform  m_color_transform;
+    SWFCxForm  m_color_transform;
     SWFMatrix  m_matrix;
-    int     m_ratio;
+    boost::uint16_t _ratio;
     std::string m_name;
     int     m_clip_depth;
     
@@ -210,19 +207,6 @@ private:
     ActionBuffers _actionBuffers;
 
     EventHandlers _eventHandlers;
-
-    // read SWF::PLACEOBJECT 
-    void readPlaceObject(SWFStream& in);
-
-    // read placeObject2 actions
-    void readPlaceActions(SWFStream& in);
-
-    // read SWF::PLACEOBJECT2 
-    void readPlaceObject2(SWFStream& in);
-
-    // read SWF::PLACEOBJECT3
-    void readPlaceObject3(SWFStream& in);
-
 };
 
 } // namespace gnash::SWF
