@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -40,8 +40,10 @@ public:
 
     virtual void executeState(MovieClip* m, DisplayList& /*dl*/) const {
 
+        LOG_ONCE( // movie_root will always log on change
         log_debug("Setting script limits: recursion %s, timeout %s",
                 _recursionLimit, _timeoutLimit);
+        );
         getRoot(*getObject(m)).setScriptLimits(_recursionLimit, _timeoutLimit);
     }
 
@@ -49,8 +51,8 @@ public:
             const RunResources& /*r*/)
     {
         assert(tag = SWF::SCRIPTLIMITS);
-        std::auto_ptr<ScriptLimitsTag> s(new ScriptLimitsTag(in));
-        m.addControlTag(s.release());
+        boost::intrusive_ptr<ControlTag> s(new ScriptLimitsTag(in));
+        m.addControlTag(s);
     }
 
 private:
