@@ -1,7 +1,7 @@
 // SetTabIndexTag.h: This tag defines the tab index of any text object (static and dynamic text objects.) 
 //
-//   Copyright (C) 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//   Copyright (C) 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,10 +17,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-#include "smart_ptr.h" // GNASH_USE_GC
 #include "SWF.h"
-#include "ShapeRecord.h"
-#include "DefinitionTag.h"
 
 // Forward declarations.
 namespace gnash {
