@@ -1,5 +1,6 @@
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,6 +18,8 @@
 
 
 #include "ShapeRecord.h"
+
+#include "TypesParser.h"
 #include "SWF.h"
 #include "SWFStream.h"
 #include "movie_definition.h"
@@ -267,9 +270,9 @@ ShapeRecord::read(SWFStream& in, SWF::TagType tag, movie_definition& m,
                             tag == SWF::DEFINESHAPE4 ||
                             tag == SWF::DEFINESHAPE4_);
 
-    if (styleInfo)
-    {
-        _bounds.read(in);
+    if (styleInfo) {
+
+        _bounds = readRect(in);
     
         IF_VERBOSE_PARSE(
             std::string b = _bounds.toString();
@@ -277,10 +280,8 @@ ShapeRecord::read(SWFStream& in, SWF::TagType tag, movie_definition& m,
         );
     
         // TODO: Store and use these. Unfinished.
-        if (tag == SWF::DEFINESHAPE4 || tag == SWF::DEFINESHAPE4_)
-        {
-            SWFRect tbound;
-            tbound.read(in);
+        if (tag == SWF::DEFINESHAPE4 || tag == SWF::DEFINESHAPE4_) {
+            const SWFRect tbound = readRect(in);
             in.ensureBytes(1);
             static_cast<void>(in.read_u8());
             LOG_ONCE(log_unimpl("DEFINESHAPE4 edge boundaries and scales"));
