@@ -1,7 +1,7 @@
 // ActionExec.cpp:  ActionScript execution, for Gnash.
 //
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -16,11 +16,9 @@
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-//
-
 
 #ifdef HAVE_CONFIG_H
-#include "gnashconfig.h" // USE_DEBUGGER
+#include "gnashconfig.h"
 #endif
 
 #include "ActionExec.h"
@@ -34,7 +32,6 @@
 #include "SWF.h"
 #include "ASHandlers.h"
 #include "as_environment.h"
-#include "debugger.h"
 #include "SystemClock.h"
 #include "CallStack.h"
 
@@ -59,14 +56,8 @@
 
 #endif
 
-
 namespace gnash {
 
-//static LogFile& dbglogfile = LogFile::getDefaultInstance();
-#ifdef USE_DEBUGGER
-static Debugger& debugger = Debugger::getDefaultInstance();
-#endif
-
 ActionExec::ActionExec(const Function& func, as_environment& newEnv,
         as_value* nRetVal, as_object* this_ptr)
     :
@@ -146,7 +137,7 @@ ActionExec::operator()()
 
     static const SWF::SWFHandlers& ash = SWF::SWFHandlers::instance();
         
-    _originalTarget = env.get_target();
+    _originalTarget = env.target();
 
     _initialStackSize = env.stack_size();
 
@@ -174,6 +165,8 @@ ActionExec::operator()()
     // must still be carried out. 
 
     // TODO: set this in gnashrc.
+    // NOTE: According to http://www.gnashdev.org/wiki/index.php/ScriptLimits
+    //       this should be 15, not 40, seconds
     const size_t maxTime = 40 * 1000;
     SystemClock clock;
 
@@ -266,34 +259,28 @@ ActionExec::operator()()
             
             // curveball.swf and feed.swf suggest that it is the
             // *current* target, not the *original* one that matters.
-            DisplayObject* guardedChar = env.get_target();
+            DisplayObject* guardedChar = env.target();
 
             if (_abortOnUnload && guardedChar && guardedChar->unloaded()) {
-                // action_execution_order_test8.c shows that the opcode
-                // guard is not SWF version based (TODO: automate it!)
-                std::stringstream ss;
-                ss << "Target of action_buffer (" << guardedChar->getTarget() 
-                    << " of type " << typeName(*guardedChar) << ") unloaded "
-                    "by execution of opcode: " << std::endl;
 
-                dumpActions(pc, next_pc, ss);
-                ss << "Discarding " << stop_pc-next_pc
-                    << " bytes of remaining opcodes: " << std::endl;
-                dumpActions(next_pc, stop_pc, ss);
-                log_debug("%s", ss.str());
+                IF_VERBOSE_ACTION(
+                    // action_execution_order_test8.c shows that the opcode
+                    // guard is not SWF version based
+                    std::stringstream ss;
+                    ss << "Target of action_buffer (" <<
+                        guardedChar->getTarget() << " of type " <<
+                        typeName(*guardedChar) << ") unloaded "
+                        "by execution of opcode: " << std::endl;
+
+                    dumpActions(pc, next_pc, ss);
+                    ss << "Discarding " << stop_pc-next_pc
+                        << " bytes of remaining opcodes: " << std::endl;
+                    dumpActions(next_pc, stop_pc, ss);
+                    log_action("%s", ss.str());
+                );
                 break;
             }
 
-
-#ifdef USE_DEBUGGER
-            debugger.setFramePointer(code.getFramePointer(pc));
-            debugger.setEnvStack(&env);
-            if (debugger.isTracing())
-            {
-                debugger.disassemble();
-            }
-#endif
-        
 #if DEBUG_STACK
             IF_VERBOSE_ACTION (
                 log_action(_("After execution: PC %d, next PC %d, "
@@ -306,15 +293,13 @@ ActionExec::operator()()
 
 
             // Do some housecleaning on branch back
-            if ( next_pc <= pc )
-            {
+            if (next_pc <= pc) {
                 // Check for script limits hit. 
                 // See: http://www.gnashdev.org/wiki/index.php/ScriptLimits
                 if (clock.elapsed() > maxTime) {
-                    boost::format fmt(_("Time exceeded"));
+                    boost::format fmt = boost::format(_("Time exceeded while executing code in %1% between pc %2% and %3%")) % code.getMovieDefinition().get_url() % next_pc % pc;
                     throw ActionLimitException(fmt.str());
                 }
-
                 // TODO: Run garbage collector ? If stack isn't too big ?
             }
 
@@ -323,14 +308,14 @@ ActionExec::operator()()
 
         }
     }
-    catch (ActionLimitException& ex) {
+    catch (const ActionLimitException&) {
         // Class execution should stop (for this frame only?)
         // Here's were we should pop-up a window to prompt user about
         // what to do next (abort or not ?)
         cleanupAfterRun(); // we expect inconsistencies here
         throw;
     }
-    catch (ActionScriptException& ex) {
+    catch (const ActionScriptException&) {
         // An unhandled ActionScript exception was thrown.
         cleanupAfterRun();
 
@@ -342,7 +327,6 @@ ActionExec::operator()()
 
         return;
     }
-    // TODO: catch other exceptions ?
 
     cleanupAfterRun();
 
@@ -370,25 +354,22 @@ bool
 ActionExec::processExceptions(TryBlock& t)
 {
 
-    switch (t._tryState)
-    {
-
+    switch (t._tryState) {
         case TryBlock::TRY_TRY:
         {
-            if (env.stack_size() && env.top(0).is_exception())
-            {
+            if (env.stack_size() && env.top(0).is_exception()) {
 #ifdef GNASH_DEBUG_TRY
                 as_value ex = env.top(0);
                 ex.unflag_exception();
-                log_debug("TRY block: Encountered exception (%s). Set PC to catch.", ex);
+                log_debug("TRY block: Encountered exception (%s). "
+                        "Set PC to catch.", ex);
 #endif
                 // We have an exception. Don't execute any more of the try
                 // block and process exception.
                 pc = t._catchOffset;
                 t._tryState = TryBlock::TRY_CATCH;
               
-                if (!t._hasName)
-                {
+                if (!t._hasName) {
                     // Used when exceptions are thrown in functions.
                     // Tests in misc-mtasc.all/exception.as
                     as_value ex = env.pop();
@@ -397,8 +378,7 @@ ActionExec::processExceptions(TryBlock& t)
                     getVM(env).setRegister(t._registerIndex, ex);
                 }
             }
-            else
-            {
+            else {
 #ifdef GNASH_DEBUG_TRY 
                 log_debug("TRY block: No exception, continuing as normal.");
 #endif
@@ -427,8 +407,7 @@ ActionExec::processExceptions(TryBlock& t)
             // If we are here, there should have been an exception
             // in 'try'. 
             
-            if (env.stack_size() && env.top(0).is_exception())
-            {
+            if (env.stack_size() && env.top(0).is_exception()) {
                 // This was thrown in "try". Remove it from
                 // the stack and remember it so that
                 // further exceptions can be caught.
@@ -440,8 +419,7 @@ ActionExec::processExceptions(TryBlock& t)
                 log_debug("CATCH block: top of stack is an exception (%s)", ex);
 #endif
 
-                if (t._hasName && !t._name.empty())
-                {
+                if (t._hasName && !t._name.empty()) {
                     // If name isn't empty, it means we have a catch block.
                     // We should set its argument to the exception value.
                     setLocalVariable(t._name, ex);
@@ -468,8 +446,7 @@ ActionExec::processExceptions(TryBlock& t)
             log_debug("FINALLY: TryBlock name = %s", t._name);                 
 #endif
             // If the exception is here, we have thrown in catch.
-            if (env.stack_size() && env.top(0).is_exception())
-            {
+            if (env.stack_size() && env.top(0).is_exception()) {
                  
                 t._lastThrow = env.pop();
 #ifdef GNASH_DEBUG_TRY 
@@ -497,8 +474,7 @@ ActionExec::processExceptions(TryBlock& t)
             // If there's no exception here, we can execute the
             // rest of the code. If there is, it will be caught
             // by the next TryBlock or stop execution.
-            if (env.stack_size() && env.top(0).is_exception())
-            {
+            if (env.stack_size() && env.top(0).is_exception()) {
                 // Check for exception handlers straight away
                 stop_pc = t._afterTriedOffset;
 #ifdef GNASH_DEBUG_TRY
@@ -510,8 +486,7 @@ ActionExec::processExceptions(TryBlock& t)
                 _tryList.pop_back();
                 return true;
             }
-            else if (t._lastThrow.is_exception())
-            {
+            else if (t._lastThrow.is_exception()) {
                 // Check for exception handlers straight away
                 stop_pc = t._afterTriedOffset;                
 #ifdef GNASH_DEBUG_TRY
@@ -558,18 +533,15 @@ ActionExec::cleanupAfterRun()
 
     vm.setSWFVersion(_origExecSWFVersion);
 
-    IF_VERBOSE_MALFORMED_SWF
-    (
+    IF_VERBOSE_MALFORMED_SWF(
         // check if the stack was smashed
-        if ( _initialStackSize > env.stack_size() )
-        {
+        if (_initialStackSize > env.stack_size()) {
             log_swferror(_("Stack smashed (ActionScript compiler bug, or "
-                    "obfuscated SWF).Taking no action to fix (as expected)."));
+                "obfuscated SWF).Taking no action to fix (as expected)."));
         }
-        else if ( _initialStackSize < env.stack_size() )
-        {
-           log_swferror(_("%d elements left on the stack after block execution.  "),
-                 env.stack_size() - _initialStackSize);
+        else if (_initialStackSize < env.stack_size()) {
+           log_swferror(_("%d elements left on the stack after block "
+                   "execution."), env.stack_size() - _initialStackSize);
         }
     );
 
@@ -582,14 +554,11 @@ void
 ActionExec::skip_actions(size_t offset)
 {
 
-    for(size_t i=0; i<offset; ++i)
-    {
-#if 1
+    for (size_t i = 0; i < offset; ++i) {
         // we need to check at every iteration because
         // an action can be longer then a single byte
-        if ( next_pc >= stop_pc )
-        {
-            IF_VERBOSE_MALFORMED_SWF (
+        if (next_pc >= stop_pc) {
+            IF_VERBOSE_MALFORMED_SWF(
                 log_swferror(_("End of DoAction block hit while skipping "
                   "%d action tags (pc:%d, stop_pc:%d) "
                   "(WaitForFrame, probably)"), offset, next_pc,
@@ -598,23 +567,20 @@ ActionExec::skip_actions(size_t offset)
             next_pc = stop_pc;
             return;
         }
-#endif
 
         // Get the opcode.
-        boost::uint8_t action_id = code[next_pc];
+        const boost::uint8_t action_id = code[next_pc];
 
         // Set default next_pc offset, control flow action handlers
         // will be able to reset it.
-        if ((action_id & 0x80) == 0)
-        {
+        if ((action_id & 0x80) == 0) {
             // action with no extra data
-            next_pc++;
+            ++next_pc;
         }
-        else
-        {
+        else {
             // action with extra data
-            boost::int16_t length = code.read_int16(next_pc+1);
-            assert( length >= 0 );
+            const boost::int16_t length = code.read_int16(next_pc + 1);
+            assert(length >= 0);
             next_pc += length + 3;
         }
     }
@@ -642,46 +608,31 @@ ActionExec::pushWith(const With& entry)
 bool
 ActionExec::delVariable(const std::string& name)
 {
-    return env.delVariableRaw(name, getScopeStack());
-}
-
-bool
-ActionExec::delObjectMember(as_object& obj, const std::string& name)
-{
-    string_table& st = getStringTable(env);
-    std::pair<bool,bool> ret = obj.delProperty(st.find(name));
-    return ret.second;
+    return gnash::delVariable(env, name, getScopeStack());
 }
 
 void
 ActionExec::setVariable(const std::string& name, const as_value& val)
 {
-    return env.set_variable(name, val, getScopeStack());
-}
-
-as_value
-ActionExec::getVariable(const std::string& name)
-{
-    return env.get_variable(name, getScopeStack());
+    gnash::setVariable(env, name, val, getScopeStack());
 }
 
 as_value
 ActionExec::getVariable(const std::string& name, as_object** target)
 {
-    return env.get_variable(name, getScopeStack(), target);
+    return gnash::getVariable(env, name, getScopeStack(), target);
 }
 
 void
 ActionExec::setLocalVariable(const std::string& name, const as_value& val)
 {
     if (isFunction()) {
-        string_table& st = getStringTable(env);
         // TODO: set local in the function object?
-        setLocal(getVM(env).currentCall(), st.find(name), val);
+        setLocal(getVM(env).currentCall(), getURI(getVM(env), name), val);
     } else {
         // TODO: set target member  ?
         //       what about 'with' stack ?
-        env.set_variable(name, val, getScopeStack());
+        gnash::setVariable(env, name, val, getScopeStack());
     }
 }
 
@@ -689,7 +640,7 @@ as_object*
 ActionExec::getTarget()
 {
     if (_withStack.empty()) {
-        return getObject(env.get_target());
+        return getObject(env.target());
     }
     return _withStack.back().object();
 }
@@ -702,14 +653,12 @@ ActionExec::pushTryBlock(TryBlock t)
     stop_pc = t._catchOffset;
 
     _tryList.push_back(t);
-
 }
 
 void
 ActionExec::pushReturn(const as_value& t)
 {
-    if (retval)
-    {
+    if (retval) {
         *retval = t;
     }
     _returning = true;
@@ -720,7 +669,8 @@ ActionExec::adjustNextPC(int offset)
 {
     const int tagPos = offset + static_cast<int>(pc);
     if (tagPos < 0) {
-        log_unimpl(_("Jump outside DoAction tag requested (offset %d before tag start)"), -tagPos);
+        log_unimpl(_("Jump outside DoAction tag requested (offset %d "
+            "before tag start)"), -tagPos);
         return;
     }
     next_pc += offset;
@@ -730,23 +680,22 @@ void
 ActionExec::dumpActions(size_t from, size_t to, std::ostream& os)
 {
     size_t lpc = from;
-    while (lpc < to)
-    {
+    while (lpc < to) {
         // Get the opcode.
-        boost::uint8_t action_id = code[lpc];
+        const boost::uint8_t action_id = code[lpc];
 
         os << " PC:" << lpc << " - EX: " <<  code.disasm(lpc) << std::endl;
 
         // Set default next_pc offset, control flow action handlers
         // will be able to reset it.
-        if ((action_id & 0x80) == 0)
-        {
+        if ((action_id & 0x80) == 0) {
             // action with no extra data
-            lpc++;
-        } else {
+            ++lpc;
+        } 
+        else {
             // action with extra data
-            boost::int16_t length = code.read_int16(lpc+1);
-            assert( length >= 0 );
+            const boost::int16_t length = code.read_int16(lpc + 1);
+            assert(length >= 0);
             lpc += length + 3;
         }
 
