@@ -1,6 +1,6 @@
 // MediaHandler.h: Base class for media handlers
 // 
-//   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,27 +20,28 @@
 #ifndef GNASH_MEDIAHANDLER_H
 #define GNASH_MEDIAHANDLER_H
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
 #include "MediaParser.h" // for videoCodecType and audioCodecType enums
 #include "dsodefs.h" // DSOEXPORT
 #include "VideoConverter.h"
+#include "GnashFactory.h"
 
+#include <vector>
 #include <memory>
+#include <map>
+#include <string>
 
 // Forward declarations
 namespace gnash {
-	class IOChannel;
-	namespace media {
-		class VideoDecoder;
-		class AudioDecoder;
-		class AudioInfo;
-		class VideoInfo;
+    class IOChannel;
+    namespace media {
+        class VideoDecoder;
+        class AudioDecoder;
+        class AudioInfo;
+        class VideoInfo;
         class VideoInput;
         class AudioInput;
-	}
+        class MediaHandler;
+    }
 }
 
 namespace gnash {
@@ -53,62 +54,55 @@ namespace gnash {
 /// The subsystem's entry point is a MediaHandler instance, which acts
 /// as a factory for parsers, decoders and encoders.
 ///
-/// Theoretically, it should be possible for actual MediaHandler
-/// implementations to be loaded at runtime, altought this is not yet
-/// implemented at time of writing (2008/10/27).
-///
 /// @todo fix http://wiki.gnashdev.org/wiki/index.php/Libmedia, is obsoleted
-///
 namespace media {
 
+struct DSOEXPORT RegisterAllHandlers
+{
+    RegisterAllHandlers();
+};
+
+typedef GnashFactory<MediaHandler, RegisterAllHandlers> MediaFactory;
+
 /// The MediaHandler class acts as a factory to provide parser and decoders
 class DSOEXPORT MediaHandler
 {
 public:
 
-	virtual ~MediaHandler() {}
+    virtual ~MediaHandler() {}
 
-	/// Return currently registered MediaHandler, possibly null.
-	static MediaHandler* get()
-	{
-		return _handler.get();
-	}
+    /// Return a description of this media handler.
+    virtual std::string description() const = 0;
 
-	/// Register a MediaHandler to use
-	static void set(std::auto_ptr<MediaHandler> mh)
-	{
-		_handler = mh;
-	}
-
-	/// Return an appropriate MediaParser for given input
-	//
-	/// @param stream
-	///	Input stream, ownership transferred
-	///
-	/// @return 0 if no parser could be created for the input
-	///
-	/// NOTE: the default implementation returns an FLVParser for FLV input
-	///       or 0 for others.
-	///
-	virtual std::auto_ptr<MediaParser>
+    /// Return an appropriate MediaParser for given input
+    //
+    /// @param stream
+    ///    Input stream, ownership transferred
+    ///
+    /// @return 0 if no parser could be created for the input
+    ///
+    /// NOTE: the default implementation returns an FLVParser for FLV input
+    ///       or 0 for others.
+    ///
+    virtual std::auto_ptr<MediaParser>
         createMediaParser(std::auto_ptr<IOChannel> stream);
 
-	/// Create a VideoDecoder for decoding what's specified in the VideoInfo
-	//
-	/// @param info VideoInfo class with all the info needed to decode
-	///             the sound correctly.
+    /// Create a VideoDecoder for decoding what's specified in the VideoInfo
+    //
+    /// @param info VideoInfo class with all the info needed to decode
+    ///             the image stream correctly.
     /// @return     Will always return a valid VideoDecoder or throw a
     ///             gnash::MediaException if a fatal error occurs.
-	virtual std::auto_ptr<VideoDecoder>
+    virtual std::auto_ptr<VideoDecoder>
         createVideoDecoder(const VideoInfo& info)=0;
 
-	/// Create an AudioDecoder for decoding what's specified in the AudioInfo
-	//
-	/// @param info AudioInfo class with all the info needed to decode
-	///             the sound correctly.
+    /// Create an AudioDecoder for decoding what's specified in the AudioInfo
+    //
+    /// @param info AudioInfo class with all the info needed to decode
+    ///             the sound correctly.
     /// @return     Will always return a valid AudioDecoder or throw a
     ///             gnash::MediaException if a fatal error occurs.
-	virtual std::auto_ptr<AudioDecoder>
+    virtual std::auto_ptr<AudioDecoder>
         createAudioDecoder(const AudioInfo& info)=0;
 
     /// Create an VideoConverter for converting between color spaces.
@@ -132,6 +126,8 @@ public:
     ///                 or null if it is not available. 
     virtual VideoInput* getVideoInput(size_t index) = 0;
 
+    virtual AudioInput* getAudioInput(size_t index) = 0;
+
     /// Return a list of available cameras.
     //
     /// This is re-generated every time the function is called.
@@ -145,35 +141,35 @@ public:
 
 protected:
 
-    /// Create an AudioDecoder for FLASH codecs 
+    /// Base constructor
+    //
+    /// This is an abstract base class, so not instantiable anyway.
+    MediaHandler() {}
+
+    /// Create an AudioDecoder for CODEC_TYPE_FLASH codecs 
     //
     /// This method is attempted as a fallback in case
     /// a mediahandler-specific audio decoder couldn't be created
-    /// for a FLASH codec.
+    /// for a CODEC_TYPE_FLASH codec.
     /// 
     /// @throws a MediaException if it can't create a decoder
     ///
     /// @param info
     ///     Informations about the audio. It is *required*
-    ///     for info.type to be media::FLASH (caller should check
+    ///     for info.type to be media::CODEC_TYPE_FLASH (caller should check
     ///     that before calling this).
     ///
     std::auto_ptr<AudioDecoder> createFlashAudioDecoder(const AudioInfo& info);
 
-	/// Return true if input stream is an FLV
+    /// Return true if input stream is an FLV
     //
     /// If this cannot read the necessary 3 bytes, it throws an IOException.
-	bool isFLV(IOChannel& stream) throw (IOException);
+    bool isFLV(IOChannel& stream);
 
-	MediaHandler() {}
-
-private:
-
-	static std::auto_ptr<MediaHandler> _handler;
 };
 
 
 } // gnash.media namespace 
 } // namespace gnash
 
-#endif // __MEDIAHANDLER_H__
+#endif 
