@@ -1,6 +1,6 @@
 // MediaParser.h: Base class for media parsers
 // 
-//   Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -61,10 +61,10 @@ enum videoFrameType
 enum codecType
 {
 	/// The internal flash codec ids
-	FLASH,
+	CODEC_TYPE_FLASH,
 
 	/// Custom codecs ids
-	CUSTOM
+	CODEC_TYPE_CUSTOM
 };
 
 /// Video codec ids as defined in flash
@@ -114,7 +114,7 @@ DSOEXPORT std::ostream& operator<< (std::ostream& os, const videoCodecType& t);
 ///
 enum audioCodecType
 {
-	/// Linear PCM, unspecified byte order 
+	/// Signed Linear PCM, unspecified byte order 
    	//
    	/// Use of this codec is deprecated (but still supported) due to
    	/// the unspecified byte order (you can only play >8bit samples
@@ -202,7 +202,7 @@ public:
     /// @param codeci
     ///     Audio codec id.
     ///     To be interpreted as a media::audioCodecType if the typei
-    ///     parameter is FLASH; otherwise it's an opaque number to use
+    ///     parameter is CODEC_TYPE_FLASH; otherwise it's an opaque number to use
     ///     for codec information transfer between a MediaParser and a
     ///     AudioDecoder from the same %media handler module.
     ///
@@ -238,7 +238,7 @@ public:
 
 	/// Codec identifier
 	//
-	/// This has to be interpreted as audioCodecType if codecType type is FLASH
+	/// This has to be interpreted as audioCodecType if codecType type is CODEC_TYPE_FLASH
 	/// or interpretation is opaque and we rely on the assumption that the AudioInfo
 	/// creator and the AudioInfo user have a way to get a shared interpretation
 	///
@@ -287,7 +287,7 @@ public:
     /// @param codeci
     ///     Video codec id.
     ///     To be interpreted as a media::videoCodecType if the typei
-    ///     parameter is FLASH; otherwise it's an opaque number to use
+    ///     parameter is CODEC_TYPE_FLASH; otherwise it's an opaque number to use
     ///     for codec information transfer between a MediaParser and a
     ///     VideoDecoder from the same %media handler module.
     ///
@@ -471,6 +471,11 @@ public:
 	///
 	DSOEXPORT boost::uint64_t getBufferLength() const;
 
+	/// Return true if both audio and video buffers are empty
+	//
+	/// NOTE: locks _qMutex
+	DSOEXPORT bool isBufferEmpty() const;
+
 	/// Return the time we want the parser thread to maintain in the buffer
 	DSOEXPORT boost::uint64_t getBufferTime() const
 	{
@@ -494,6 +499,8 @@ public:
 	/// @param ts will be set to timestamp of next available frame
 	/// @return false if no frame is available yet
 	///
+	/// NOTE: locks _qMutex
+	///
 	DSOEXPORT bool nextFrameTimestamp(boost::uint64_t& ts) const;
 
 	/// Get timestamp of the video frame which would be returned on nextVideoFrame
@@ -501,6 +508,8 @@ public:
 	/// @return false if there no video frame left
 	///         (either none or no more)
 	///
+	/// NOTE: locks _qMutex
+	///
 	DSOEXPORT bool nextVideoFrameTimestamp(boost::uint64_t& ts) const;
 
 	/// Returns the next video frame in the parsed buffer, advancing video cursor.
@@ -517,6 +526,8 @@ public:
 	/// @return false if there no video frame left
 	///         (either none or no more)
 	///
+	/// NOTE: locks _qMutex
+	///
 	DSOEXPORT bool nextAudioFrameTimestamp(boost::uint64_t& ts) const;
 
 	/// Returns the next audio frame in the parsed buffer, advancing audio cursor.
@@ -637,18 +648,6 @@ protected:
 	///
 	void pushEncodedVideoFrame(std::auto_ptr<EncodedVideoFrame> frame);
 
-	/// Return pointer to next encoded video frame in buffer
-	//
-	/// If no video is present, or queue is empty, 0 is returned
-	///
-	const EncodedVideoFrame* peekNextVideoFrame() const;
-
-	/// Return pointer to next encoded audio frame in buffer
-	//
-	/// If no video is present, or queue is empty, 0 is returned
-	///
-	const EncodedAudioFrame* peekNextAudioFrame() const;
-
 	/// The stream used to access the file
 	std::auto_ptr<IOChannel> _stream;
 	mutable boost::mutex _streamMutex;
@@ -712,11 +711,29 @@ protected:
 	/// The parser, when obtained a lock on _streamMutex, will check this
 	/// flag, if found to be true will clear the buffers and reset to false.
 	bool _seekRequest;
+
 private:
 
 	typedef std::deque<EncodedVideoFrame*> VideoFrames;
 	typedef std::deque<EncodedAudioFrame*> AudioFrames;
 
+	/// Return pointer to next encoded video frame in buffer
+	//
+	/// If no video is present, or queue is empty, 0 is returned
+	/// 
+	/// NOTE: Caller is expected to hold a lock on _qMutex
+	/// 
+	const EncodedVideoFrame* peekNextVideoFrame() const;
+
+	/// Return pointer to next encoded audio frame in buffer
+	//
+	/// If no video is present, or queue is empty, 0 is returned
+	/// 
+	/// NOTE: Caller is expected to hold a lock on _qMutex
+	///
+	const EncodedAudioFrame* peekNextAudioFrame() const;
+
+
 	/// Queue of video frames (the video buffer)
 	//
 	/// Elements owned by this class.
