@@ -1,7 +1,7 @@
 // AudioInputGst.cpp: Audio input processing using Gstreamer.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -24,10 +24,12 @@
 
 #include "gst/gst.h"
 #include "AudioInputGst.h"
-#include <gst/interfaces/propertyprobe.h>
 #include "log.h"
 #include "rc.h"
 
+#include <cstring>
+#include <gst/interfaces/propertyprobe.h>
+
 namespace {
     //get rc file for default mic selection
     gnash::RcInitFile& rcfile = gnash::RcInitFile::getDefaultInstance();
@@ -61,7 +63,7 @@ GnashAudioPrivate::GnashAudioPrivate() {
 AudioInputGst::AudioInputGst() 
     :
     _activityLevel(-1),
-    _gain(0),
+    _gain(50),
     _index(0),
     _muted(true),
     _rate(8000),
@@ -82,14 +84,13 @@ AudioInputGst::AudioInputGst()
     audioCreateSaveBin(_globalAudio);
 }
 
-AudioInputGst::~AudioInputGst() {
-    log_unimpl("Audio Input destructor");
+AudioInputGst::~AudioInputGst()
+{
 }
 
 void
 AudioInputGst::findAudioDevs() 
 {
-    _numdevs = 0;
     
     //enumerate audio test sources
     GstElement *element;
@@ -97,14 +98,12 @@ AudioInputGst::findAudioDevs()
     
     if (element == NULL) {
         log_error("%s: Could not create audio test source", __FUNCTION__);
-        _audioVect.push_back(NULL);
-        _numdevs += 1;
+	return;
     } else {
         _audioVect.push_back(new GnashAudio);
-        _audioVect[_numdevs]->setElementPtr(element);
-        _audioVect[_numdevs]->setGstreamerSrc(g_strdup_printf("audiotestsrc"));
-        _audioVect[_numdevs]->setProductName(g_strdup_printf("audiotest"));
-        _numdevs += 1;
+        _audioVect.back()->setElementPtr(element);
+        _audioVect.back()->setGstreamerSrc(g_strdup_printf("audiotestsrc"));
+        _audioVect.back()->setProductName(g_strdup_printf("audiotest"));
     }
     
 #ifdef HAS_GSTREAMER_PLUGINS_BASE
@@ -114,7 +113,16 @@ AudioInputGst::findAudioDevs()
     element = NULL;
 
     element = gst_element_factory_make ("pulsesrc", "pulsesrc");
+    if ( ! element ) {
+        log_error("%s: Could not create pulsesrc element", __FUNCTION__);
+        return;
+    }
     probe = GST_PROPERTY_PROBE (element);
+    if ( ! probe ) {
+        log_error("%s: Could not get property probe from pulsesrc element",
+            __FUNCTION__);
+        return;
+    }
     devarr = gst_property_probe_probe_and_get_values_name (probe, "device");
     for (size_t i = 0; devarr != NULL && i < devarr->n_values; ++i) {
         GValue *val;
@@ -131,14 +139,13 @@ AudioInputGst::findAudioDevs()
         }
         else { 
             _audioVect.push_back(new GnashAudio);
-            _audioVect[_numdevs]->setElementPtr(element);
-            _audioVect[_numdevs]->setGstreamerSrc(g_strdup_printf("pulsesrc"));
-            _audioVect[_numdevs]->setProductName(dev_name);
+            _audioVect.back()->setElementPtr(element);
+            _audioVect.back()->setGstreamerSrc(g_strdup_printf("pulsesrc"));
+            _audioVect.back()->setProductName(dev_name);
             
             gchar *location;
             g_object_get (element, "device", &location , NULL);
-            _audioVect[_numdevs]->setDevLocation(location);
-            _numdevs += 1;
+            _audioVect.back()->setDevLocation(location);
         }
     }
     if (devarr) {
@@ -309,7 +316,7 @@ AudioInputGst::audioChangeSourceBin(GnashAudioPrivate *audio)
     command = g_strdup_printf ("%s name=audioSource device=%s ! capsfilter name=capsfilter caps=audio/x-raw-int,signed=true,channels=2,rate=%i;audio/x-raw-float,channels=2,rate=%i ! rgvolume pre-amp=%f",
         audio->_audioDevice->getGstreamerSrc(),
         audio->_audioDevice->getDevLocation(),
-        _rate, _rate, _gain);
+        _rate, _rate, gstgain());
     
     log_debug ("GstPipeline command is: %s\n", command);
     
@@ -365,7 +372,7 @@ AudioInputGst::audioCreateSourceBin(GnashAudioPrivate *audio)
     command = g_strdup_printf ("%s name=audioSource device=%s ! capsfilter name=capsfilter caps=audio/x-raw-int,signed=true,channels=2,rate=%i;audio/x-raw-float,channels=2,rate=%i ! rgvolume pre-amp=%f",
         audio->_audioDevice->getGstreamerSrc(),
         audio->_audioDevice->getDevLocation(),
-        _rate, _rate, _gain);
+        _rate, _rate, gstgain());
     
     log_debug ("GstPipeline command is: %s", command);
     
