@@ -1,6 +1,6 @@
 // AudioInput.h: Audio input processing using Gstreamer.
 // 
-//   Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -42,26 +42,26 @@ class GnashAudio {
         ///       to the audio source element.
         ///
         /// @return GstElement* to the audio source element
-        GstElement* getElementPtr() {return _element;};
+        GstElement* getElementPtr() {return _element;}
         
         /// \brief Accessor to set the private _element variable from
         ///       the GnashAudio class.
         ///
         /// @param element The GstElement pointer to the audio source element.
-        void setElementPtr(GstElement* element) {_element = element;};
+        void setElementPtr(GstElement* element) {_element = element;}
         
         /// \brief Accessor to get the private _devLocation variable from
         ///       the GnashAudio class.
         ///
         /// @return The _devLocation private variable from GnashAudio class.
-        gchar* getDevLocation() {return _devLocation;};
+        gchar* getDevLocation() {return _devLocation;}
         
         /// \brief Accessor to set the private _devLocation variable from
         ///       the GnashAudio class.
         ///
         /// @param l A gchar* containing the physical location of the audio
         ///       input hardware device.
-        void setDevLocation(gchar *l) {_devLocation = l;};
+        void setDevLocation(gchar *l) {_devLocation = l;}
         
         /// \brief Accessor to return the private _gstreamerSrc variable
         ///       from the GnashAudio class.
@@ -69,28 +69,28 @@ class GnashAudio {
         /// @return The _gstreamerSrc variable from the GnashAudio class.
         ///        which should contain the type of the Gstreamer audio source
         ///        element (e.g. pulsesrc).
-        gchar* getGstreamerSrc() {return _gstreamerSrc;};
+        gchar* getGstreamerSrc() {return _gstreamerSrc;}
         
         /// \brief Accessor to set the private _gstreamerSrc variable
         ///       from the GnashAudio class.
         ///
         /// @param s A gchar* containing the type of the Gstreamer source
         ///         element type (e.g. pulsesrc)
-        void setGstreamerSrc(gchar *s) {_gstreamerSrc = s;};
+        void setGstreamerSrc(gchar *s) {_gstreamerSrc = s;}
         
         /// \brief Accessor to get the private _productName variable
         ///       from the GnashAudio class.
         ///
         /// @return A gchar* containing the audio input's hardware name
         ///       (e.g. HDA Intel).
-        gchar* getProductName() {return _productName;};
+        gchar* getProductName() {return _productName;}
         
         /// \brief Accessor to set the private _productName variable
         ///       from the GnashAudio class.
         ///
         /// @param n A gchar* to the hardware input device's hardware name
         ///         (e.g. HDA Intel).
-        void setProductName(gchar *n) {_productName = n;};
+        void setProductName(gchar *n) {_productName = n;}
 
         /// Constructor for the GnashAudio class.
         GnashAudio();
@@ -265,7 +265,7 @@ public:
     /// it here?
     virtual void setGain(double g) {
         assert (g >= 0 && g <= 100);
-        _gain = (g - 50) * 1.2;
+        _gain = g;
         audioChangeSourceBin(getGlobalAudio());
     }
 
@@ -275,7 +275,7 @@ public:
     /// TODO: shouldn't we query the value from the input rather than storing
     /// it here?
     virtual double gain() const {
-        return (_gain / 1.2) + 50;
+        return _gain;
     }
     
     virtual void setIndex(int i) {
@@ -496,13 +496,7 @@ private:
     /// \brief Function returns the total number of devices detected (useful in
     ///  iterating through the _audioVect vector.
     /// @return The _numdevs private variable from the AudioInputGst class.
-    int getNumdevs() {return _numdevs;}
-    
-    /// \brief Function sets the private _numdevs variable in the AudioInputGst
-    /// class.
-    /// @param n The integer value representing the desired value to set _numdevs
-    /// to.
-    void setNumdevs(int n) {_numdevs = n;}
+    int getNumdevs() const { return _audioVect.size(); }
     
     /// \brief Function returns a pointer to the private _audioVect element from
     ///  AudioInputGst class.
@@ -512,12 +506,10 @@ private:
     ///   structure currently being worked with.
     GnashAudioPrivate* getGlobalAudio() {return _globalAudio;}
     
-    
+    /// Interface range is 0..100, gst range is -60 to 60
+    double gstgain() { return (gain() - 50) * 1.2; }
+
 private:
-    /// \var _numdevs
-    /// \brief Contains an integer value representing the number of devices found
-    /// on the machine.
-    int _numdevs;
     
     /// \var _audioVect
     /// \brief A vector of GnashAudio pointers. This is used when storing information
@@ -538,4 +530,4 @@ private:
 } // gnash.media namespace 
 } // gnash namespace
 
-#endif // __AUDIOINPUT_H__
+#endif
