@@ -1,6 +1,6 @@
 // 
-//     Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//     Foundation, Inc
+//     Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+//     Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -14,10 +14,7 @@
 // 
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA    02110-1301    USA
-
-// 
-//
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 #ifndef RENDER_HANDLER_H
 #define RENDER_HANDLER_H
@@ -149,8 +146,7 @@
 
 #include "dsodefs.h" // for DSOEXPORT
 
-#include "FileTypes.h"
-#include "gnash.h" // Quality
+#include "GnashEnums.h" 
 #include "Range2d.h"
 #include "Point2d.h"
 #include "RGBA.h"
@@ -160,12 +156,15 @@
 
 // Forward declarations.
 namespace gnash {
+    class IOChannel;
     class CachedBitmap;
     class rgba;
+    class Transform;
     class SWFMatrix;
-    class cxform;
     class FillStyle;
     class LineStyle;
+    class Shape;
+    class MorphShape;
 
     // XXX: GnashImageProxy (delayed image rendering)
     class GnashVaapiImageProxy;
@@ -173,11 +172,9 @@ namespace gnash {
     namespace SWF {
         class ShapeRecord;
     }
-
-    class Shape;
-    class MorphShape;
-
-    class GnashImage;
+    namespace image {
+        class GnashImage;
+    }
 }
 
 namespace gnash {
@@ -199,6 +196,9 @@ public:
 
     virtual ~Renderer() {}
 
+    /// Return a description of this renderer.
+    virtual std::string description() const = 0;
+
     /// ==================================================================
     /// Interfaces for adjusting renderer output.
     /// ==================================================================
@@ -221,7 +221,8 @@ public:
     /// Given an image, returns a pointer to a bitmap_info class
     /// that can later be passed to FillStyleX_bitmap(), to set a
     /// bitmap fill style.
-    virtual CachedBitmap* createCachedBitmap(std::auto_ptr<GnashImage> im) = 0;
+    virtual CachedBitmap* createCachedBitmap(
+            std::auto_ptr<image::GnashImage> im) = 0;
 
 
     /// ==================================================================
@@ -249,8 +250,8 @@ public:
     ///   The width and height determine the size of the Flash video instance
     ///   on the stage (in TWIPS) prior to SWFMatrix transformations.         
     ///
-    virtual void drawVideoFrame(GnashImage* frame, const SWFMatrix* mat,
-            const SWFRect* bounds, bool smooth) = 0;
+    virtual void drawVideoFrame(image::GnashImage* frame,
+            const Transform& xform, const SWFRect* bounds, bool smooth) = 0;
 
     /// Draw a line-strip directly, using a thin, solid line.
     //
@@ -281,12 +282,12 @@ public:
     /// When masked==false, then any potential mask currently active will be
     /// ignored, otherwise it is respected.
     ///
-    virtual void draw_poly(const point* corners, size_t corner_count, 
+    virtual void draw_poly(const std::vector<point>& corners, 
         const rgba& fill, const rgba& outline, const SWFMatrix& mat,
         bool masked) = 0;
         
-    virtual void drawShape(const SWF::ShapeRecord& shape, const cxform& cx,
-            const SWFMatrix& worldMat) = 0;
+    virtual void drawShape(const SWF::ShapeRecord& shape,
+            const Transform& xform) = 0;
         
     /// \brief
     /// Draws a glyph (font character).
@@ -305,7 +306,6 @@ public:
     virtual void drawGlyph(const SWF::ShapeRecord& rec, const rgba& color,
            const SWFMatrix& mat) = 0;
 
-
     /// Draw the current rendering buffer to an image file.
     //
     /// Although this can be done at any time during the rendering cycle
@@ -317,7 +317,7 @@ public:
     ///                 Note that not all FileTypes are images: rendering
     ///                 to an FLV will not work.
     virtual void renderToImage(boost::shared_ptr<IOChannel> /*io*/,
-        FileType /*type*/) const {
+        FileType /*type*/, int /*quality*/) const {
 
         log_debug(_("Rendering to image not implemented for this "
             "renderer"));
@@ -340,33 +340,9 @@ public:
     /// Parameters are world coordinates (TWIPS).
     ///
     /// For more info see page \ref region_update.
-    ///
-    virtual void set_invalidated_region(const SWFRect& /*bounds*/) {}
-
     virtual void set_invalidated_regions(const InvalidatedRanges& /*ranges*/)
     {        
     }
-    
-    /// Bracket the displaying of a frame from a movie.
-    //
-    /// Set up to render a full frame from a movie and fills the
-    /// background. Sets up necessary transforms, to scale the
-    /// movie to fit within the given dimensions.    Call
-    /// end_display() when you're done.
-    ///
-    /// The rectangle (viewport_x0, viewport_y0, viewport_x0 +
-    /// viewport_width, viewport_y0 + viewport_height) defines the
-    /// window coordinates taken up by the movie.
-    ///
-    /// The rectangle (x0, y0, x1, y1) defines the pixel
-    /// coordinates of the movie that correspond to the viewport
-    /// bounds.
-    ///
-    virtual void begin_display(const rgba& background_color, 
-                    int viewport_width, int viewport_height,
-                    float x0, float x1, float y0, float y1) = 0;
-
-    virtual void end_display() = 0;
 
     /// ==================================================================
     /// Machinery for delayed images rendering (e.g. Xv with YV12 or VAAPI)
@@ -377,14 +353,10 @@ public:
     typedef std::vector<RenderImage> RenderImages;
 
     // Get first render image
-    virtual RenderImages::iterator getFirstRenderImage()
-            { return _render_images.begin(); }
     virtual RenderImages::const_iterator getFirstRenderImage() const
             { return _render_images.begin(); }
 
     // Get last render image
-    virtual RenderImages::iterator getLastRenderImage()
-            { return _render_images.end(); }
     virtual RenderImages::const_iterator getLastRenderImage() const
             { return _render_images.end(); }
     
@@ -413,13 +385,22 @@ public:
     /// ==================================================================
     
     /// Converts world coordinates to pixel coordinates
-    virtual geometry::Range2d<int> world_to_pixel(const SWFRect& worldbounds) = 0;
+    virtual geometry::Range2d<int> world_to_pixel(const SWFRect& worldbounds)
+        const = 0;
     
+    geometry::Range2d<int> world_to_pixel(const geometry::Range2d<int>& wb)
+        const
+    {
+        if ((wb.isNull() || wb.isWorld())) return wb;
+        return world_to_pixel(SWFRect(wb.getMinX(), wb.getMinY(),
+                       wb.getMaxX(), wb.getMaxY()));
+    }
+        
     /// Converts pixel coordinates to world coordinates (TWIPS)
-    virtual point pixel_to_world(int x, int y) = 0;
+    virtual point pixel_to_world(int x, int y) const = 0;
     
-    virtual geometry::Range2d<int> pixel_to_world(
-                    const geometry::Range2d<int>& pixelbounds)
+    geometry::Range2d<int> pixel_to_world(
+                    const geometry::Range2d<int>& pixelbounds) const
     {
         point topleft = pixel_to_world(
                         pixelbounds.getMinX(), pixelbounds.getMinY());
@@ -429,22 +410,7 @@ public:
         return geometry::Range2d<int> (topleft.x, topleft.y, 
             bottomright.x, bottomright.y);
     }
-    
-    virtual geometry::Range2d<int> world_to_pixel(
-                    const geometry::Range2d<int>& worldbounds)
-    {
-        if ((worldbounds.isNull() || worldbounds.isWorld())) return worldbounds;
 
-	// We always get compiler warnings on casting floats to int
-	// here, so we cast it ourselves to get rid of the warning
-	// message. Note that in both cases this rounds the float to
-	// an integer by dropping the decimal part.
-        return world_to_pixel(SWFRect(static_cast<int>(worldbounds.getMinX()),
-				   static_cast<int>(worldbounds.getMinY()),
-				   static_cast<int>(worldbounds.getMaxX()),
-				   static_cast<int>(worldbounds.getMaxY())));
-    }
-        
     /// \brief
     /// Checks if the given bounds are (partially) in the current drawing
     /// clipping area.
@@ -455,31 +421,9 @@ public:
     /// By default this method always returns true, which will ensure correct
     /// rendering. If possible, it should be re-implemented by the renderer 
     /// handler for better performance.
-    /// 'bounds' contains TWIPS coordinates.
-    ///
-    /// TODO: Take a Range2d<T> rather then a gnash::SWFRect ?
-    ///             Would T==int be good ? TWIPS as integer types ?
-    ///
-    /// See also gnash::renderer::bounds_in_clipping_area
-    ///
-    virtual bool bounds_in_clipping_area(const SWFRect& bounds) {
-        return bounds_in_clipping_area(bounds.getRange());
-    }
-    
-    virtual bool bounds_in_clipping_area(const InvalidatedRanges& ranges)
-    {
-        for (unsigned int rno=0; rno<ranges.size(); rno++) 
-        {
-            if (bounds_in_clipping_area(ranges.getRange(rno)))
-                return true;
-        }
-                
-        return false;
-    }
-    
-    virtual bool bounds_in_clipping_area(
-            const geometry::Range2d<int>& /*bounds*/)
-    {
+    /// b contains TWIPS coordinates.
+    virtual bool bounds_in_clipping_area(const geometry::Range2d<int>& /*b*/)
+        const {
         return true;
     }
 
@@ -503,58 +447,6 @@ public:
         return false; // avoid compiler warning        
     }
     
-    
-    /// Returns the average RGB color for a square block on the stage. The 
-    /// width and height of the block is defined by "radius" and x/y refer
-    /// to the center of the block. radius==1 equals getPixel() and radius==0
-    /// is illegal. For even "radius" values, the center point is not exactly
-    /// defined. 
-    /// The function returns false when at least one pixel of the block was
-    /// outside the main frame buffer. In that case the value in color_return
-    /// is undefined.
-    /// This implementation is provided for simplicity. Renderers should
-    /// implement a specialized version for better performance.
-    virtual bool getAveragePixel(rgba& color_return, int x, int y, 
-        unsigned int radius) const
-    {
-    
-        assert(radius>0); 
-    
-        // optimization:
-        if (radius==1) return getPixel(color_return, x, y);
-    
-        unsigned int r=0, g=0, b=0, a=0;
-        
-        x -= radius/2;
-        y -= radius/2;
-        
-        int xe = x+radius;
-        int ye = y+radius;
-
-        rgba pixel;
-        
-        for (int yp=y; yp<ye; yp++)
-        for (int xp=x; xp<xe; xp++)
-        {
-            if (!getPixel(pixel, xp, yp))
-                return false;
-                
-            r += pixel.m_r;            
-            g += pixel.m_g;            
-            b += pixel.m_b;            
-            a += pixel.m_a;            
-        }
-        
-        int pcount = radius*radius; 
-        color_return.m_r = r / pcount; 
-        color_return.m_g = g / pcount; 
-        color_return.m_b = b / pcount; 
-        color_return.m_a = a / pcount; 
-        
-        return true;
-    }
-    
-    
     /// \brief
     /// Initializes the renderer for off-screen rendering used by the    
     /// testsuite.
@@ -592,6 +484,53 @@ public:
     
 #endif
 
+    class External 
+    {
+    public:
+        /// Prepare the renderer for external rendering
+        //
+        /// Note that all arguments except the background colour are useless
+        /// outside the ogl renderer.
+        External(Renderer& r, const rgba& c, int w = 0, int h = 0,
+                float x0 = 0, float x1 = 0, float y0 = 0, float y1 = 0)
+            :
+            _r(r)
+        {
+            _r.begin_display(c, w, h, x0, x1, y0, y1);
+        }
+
+        ~External() {
+            _r.end_display();
+        }
+
+    private:
+        Renderer& _r;
+    };
+    
+    class Internal 
+    {
+    public:
+        /// Prepare the renderer for internal rendering
+        Internal(Renderer& r, image::GnashImage& im)
+            :
+            _r(r),
+            _ext(_r.startInternalRender(im))
+        {
+        }
+
+        Renderer* renderer() const {
+            return _ext;
+        }
+
+        ~Internal() {
+            _r.endInternalRender();
+        }
+
+    private:
+        Renderer& _r;
+        Renderer* _ext;
+    };
+
 protected:
 
     /// Kept in parallel with movie_root's setting.
@@ -599,6 +538,37 @@ protected:
 
     // Delayed imaged to render
     RenderImages _render_images;
+
+private:
+
+    /// Bracket the displaying of a frame from a movie.
+    //
+    /// Set up to render a full frame from a movie and fills the
+    /// background. Sets up necessary transforms, to scale the
+    /// movie to fit within the given dimensions.    Call
+    /// end_display() when you're done.
+    //
+    /// Most of the arguments are only for the ogl renderer. See documentation
+    /// in that class. Do not use these arguments for new renderers!
+    virtual void begin_display(const rgba& background_color, 
+                    int viewport_width, int viewport_height,
+                    float x0, float x1, float y0, float y1) = 0;
+
+    virtual void end_display() = 0;
+
+    /// Setup the renderer to draw to an internal buffer.
+    //
+    /// Implementations are free to return a new renderer if they choose.
+    //
+    /// @return         0 if the renderer does not support this.
+    virtual Renderer* startInternalRender(image::GnashImage& buffer) = 0;
+
+    /// Finish internal rendering.
+    //
+    /// Any rendering after this function has been called must apply to the
+    /// external buffer.
+    virtual void endInternalRender() = 0;
+
 }; 
 
 } // namespace gnash
