@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -71,7 +72,7 @@ KlashFactory::~KlashFactory () {
 KParts::Part *KlashFactory::createPartObject
   (QWidget *wparent, const char *wname,
    QObject *parent, const char * name,
-   const char * cls, const QStringList & args) {
+   const char * /*cls*/, const QStringList & args) {
       //kdDebug() << "KlashFactory::createPartObject " << cls << endl;
       return new KlashPart (wparent, wname, parent, name, args);
 }
