@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software
 //   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
@@ -21,9 +21,16 @@
 #include "gnashconfig.h"
 #endif
 
+#include <boost/format.hpp>
+#include <boost/scoped_array.hpp>
+#include <boost/algorithm/string/replace.hpp>
+#include <boost/algorithm/string/find.hpp>
+#include <cassert>
+#include <string>
 #include <cstdlib> // getenv
 #include <stdlib.h> // putenv
 #include <sys/types.h>
+
 #if defined(HAVE_WINSOCK_H) && !defined(__OS2__)
 # include <winsock2.h>
 # include <windows.h>
@@ -34,7 +41,6 @@
 # include <sys/socket.h>
 #endif
 
-#include <boost/format.hpp>
 
 #define MIME_TYPES_HANDLED  "application/x-shockwave-flash"
 // The name must be this value to get flash movies that check the
@@ -51,7 +57,7 @@
 
 #define PLUGIN_DESCRIPTION \
   "Shockwave Flash "FLASH_VERSION"<br>Gnash "VERSION", the GNU SWF Player. \
-  Copyright (C) 2006, 2007, 2008, 2009, 2010 \
+  Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011 \
   <a href=\"http://www.fsf.org\">Free \
   Software Foundation</a>, Inc. <br> \
   Gnash comes with NO WARRANTY, to the extent permitted by law. \
@@ -68,13 +74,20 @@
 #include "StringPredicates.h"
 #include "external.h"
 #include "callbacks.h"
+#if NPAPI_VERSION == 190
+#include "npupp.h"
+#else
 #include "npapi.h"
 #include "npruntime.h"
 #include "npfunctions.h"
+#endif
 #include "GnashNPVariant.h"
 
 #include <boost/tokenizer.hpp>
 #include <boost/algorithm/string/join.hpp>
+#include <boost/algorithm/string/split.hpp>
+#include <boost/algorithm/string/classification.hpp>
+#include <boost/algorithm/string/trim.hpp>
 #include <boost/format.hpp>
 #include <sys/param.h>
 #include <csignal>
@@ -97,6 +110,13 @@
 #define PATH_MAX 1024
 #endif
 
+// Macro to prevent repeated logging calls for the same
+// event
+#define LOG_ONCE(x) { \
+    static bool warned = false; \
+    if (!warned) { warned = true; x; } \
+}
+
 // For scriptable plugin support
 #include "pluginScriptObject.h"
 
@@ -118,8 +138,6 @@ static bool waitforgdb = false;
 static bool createSaLauncher = false;
 
 static const char* getPluginDescription();
-// void GnashLogDebug(const std::string& msg);
-// void GnashLogError(const std::string& msg);
 
 static const char*
 getPluginDescription() 
@@ -325,10 +343,12 @@ NS_PluginGetValue(NPPVariable aVariable, void *aValue)
       case NPPVpluginScriptableNPObject:
           break;
 
+#if NPAPI_VERSION != 190
       case NPPVpluginUrlRequestsDisplayedBool:
           break;
       case NPPVpluginWantsAllNetworkStreams:
           break;
+#endif
           
       default:
           err = NPERR_INVALID_PARAM;
@@ -378,9 +398,6 @@ nsPluginInstance::nsPluginInstance(nsPluginCreateData* data)
     _width(0),
     _height(0),
     _streamfd(-1),
-    _ichan(0),
-    _ichanWatchId(0),
-    _controlfd(-1),
     _childpid(0),
     _filefd(-1),
     _name(),
@@ -407,12 +424,10 @@ nsPluginInstance::nsPluginInstance(nsPluginCreateData* data)
         _params[name] = val;
     }
 
-#if 1
     if (NPNFuncs.version >= 14) { // since NPAPI start to support
-        _scriptObject = (GnashPluginScriptObject *)NPNFuncs.createobject(
-            _instance, GnashPluginScriptObject::marshalGetNPClass());
+        _scriptObject = (GnashPluginScriptObject*) NPN_CreateObject( _instance,
+            GnashPluginScriptObject::marshalGetNPClass());
     }
-#endif
     
     return;
 }
@@ -427,7 +442,6 @@ cleanup_childpid(gpointer data)
 
     if (rv <= 0) {
         // The child process has not exited; it may be deadlocked. Kill it.
-//        gnash::log_error("BUG: Child process is stuck. Killing it.");
 
         kill(*pid, SIGKILL);
         waitpid(*pid, &status, 0);
@@ -445,14 +459,18 @@ nsPluginInstance::~nsPluginInstance()
 {
 //    gnash::log_debug("plugin instance destruction");
 
-    if ( _ichanWatchId ) {
-        g_source_remove(_ichanWatchId);
-        _ichanWatchId = 0;
+    if (_scriptObject) {
+        NPN_ReleaseObject(_scriptObject);
     }
 
+    do { } while (g_source_remove_by_user_data(this));
+
     if (_childpid > 0) {
-        // When the child has terminated (signaled by _controlfd), it remains
-        // as a defunct process and we remove it from the kernel table now.
+        // When the child has terminated (signaled by GTK through GtkSocket), it
+        // remains as a defunct process and we remove it from the kernel table now.
+        
+        // FIXME: we should ideally do this before the GtkSocket goes away, but
+        // after the delete signal has been sent.
         
         // If all goes well, Gnash will already have terminated.
         int status;
@@ -512,14 +530,6 @@ nsPluginInstance::shut()
         }
     }
 
-    if (_controlfd != -1) {
-        _scriptObject->closePipe(_controlfd);
-        if (close(_controlfd) != 0) {
-            gnash::log_error("Gnash plugin failed to close the control socket!");
-        }
-    }
-
-
 }
 /// \brief Set the window to be used to render in
 ///
@@ -569,7 +579,7 @@ nsPluginInstance::GetValue(NPPVariable aVariable, void *aValue)
     if (aVariable == NPPVpluginScriptableNPObject) {
         if (_scriptObject) {
             void **v = (void **)aValue;
-            NPNFuncs.retainobject(_scriptObject);
+            NPN_RetainObject(_scriptObject);
             *v = _scriptObject;
         } else {
             gnash::log_debug("_scriptObject is not assigned");
@@ -581,14 +591,6 @@ nsPluginInstance::GetValue(NPPVariable aVariable, void *aValue)
     return NS_PluginGetValue(aVariable, aValue);
 }
 
-#if 0
-// FIXME: debugging stuff, will be gone soon after I figure how this works
-void myfunc(void */* param */)
-{
-    gnash::log_debug("Here I am!!!\n");
-}
-#endif
-
 /// \brief Open a new data stream
 ///
 /// Opens a new incoming data stream, which is the flash movie we want
@@ -613,12 +615,6 @@ nsPluginInstance::NewStream(NPMIMEType /*type*/, NPStream* stream,
     }
     _swf_url = stream->url;
 
-#if 0
-    // FIXME: debugging crap for now call javascript
-    NPN_PluginThreadAsyncCall(_instance, myfunc, NULL);
-    // gnash::log_debug("FIXME: %s", getEmbedURL());
-#endif
-    
     if (!_swf_url.empty() && _window) {
         startProc();
     }
@@ -630,7 +626,6 @@ nsPluginInstance::NewStream(NPMIMEType /*type*/, NPStream* stream,
 NPError
 nsPluginInstance::DestroyStream(NPStream* /*stream*/, NPError /*reason*/)
 {
-
     if (_streamfd != -1) {
         if (close(_streamfd) == -1) {
             perror("closing _streamfd");
@@ -678,60 +673,44 @@ nsPluginInstance::handlePlayerRequests(GIOChannel* iochan, GIOCondition cond)
 
     if ( cond & G_IO_HUP ) {
         gnash::log_debug("Player control socket hang up");
-        // Returning false here will cause the "watch" to be removed. This watch
-        // is the only reference held to the GIOChannel, so it will be
-        // destroyed. We must make sure we don't attempt to destroy it again.
-        _ichanWatchId = 0;
         return false;
     }
 
     assert(cond & G_IO_IN);
 
-    gnash::log_debug("Checking player requests on fd #%d",
+    gnash::log_debug("Checking player requests on FD #%d",
               g_io_channel_unix_get_fd(iochan));
 
-    GError* error = 0;
-    //    g_io_channel_set_flags(iochan, G_IO_FLAG_NONBLOCK, &error);
+    const size_t buf_size = 1;
+    gchar buffer[buf_size];
 
-    size_t retries = 5;
-    gchar* request = 0;
-    gsize requestSize = 0;
     do {
-        // When in non-blocking mode, we'll get several iterations of this
-        // loop while waiting for data. if data never arrives, we'd be stuck
-        // looping here forever, so this is our escape from that loop.
-        if (retries-- <= 0) {
-            gnash::log_error("Too many attempts to read from the player!");
-            return false;
-        }
-        error = 0;
-        request = 0;
-        requestSize = 0;
-        GIOStatus status = g_io_channel_read_line(iochan, &request,
-                           &requestSize, NULL, &error);
+        GError* error = 0;
+        gsize bytes_read = 0;
+
+        GIOStatus status = g_io_channel_read_chars(iochan, buffer, buf_size,
+                                                   &bytes_read, &error);
+
         switch (status) {
           case G_IO_STATUS_ERROR:
               gnash::log_error("error reading request line: %s",
-                               error->message);
+                               error ? error->message : "unspecified error");
               g_error_free(error);
               return false;
           case G_IO_STATUS_EOF:
-              gnash::log_error("EOF (error: %s", error->message);
+              gnash::log_error("EOF (error: %s)", 
+                               error ? error->message : "unspecified error");
               g_error_free(error);
               return false;
           case G_IO_STATUS_AGAIN:
-              gnash::log_debug("read again: nonblocking mode set ");
+              gnash::log_debug("read again");
               continue;
           case G_IO_STATUS_NORMAL:
               // process request
-              // Get rid of the newline on the end if there is one. The string
-              // is also NULL terninated, so the requestSize includes it in
-              // the total.
-              if (request[requestSize-1] == '\n') {
-                  request[requestSize-1] = 0;
-                  requestSize--;
-              }
-              gnash::log_debug("Normal read: %s", request);
+              _requestbuf.append(buffer, buffer+bytes_read);
+#if 0
+              gnash::log_debug("Normal read: %s", std::string(buffer, buffer+bytes_read));
+#endif
               break;
           default:
               gnash::log_error("Abnormal status!");
@@ -740,60 +719,102 @@ nsPluginInstance::handlePlayerRequests(GIOChannel* iochan, GIOCondition cond)
     } while (g_io_channel_get_buffer_condition(iochan) & G_IO_IN);
 
     // process request..
-    processPlayerRequest(request, requestSize);
-    g_free(request);
+    processPlayerRequest();
     
     return true;
 }
 
-bool
-nsPluginInstance::processPlayerRequest(gchar* buf, gsize linelen)
+// This GIOFunc handler removes the source from the mainloop.
+gboolean 
+remove_handler(GIOChannel*, GIOCondition, gpointer)
 {
-    // gnash::log_debug(__PRETTY_FUNCTION__);
+    return FALSE;
+}
 
-   // log_debug("SCRIPT OBJECT %d: %x", __LINE__, this->getScriptObject());
 
-    if ( linelen < 4 ) {
-        if (buf) {
-            gnash::log_error("Invalid player request (too short): %s", buf);
-        } else {
-            gnash::log_error("Invalid player request (too short): %d bytes", linelen);
-        }
+// There may be multiple Invoke messages in a single packet, so each
+// packet needs to be broken up into separate messages to be parsed.
+bool
+nsPluginInstance::processPlayerRequest()
+{
+#if 0
+     gnash::log_debug(__PRETTY_FUNCTION__);
+
+     log_debug("SCRIPT OBJECT %d: %x", __LINE__, this->getScriptObject());
+#endif
+
+    if ( _requestbuf.size() < 4 ) {
+        gnash::log_error("Invalid player request (too short): %s", _requestbuf);
         return false;
     }
-    
-    ExternalInterface::invoke_t *invoke = ExternalInterface::parseInvoke(buf);
 
-    if (!invoke->name.empty()) {
-        gnash::log_debug("Requested method is: %s", invoke->name);
-    }
-    
-    // The invoke message is also used for getURL. In this case there are 4
-    // possible arguments.
-    if (invoke) {
+
+    std::string& packet = _requestbuf;
+    do {
+        boost::trim_left(packet);
+        if (packet.empty()) {
+            return false;
+        }
+
+        std::string term = "</invoke>";
+        std::string::size_type pos = packet.find(term);
+        // no terminator, bad message
+        if (pos == std::string::npos) {
+            gnash::log_debug("Incomplete Invoke message. Probably a fragment.");
+            return false;
+        }
+         
+        // Extract a message from the packet
+        std::string msg = packet.substr(0, pos + term.size());
+        boost::shared_ptr<plugin::ExternalInterface::invoke_t> invoke =
+            plugin::ExternalInterface::parseInvoke(msg);
+
+        // drop the parsed message from the packet
+        packet.erase(0, msg.size());
+        
+        if (!invoke) {
+            log_error("Failed to parse invoke message: %s", msg);
+            return false;
+        }
+
+        if (!invoke->name.empty()) {
+            gnash::log_debug("Requested method is: %s", invoke->name);
+        } else {
+            gnash::log_error("Invoke request missing a name to invoke.");
+            continue;
+        }
+        
         if (invoke->name == "getURL") {
-            // gnash::log_debug("Got a getURL() request: %", invoke->args[0].get());
-
+            
+            assert(invoke->args.size() > 1);
+            
             // The first argument is the URL string.
             std::string url = NPStringToString(NPVARIANT_TO_STRING(
                                                    invoke->args[0].get()));
+#if 0
+            gnash::log_debug("Got a getURL() request: %s", url);
+#endif
             // The second is the method, namely GET or POST.
             std::string op = NPStringToString(NPVARIANT_TO_STRING(
                                                   invoke->args[1].get()));
             // The third is the optional target, which is something like
-            // _blank or _self. NONE means no target.
+            // _blank or _self.
             std::string target;
+            
             // The fourth is the optional data. If there is data, the target
             // field is always set so this argument is on the correct index.
-            // No target is "NONE".
             std::string data;
+            
             if (invoke->args.size() >= 3) {
                 target = NPStringToString(NPVARIANT_TO_STRING(
                                               invoke->args[2].get()));
-                if (target == "NONE") {
-                    target.clear();
-                }
             }
+            
+            // An empty target defaults to "_self"
+            // This is _required_ for chromium,
+            // see https://savannah.gnu.org/bugs/?32425
+            if ( target.empty() ) target = "_self";
+            
             if (invoke->args.size() == 4) {
                 data = NPStringToString(NPVARIANT_TO_STRING(
                                             invoke->args[3].get()));
@@ -807,11 +828,14 @@ nsPluginInstance::processPlayerRequest(gchar* buf, gsize linelen)
                                  data);
                 NPN_PostURL(_instance, url.c_str(), target.c_str(), data.size(),
                             data.c_str(), false);
-                return true;
+            } else {
+                log_error("Unexpected op in getURL (expected POST or GET).");
             }
             
-            return true;
+            continue; 
         } else if (invoke->name == "fsCommand") {
+            
+            assert(invoke->args.size() > 1);
             std::string command = NPStringToString(NPVARIANT_TO_STRING(
                                                        invoke->args[0].get()));
             std::string arg = NPStringToString(NPVARIANT_TO_STRING(
@@ -828,8 +852,10 @@ nsPluginInstance::processPlayerRequest(gchar* buf, gsize linelen)
                              jsurl.str(), tgt);
             
             NPN_GetURL(_instance, jsurl.str().c_str(), tgt);
-            return true;
+            continue;
         } else if (invoke->name == "addMethod") {
+            
+            assert(!invoke->args.empty());
             // Make this flash function accessible to Javascript. The
             // actual callback lives in libcore/movie_root, but it
             // needs to be on the list of supported remote methods so
@@ -839,39 +865,49 @@ nsPluginInstance::processPlayerRequest(gchar* buf, gsize linelen)
             NPIdentifier id = NPN_GetStringIdentifier(method.c_str());
             // log_debug("SCRIPT OBJECT addMethod: %x, %s", (void *)_scriptObject, method);
             this->getScriptObject()->AddMethod(id, remoteCallback);
-            return true;
+            continue;
         }
-        NPVariant *result = 0;
+        
+        NPVariant result;
+        VOID_TO_NPVARIANT(result);
+        bool invokeResult = false;
+        
         // This is the player invoking a method in Javascript
-        uint32_t count = 0;
-        NPVariant args;
-        if (!invoke->name.empty()) {
-            NPIdentifier id = NPN_GetStringIdentifier(invoke->name.c_str());      
+        if (!invoke->name.empty() && !invoke->args.empty()) {
+            //Convert the as_value argument to NPVariant
+            const size_t count = invoke->args.size() - 1;
+            boost::scoped_array<NPVariant> args(new NPVariant[count]);
+            //Skip the first argument
+            for (size_t i = 0; i < count; ++i) {
+                invoke->args[i+1].copy(args[i]);
+            }
+            
+            NPIdentifier id = NPN_GetStringIdentifier(invoke->name.c_str());
             gnash::log_debug("Invoking JavaScript method %s", invoke->name);
-            NPN_Invoke(_instance, _scriptObject, id, &args, count, result);
+            NPObject* windowObject;
+            NPN_GetValue(_instance, NPNVWindowNPObject, &windowObject);
+            invokeResult=NPN_Invoke(_instance, windowObject, id, args.get(),
+                                    count, &result);
+            NPN_ReleaseObject(windowObject);
         }
         // We got a result from invoking the Javascript method
         std::stringstream ss;
-        if (result) {
-            NPN_ReleaseVariantValue(result);
-            ss << ExternalInterface::convertNPVariant(result);
+        if (invokeResult) {
+            ss << plugin::ExternalInterface::convertNPVariant(&result);
+            NPN_ReleaseVariantValue(&result);
         } else {
             // Send response
             // FIXME: "securityError" also possible, check domain
-            ss << ExternalInterface::makeString("Error");
+            ss << plugin::ExternalInterface::makeString("Error");
         }
         size_t ret = _scriptObject->writePlayer(ss.str());
         if (ret != ss.str().size()) {
             log_error("Couldn't write the response to Gnash, network problems.");
             return false;
         }
-        return true;
-    } else {
-        gnash::log_error("Unknown player request: " + std::string(buf));
-        return false;
-    }
-
-    return false;
+    } while (!packet.empty());
+    
+    return true;
 }
 
 std::string
@@ -906,7 +942,6 @@ getGnashExecutable()
         return "";
     }
 
-
     return procname;
 }
 
@@ -933,6 +968,8 @@ create_standalone_launcher(const std::string& page_url, const std::string& swf_u
     }
 
     saLauncher << "#!/bin/sh" << std::endl
+               << "export GNASH_COOKIES_IN="
+               << "/tmp/gnash-cookies." << getpid() << std::endl
                << getGnashExecutable() << " ";
 
     if (!page_url.empty()) {
@@ -943,7 +980,11 @@ create_standalone_launcher(const std::string& page_url, const std::string& swf_u
         itEnd = params.end(); it != itEnd; ++it) {
         const std::string& nam = it->first; 
         const std::string& val = it->second;
-        saLauncher << "-P '" << nam << "=" << val << "' ";
+        saLauncher << "-P '"
+                   << boost::algorithm::replace_all_copy(nam, "'", "'\\''")
+                   << "="
+                   << boost::algorithm::replace_all_copy(val, "'", "'\\''")
+                   << "' ";
     }
 
     saLauncher << "'" << swf_url << "' "
@@ -954,6 +995,184 @@ create_standalone_launcher(const std::string& page_url, const std::string& swf_u
 #endif
 }
 
+std::string
+nsPluginInstance::getDocumentProp(const std::string& propname) const
+{
+    std::string rv;
+
+    NPObject* windowobj;
+    NPError err = NPN_GetValue(_instance, NPNVWindowNPObject, &windowobj);
+    if (err != NPERR_NO_ERROR || !windowobj) {
+        return rv;
+    }
+
+    boost::shared_ptr<NPObject> window_obj(windowobj, NPN_ReleaseObject);
+  
+    NPIdentifier doc_id = NPN_GetStringIdentifier("document");
+
+    NPVariant docvar;
+    if(! NPN_GetProperty(_instance, windowobj, doc_id, &docvar) ) {
+        return rv;
+    }
+
+    boost::shared_ptr<NPVariant> doc_var(&docvar, NPN_ReleaseVariantValue);
+
+    if (!NPVARIANT_IS_OBJECT(docvar)) {
+        return rv;
+    }
+
+    NPObject* doc_obj = NPVARIANT_TO_OBJECT(docvar);
+    
+    NPIdentifier prop_id = NPN_GetStringIdentifier(propname.c_str());
+
+    NPVariant propvar;
+    if (!NPN_GetProperty(_instance, doc_obj, prop_id, &propvar)) {
+        return rv;
+    }
+
+    boost::shared_ptr<NPVariant> prop_var(&propvar, NPN_ReleaseVariantValue);
+
+    if (!NPVARIANT_IS_STRING(propvar)) {
+        return rv;
+    }
+
+    const NPString& prop_str = NPVARIANT_TO_STRING(propvar);
+
+    rv = NPStringToString(prop_str);
+    return rv;
+}
+
+
+
+void
+nsPluginInstance::setupCookies(const std::string& pageurl)
+{
+    // In pre xulrunner 1.9, (Firefox 3.1) this function does not exist,
+    // so we can't use it to read the cookie file. For older browsers
+    // like IceWeasel on Debian lenny, which pre dates the cookie support
+    // in NPAPI, you have to block all Cookie for sites like YouTube to
+    // allow Gnash to work.
+#if NPAPI_VERSION != 190
+    if (!NPNFuncs.getvalueforurl) {
+        LOG_ONCE( gnash::log_debug("Browser doesn't support reading cookies") );
+        return;
+    }
+#else
+    LOG_ONCE( log_debug("Browser doesn't support reading cookies via NPAPI.") );
+#endif
+
+    // Cookie appear to drop anything past the domain, so we strip
+    // that off.
+    std::string::size_type pos;
+    pos = pageurl.find("/", pageurl.find("//", 0) + 2) + 1;
+    std::string url = pageurl.substr(0, pos);
+
+    std::string ncookie;
+ 
+    char *cookie = 0;
+    uint32_t length = 0;
+
+#if NPAPI_VERSION != 190
+    NPError rv = NPN_GetValueForURL(_instance, NPNURLVCookie, url.c_str(),
+                       &cookie, &length);
+#else
+    NPError rv = NPERR_GENERIC_ERROR;
+#endif
+
+    // Firefox does not (always) return the cookies that are associated
+    // with a domain name through GetValueForURL.
+    if (rv == NPERR_GENERIC_ERROR) {
+        log_debug("Trying window.document.cookie for cookies");
+        ncookie = getDocumentProp("cookie");
+    }
+
+    if (cookie) {
+        ncookie.assign(cookie, length);
+        NPN_MemFree(cookie);
+    }
+
+    if (ncookie.empty()) {
+        gnash::log_debug("No stored Cookie for %s", url);
+        return;
+    }
+
+    gnash::log_debug("The Cookie for %s is %s", url, ncookie);
+    std::ofstream cookiefile;
+    std::stringstream ss;
+    ss << "/tmp/gnash-cookies." << getpid();
+
+    cookiefile.open(ss.str().c_str(), std::ios::out | std::ios::trunc);
+
+    // Firefox provides cookies in the following format:
+    //
+    // cookie1=value1;cookie2=value2;cookie3=value3
+    //
+    // Whereas libcurl expects cookies in the following format:
+    //
+    // Set-Cookie: cookie1=value1;
+    // Set-Cookie: cookie2=value2;
+  
+    typedef boost::char_separator<char> char_sep;
+    typedef boost::tokenizer<char_sep> tokenizer;
+    tokenizer tok(ncookie, char_sep(";"));
+
+    for (tokenizer::iterator it=tok.begin(); it != tok.end(); ++it) {
+        cookiefile << "Set-Cookie: " << *it << std::endl;
+    }
+ 
+    cookiefile.close();
+  
+    if (setenv("GNASH_COOKIES_IN", ss.str().c_str(), 1) < 0) {
+        gnash::log_error(
+            "Couldn't set environment variable GNASH_COOKIES_IN to %s",
+            ncookie);
+    }
+}
+
+void
+nsPluginInstance::setupProxy(const std::string& url)
+{
+    // In pre xulrunner 1.9, (Firefox 3.1) this function does not exist,
+    // so we can't use it to read the proxy information.
+#if NPAPI_VERSION != 190
+    if (!NPNFuncs.getvalueforurl) return;
+#endif
+
+    char *proxy = 0;
+    uint32_t length = 0;
+#if NPAPI_VERSION != 190
+    NPN_GetValueForURL(_instance, NPNURLVProxy, url.c_str(),
+                       &proxy, &length);
+#endif
+    if (!proxy) {
+        gnash::log_debug("No proxy setting for %s", url);
+        return;
+    }
+
+    std::string nproxy (proxy, length);
+    NPN_MemFree(proxy);
+
+    gnash::log_debug("Proxy setting for %s is %s", url, nproxy);
+
+    std::vector<std::string> parts;
+    boost::split(parts, nproxy,
+        boost::is_any_of(" "), boost::token_compress_on);
+    if ( parts[0] == "DIRECT" ) {
+        // no proxy
+    }
+    else if ( parts[0] == "PROXY" ) {
+        if (setenv("http_proxy", parts[1].c_str(), 1) < 0) {
+            gnash::log_error(
+                "Couldn't set environment variable http_proxy to %s",
+                nproxy);
+        }
+    }
+    else {
+        gnash::log_error("Unknown proxy type: %s", nproxy);
+    }
+
+}
+
 std::vector<std::string>
 nsPluginInstance::getCmdLine(int hostfd, int controlfd)
 {
@@ -977,43 +1196,8 @@ nsPluginInstance::getCmdLine(int hostfd, int controlfd)
         arg_vec.push_back(pageurl);
     }
 
-    // In pre xulrunner 1.9, (Firefox 3.1) this function does not exist,
-    // so we can't use it to read the cookie file. For older browsers
-    // like IceWeasel on Debian lenny, which pre dates the cookie support
-    // in NPAPI, you have to block all Cookie for sites like YouTube to
-    // allow Gnash to work.
-    if (NPNFuncs.getvalueforurl) {
-        // Cookie appear to drop anything past the domain, so we strip
-        // that off.
-        std::string::size_type pos;
-        pos = pageurl.find("/", pageurl.find("//", 0) + 2) + 1;
-        std::string url = pageurl.substr(0, pos);
-        
-        char *cookie = 0;
-        uint32_t length = 0;
-        NPN_GetValueForURL(_instance, NPNURLVCookie, url.c_str(),
-                           &cookie, &length);
-        std::string ncookie (cookie, length);
-        if (cookie) {
-            gnash::log_debug("The Cookie for %s is %s", url, ncookie);
-            std::ofstream cookiefile;
-            std::stringstream ss;
-            ss << "/tmp/gnash-cookies." << getpid(); 
-            
-            cookiefile.open(ss.str().c_str(), std::ios::out | std::ios::trunc);
-            cookiefile << "Set-Cookie: " << ncookie << std::endl;
-            cookiefile.close();
-            
-            if (setenv("GNASH_COOKIES_IN", ss.str().c_str(), 1) < 0) {
-                gnash::log_error(
-                    "Couldn't set environment variable GNASH_COOKIES_IN to %s",
-                    ncookie);
-            }
-            NPN_MemFree(cookie);
-        } else {
-            gnash::log_debug("No stored Cookie for %s", url);
-        }    
-    }
+    setupCookies(pageurl);
+    setupProxy(pageurl);
 
     std::stringstream pars;
     pars << "-x "  <<  _window          // X window ID to render into
@@ -1091,6 +1275,19 @@ wait_for_gdb()
     }
 }
 
+void
+nsPluginInstance::setupIOChannel(int fd, GIOFunc handler, GIOCondition signals) const
+{
+    GIOChannel* ichan = g_io_channel_unix_new(fd);
+    g_io_channel_set_close_on_unref(ichan, true);
+
+    gnash::log_debug("New IO Channel for fd #%d",
+                     g_io_channel_unix_get_fd(ichan));
+    g_io_add_watch(ichan, signals, handler, (gpointer)this); 
+    g_io_channel_unref(ichan);
+}
+
+
 void
 nsPluginInstance::startProc()
 {
@@ -1101,20 +1298,20 @@ nsPluginInstance::startProc()
 
     int ret = socketpair(AF_UNIX, SOCK_STREAM, 0, p2c_pipe);
     if (ret == -1) {
-        gnash::log_error("ERROR: socketpair(p2c) failed: %s", strerror(errno));
+        gnash::log_error("socketpair(p2c) failed: %s", strerror(errno));
         return;
     }
     _streamfd = p2c_pipe[1];
 
     ret = socketpair(AF_UNIX, SOCK_STREAM, 0, c2p_pipe);
     if (ret == -1) {
-        gnash::log_error("ERROR: socketpair(c2p) failed: %s", strerror(errno));
+        gnash::log_error("socketpair(c2p) failed: %s", strerror(errno));
         return;
     }
 
     ret = socketpair(AF_UNIX, SOCK_STREAM, 0, p2c_controlpipe);
     if (ret == -1) {
-        gnash::log_error("ERROR: socketpair(control) failed: %s", strerror(errno));
+        gnash::log_error("socketpair(control) failed: %s", strerror(errno));
         return;
     }
 
@@ -1143,7 +1340,7 @@ nsPluginInstance::startProc()
     
     // If the fork failed, childpid is -1. So print out an error message.
     if (_childpid == -1) {
-        gnash::log_error("ERROR: dup2() failed: " + std::string(strerror(errno)));
+        gnash::log_error("dup2() failed: %s", strerror(errno));
         return;
     }
     
@@ -1154,7 +1351,7 @@ nsPluginInstance::startProc()
         ret = close (p2c_pipe[0]);
         if (ret == -1) {
             // this is not really a fatal error, so continue best as we can
-            gnash::log_error("ERROR: p2c_pipe[0] close() failed: %s",
+            gnash::log_error("p2c_pipe[0] close() failed: %s",
                              strerror(errno));
         }
         
@@ -1162,22 +1359,19 @@ nsPluginInstance::startProc()
         ret = close (c2p_pipe[1]);
         if (ret == -1) {
             // this is not really a fatal error, so continue best as we can
-            gnash::log_error("ERROR: c2p_pipe[1] close() failed: %s",
+            gnash::log_error("c2p_pipe[1] close() failed: %s",
                              strerror(errno));
             gnash::log_debug("Forked successfully but with ignorable errors.");
         } else {
             gnash::log_debug("Forked successfully, child process PID is %d",
                              _childpid);
         }
+
+        setupIOChannel(c2p_pipe[0], (GIOFunc)handlePlayerRequestsWrapper,
+                                    (GIOCondition)(G_IO_IN|G_IO_HUP));
         
-        GIOChannel* ichan = g_io_channel_unix_new(c2p_pipe[0]);
-        gnash::log_debug("New IO Channel for fd #%d",
-                         g_io_channel_unix_get_fd(ichan));
-        g_io_channel_set_close_on_unref(ichan, true);
-        _ichanWatchId = g_io_add_watch(ichan, 
-                                       (GIOCondition)(G_IO_IN|G_IO_HUP), 
-                                       (GIOFunc)handlePlayerRequestsWrapper,
-                                       this);
+        setupIOChannel(p2c_controlpipe[1], remove_handler, G_IO_HUP);
+
         return;
     }
     
@@ -1192,7 +1386,7 @@ nsPluginInstance::startProc()
     ret = dup2 (p2c_pipe[0], fileno(stdin));
 
     if (ret == -1) {
-        gnash::log_error("ERROR: dup2() failed: " + std::string(strerror(errno)));
+        gnash::log_error("dup2() failed: %s", strerror(errno));
     }
     
     // Close all of the browser's file descriptors that we just inherited
@@ -1215,52 +1409,20 @@ nsPluginInstance::startProc()
     exit (-1);
 }
 
+
+
+
+
 std::string
 nsPluginInstance::getCurrentPageURL() const
 {
-    NPP npp = _instance;
+    // Return:
+    //  window.document.baseURI
+    //
+    // Was (bogus):
+    //  window.document.location.href
 
-    NPIdentifier sDocument = NPN_GetStringIdentifier("document");
-
-    NPObject *window;
-    NPN_GetValue(npp, NPNVWindowNPObject, &window);
-
-    NPVariant vDoc;
-    NPN_GetProperty(npp, window, sDocument, &vDoc);
-    NPN_ReleaseObject(window);
-
-    if (!NPVARIANT_IS_OBJECT(vDoc)) {
-        gnash::log_error("Can't get window object");
-        return NULL;
-    }
-    
-    NPObject* npDoc = NPVARIANT_TO_OBJECT(vDoc);
-
-    NPIdentifier sLocation = NPN_GetStringIdentifier("location");
-    NPVariant vLoc;
-    NPN_GetProperty(npp, npDoc, sLocation, &vLoc);
-    NPN_ReleaseObject(npDoc);
-
-    if (!NPVARIANT_IS_OBJECT(vLoc)) {
-        gnash::log_error("Can't get window.location object");
-        return NULL;
-    }
-
-    NPObject* npLoc = NPVARIANT_TO_OBJECT(vLoc);
-
-    NPIdentifier sProperty = NPN_GetStringIdentifier("href");
-    NPVariant vProp;
-    NPN_GetProperty(npp, npLoc, sProperty, &vProp);
-    NPN_ReleaseObject(npLoc);
-
-    if (!NPVARIANT_IS_STRING(vProp)) {
-        gnash::log_error("Can't get window.location.href object");
-        return NULL;
-    }
-
-    const NPString& propValue = NPVARIANT_TO_STRING(vProp);
-
-    return NPStringToString(propValue);
+    return getDocumentProp("baseURI");
 }
 
 void
