@@ -27,11 +27,13 @@
 #endif
 
 #include "noseek_fd_adapter.h"
+#include "IOChannel.h"
 #include "tu_file.h"
 
 #include <cstdio>
 #include <iostream>
 #include <cassert>
+#include <memory>
 
 #include "GnashSystemIOHeaders.h"
 #include <sys/types.h>
@@ -39,6 +41,7 @@
 #include <fcntl.h>
 #include <string.h>
 #include <sstream>
+#include <limits>
 
 using namespace std;
 
@@ -147,9 +150,23 @@ main(int /*argc*/, char** /*argv*/)
 
 
     FILE* f = std::fopen(cachename, "r");
-	gnash::tu_file orig(f, false);
+    std::auto_ptr<gnash::IOChannel> orig = gnash::makeFileChannel(f, false);
 	lseek(raw, 0, SEEK_SET);
-	compare_reads(&orig, raw, "cache", "raw");
+	compare_reads(orig.get(), raw, "cache", "raw");
+
+
+    if (sizeof(size_t) != sizeof(std::streamoff)) {
+        std::streampos pos = std::numeric_limits<size_t>::max();
+        pos += orig->size() / 2;
+        // Check that seek() handles integer overflow situations gracefully.
+        if (orig->seek(pos)) {
+            runtest.fail("Successfully sought to an invalid position.");
+        } else {
+            runtest.pass("Gracefully handled invalid seek.");
+        }
+    }
+
+
 
 	return 0;
 }
