@@ -1,5 +1,6 @@
 /* 
- *   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+ *   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+ *   2011 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -47,7 +48,7 @@ main(int /*argc*/, char** /*argv*/)
 	MovieClip* root = tester.getRootMovie();
 	assert(root);
 
-	check_equals(root->get_frame_count(), 3);
+	check_equals(root->get_frame_count(), 4);
 	check_equals(root->getPlayState(), MovieClip::PLAYSTATE_PLAY);
 	check_equals(root->get_current_frame(), 0);
 
@@ -75,14 +76,50 @@ main(int /*argc*/, char** /*argv*/)
 	check_pixel(cOmr.x, cOmr.y, 4, green, 2); // O right side
 	check_pixel(cOum.x, cOum.y, 20, white, 2); // O underline (none)
 
-	for (int i=0; i<2; ++i) tester.advance(); // get to the end
+    tester.advance();
+
+    // Move to left part of O
+    tester.movePointerTo(cOml.x, cOml.y);
+
+    // Click there
+    tester.click();
+
+    // Move to centre of O
+    tester.movePointerTo(cOmm.x, cOmm.y);
+
+    // Click there a lot (shouldn't be registered)
+    tester.click();
+    tester.click();
+    tester.click();
+    tester.click();
+    tester.click();
+    tester.click();
+    tester.click();
+    tester.click();
+
+    // Move to right part of O
+    tester.movePointerTo(cOmr.x, cOmr.y);
+
+    // Click there.
+    tester.click();
+    tester.movePointerTo(1, 1);
+
+    // Just for fun. These shouldn't be registered.
+    tester.click();
+    tester.click();
+    tester.click();
+    tester.click();
+    tester.click();
+
+	for (int i=0; i<3; ++i) tester.advance(); // get to the end
+
+	VM& vm = tester.vm();
 
-	string_table& st = VM::get().getStringTable();
 	as_value eot;
-	bool endOfTestFound = getObject(root)->get_member(st.find("endoftest"), &eot);
+	bool endOfTestFound = getObject(root)->get_member(getURI(vm, "endoftest"), &eot);
 	check(endOfTestFound);
 	check(eot.is_bool());
-	check(eot.to_bool());
+	check(eot.to_bool(8));
 
 	// TODO: use check_pixel for checking bacground colors
 }
