@@ -1,5 +1,5 @@
 /* 
- *   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
+ *   Copyright (C) 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -53,15 +53,13 @@ main(int /*argc*/, char** /*argv*/)
 	assert(root);
 
 	// for variables lookup (consistency checking)
-	string_table& st = getStringTable(*root);
+	VM& vm = getVM(*getObject(root));
 
 	rgba white(255, 255, 255, 255); // background color
 	rgba blue(0, 0, 255, 255);      // blue circles fill color
 	rgba green(0, 255, 0, 255);     // green circles fill color
 	rgba red(255, 0, 0, 255);       // red circles fill color
 
-	int cs = 20;             // circles size
-
 	point out(350, 100);  // out of any drawing
 
 	point rc1(50, 50);    // first red circle
@@ -97,7 +95,7 @@ main(int /*argc*/, char** /*argv*/)
 		const MovieClip* loadedTarget = 0;
 		//const DisplayObject* ch = tester.findDisplayItemByName(*root, "loadedTarget");
 		const DisplayObject* ch = tester.findDisplayItemByDepth(*root, 30);
-		if ( ch ) loadedTarget = ch->to_movie();
+		if ( ch ) loadedTarget = const_cast<DisplayObject*>(ch)->to_movie();
 		if ( loadedTarget )
 		{
 			const DisplayObject* target100 = tester.findDisplayItemByName(*loadedTarget, "target100");
@@ -113,7 +111,7 @@ main(int /*argc*/, char** /*argv*/)
 		{
 			check(!"loadTarget was never loaded");
 			cerr << "Root display list is: " << endl;
-			root->getDisplayList().dump();
+            std::cerr << root->getDisplayList();
 			exit(EXIT_FAILURE);
 		}
 
@@ -170,12 +168,12 @@ main(int /*argc*/, char** /*argv*/)
 	// Consistency check !!
 	as_value eot;
 	// It's an swf6, so lowercase 'ENDOFTEST'
-	bool endOfTestFound = root->get_member(st.find("endoftest"), &eot);
+	bool endOfTestFound = getObject(root)->get_member(getURI(vm, "endoftest"), &eot);
 	check(endOfTestFound);
 	if ( endOfTestFound )
 	{
-		cerr << eot.toDebugString() << endl;
-		check_equals(eot.to_bool(), true);
+		cerr << eot << endl;
+		check_equals(eot.to_bool(8), true);
 	}
 	else
 	{
