@@ -1,5 +1,5 @@
 /* 
- *   Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+ *   Copyright (C) 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -51,7 +51,7 @@ main(int /*argc*/, char** /*argv*/)
 	assert(root);
 
 	// for variables lookup (consistency checking)
-	string_table& st = getStringTable(*getObject(root));
+	VM& vm = getVM(*getObject(root));
 
 	check_equals(root->get_frame_count(), 2);
 	check_equals(root->get_current_frame(), 0);
@@ -72,13 +72,13 @@ main(int /*argc*/, char** /*argv*/)
 	// Consistency check !!
 	as_value eot;
 	// It's an swf6, so lowercase 'ENDOFTEST'
-	bool endOfTestFound = getObject(root)->get_member(st.find("endoftest"), &eot);
+	bool endOfTestFound = getObject(root)->get_member(getURI(vm, "endoftest"), &eot);
 	check(endOfTestFound);
 
 	if ( endOfTestFound )
 	{
-		cerr << eot.toDebugString() << endl;
-		check_equals(eot.to_bool(), true);
+		cerr << eot << endl;
+		check_equals(eot.to_bool(8), true);
 	}
 	else
 	{
