@@ -1,5 +1,5 @@
 /* 
- *   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
+ *   Copyright (C) 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
 
 #include "MovieTester.h"
 #include "MovieClip.h"
-#include "character.h"
+#include "DisplayObject.h"
 #include "DisplayList.h"
 #include "log.h"
 #include "GnashException.h"
